// Analyser.cpp: implementation of the Analyser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "math.h"
#include "ModelAnalyser.h"
#include "Analyser.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

SampleHolder::SampleHolder(int NCell, int NLevel, void* &Point):CurrentTime(*((float*)Point))
{
	Pat = (short*)((float*)Point+1);
	Pot	= (PotTh*)(Pat + NLevel);
	Point = (void*)(Pot + NCell);
}

CumulatedComplex::CumulatedComplex()
{
	AverageTime = AverageVrs = DiameterVrs = 0.0f;
	if(!PitSet.empty())		PitSet.clear();
	if(!Binding.empty())	Binding.clear();
}

CumulatedComplex::~CumulatedComplex()
{
	if(!PitSet.empty())		PitSet.clear();
	if(!Binding.empty())	Binding.clear();
}

void CumulatedComplex::Save(HANDLE hFile)
{
	if(hFile == NULL) return;
	DWORD dR=0, Control=0;	SPit Pit;
	AverageTime = AverageVrs = DiameterVrs = 0.0f;
	for(vector<SPit>::iterator ps= PitSet.begin(); ps!=PitSet.end(); ps++){
		AverageTime += (*ps).Time;
		AverageVrs  += (*ps).Vrs;
		if(DiameterVrs < fabs((*ps).Vrs))DiameterVrs = fabs((*ps).Vrs);
	}
	int Length = 4*3+4+sizeof(Pit)*PitSet.size()+4+4*Binding.size();
	::WriteFile(hFile,&Length,4,&dR,NULL);				Control+=dR;
	::WriteFile(hFile,&AverageTime,4,&dR,NULL);			Control+=dR;
	::WriteFile(hFile,&AverageVrs,4,&dR,NULL);			Control+=dR;
	::WriteFile(hFile,&DiameterVrs,4,&dR,NULL);			Control+=dR;
	
	int Pitsizeof = PitSet.size();
	int Binsizeof = Binding.size();
	::WriteFile(hFile,&Pitsizeof,4,&dR,NULL);				Control+=dR;
	::WriteFile(hFile,&Binsizeof,4,&dR,NULL);				Control+=dR;
	for(int p=0; p<Pitsizeof; p++){
		Pit = PitSet[p];
		::WriteFile(hFile,&Pit,sizeof(Pit),&dR,NULL);	Control+=dR;
	}
	
	for(p=0; p<Binsizeof; p++){
		int Temp = Binding[p];
		::WriteFile(hFile,&Temp,4,&dR,NULL);			Control+=dR;
	}
	if((int)Control != Length)
		AfxMessageBox( "Error in Length of Record" );
}

void CumulatedComplex::Load(HANDLE hFile)
{
	if(hFile == NULL) return;
	if(!PitSet.empty())		PitSet.clear();
	if(!Binding.empty())	Binding.clear();

	DWORD dR;
	int Length =0;
	::ReadFile(hFile,&Length,4,&dR,NULL);
	::ReadFile(hFile,&AverageTime,4,&dR,NULL);
	::ReadFile(hFile,&AverageVrs,4,&dR,NULL);
	::ReadFile(hFile,&DiameterVrs,4,&dR,NULL);
	int Pitsizeof = 0;
	int Binsizeof = 0;
	::ReadFile(hFile,&Pitsizeof,4,&dR,NULL);
	::ReadFile(hFile,&Binsizeof,4,&dR,NULL);
	for(int p=0; p<Pitsizeof; p++){
		SPit Pit;
		::ReadFile(hFile,&Pit,sizeof(Pit),&dR,NULL);
		PitSet.push_back(Pit);
	}
	for(p=0; p<Binsizeof; p++){
		int Temp =0;
		::ReadFile(hFile,&Temp,4,&dR,NULL);
		Binding.push_back(Temp);
	}
}

ComplexHolder::ComplexHolder(void* &MPoint)
	:Length(*((int*)MPoint)),
	 AverageTime(*((float*)MPoint+1)),
	 AverageVrs(*((float*)MPoint+2)),
	 DiameterVrs(*((float*)MPoint+3)),
	 NPit(*((int*)MPoint+4)),
	 NBinding(*((int*)MPoint+5))
{
	PitSet = (SPit*)((char*)MPoint+24);
	Binding = (int*)((char*)PitSet+NPit*sizeof(*PitSet));
	PointerHolder = MPoint;
	MPoint = (void*)((int*)Binding+NBinding);
	if(((char*)MPoint-(char*)PointerHolder) != Length)
		AfxMessageBox( "Error in Length of Record" );
	BitList = NULL;LengthBitList=0;
}

ComplexHolder::~ComplexHolder()
{
	if(BitList != NULL) delete[] BitList;
	LengthBitList = 0;
}

bool* ComplexHolder::GetBitList(int NCell)
{
	if(BitList == NULL){
		BitList = new bool[NCell];
		for(int c=0; c<NCell; c++)BitList[c]=false;
		for(int cl=0; cl<NPit; cl++){
			if((PitSet[cl].N<NCell)&(PitSet[cl].N>0)){
				BitList[PitSet[cl].N]=true;
			}
		}
		LengthBitList=NCell;
	}
	else if(NCell != LengthBitList){
		delete[] BitList;
		BitList = new bool[NCell];
		for(int c=0; c<NCell; c++)BitList[c]=false;
		for(int cl=0; cl<NPit; cl++){
			if((PitSet[cl].N<NCell)&(PitSet[cl].N>0)){
				BitList[PitSet[cl].N]=true;
			}
		}
		LengthBitList=NCell;
	}
	return BitList;
}

bool ComplexHolder::FreeBitList(bool* &Bl)
{
	if(BitList == NULL) return false;
	if(Bl != BitList) return false;
	Bl = NULL;
	delete[] BitList;
	BitList = NULL; LengthBitList=0;
	return true;
}

//////////////////////////////////////////////////////////////////////

DynamicCluster::DynamicCluster(int NC, vector <ComplexHolder> &CL): ComplexList(CL)
{
	NCell = NC;
	if(!AttachComplexList.empty()) AttachComplexList.clear();
	AverageTime = MaxTimeDistance = Dispersion = 0.0;
	DMeasure= true;
}

DynamicCluster::~DynamicCluster()
{
	if(!AttachComplexList.empty()) AttachComplexList.clear();
}

int DynamicCluster::AddComplex(int NComplex)
{
	if(NComplex>=ComplexList.size()) return-1;
	AttachComplexList.push_back(NComplex);
	DMeasure = false;
	return AttachComplexList.size();
}

bool DynamicCluster::DeleteComplex(int NComplex)
{
	if(NComplex>=ComplexList.size()) return false;
	bool Ready = false;
	vector <int>::iterator pAc = AttachComplexList.begin();
	while((pAc <= AttachComplexList.end())|(!Ready)){
		if(Ready = ((*pAc) == NComplex)){
			pAc = AttachComplexList.erase(pAc);
			DMeasure = false;
			return Ready;
		}
		else
			pAc++;
	}
	return false;
}

void DynamicCluster::ReCulculate()
{
	AverageTime = MaxTimeDistance = Dispersion = 0.0;
	if(AttachComplexList.empty()) return;
	for(vector <int>::iterator pAc = AttachComplexList.begin(); pAc != AttachComplexList.end(); pAc++)
		AverageTime += ComplexList[*pAc].AverageTime;
	AverageTime /= (double)AttachComplexList.size();

	for(pAc = AttachComplexList.begin(); pAc != AttachComplexList.end(); pAc++){
		double Dis = (AverageTime - ComplexList[*pAc].AverageTime)
			*(AverageTime - ComplexList[*pAc].AverageTime);
		Dispersion += Dis;
		if(MaxTimeDistance < Dis)MaxTimeDistance = Dis;
	}
	Dispersion /= (double)(AttachComplexList.size()-1);
	DMeasure = true;
}

double DynamicCluster::GetDistance(double ComplexTime)
{
	if(DMeasure) ReCulculate();
	return((ComplexTime-AverageTime)*(ComplexTime-AverageTime));
}

double DynamicCluster::GetDistance(int NComplex)
{
	if(ComplexList.size()>=NComplex) return -1;
	return GetDistance((double)ComplexList[NComplex].AverageTime);
}

double DynamicCluster::GetMinimalDistance(double ComplexTime)
{
	if(AttachComplexList.empty()) return -1.0;
	double ShortTime = ComplexList[AttachComplexList[0]].AverageTime - ComplexTime;
	ShortTime = ShortTime*ShortTime;
	for(vector <int>::iterator pAc = AttachComplexList.begin(); pAc != AttachComplexList.end(); pAc++){
		double DTime = (ComplexList[*pAc].AverageTime-ComplexTime)*(ComplexList[*pAc].AverageTime-ComplexTime);
		if(ShortTime > DTime) ShortTime = DTime;
	}
	return ShortTime;
}

double DynamicCluster::GetMinimalDistance(int NComplex)
{
	if(ComplexList.size()>=NComplex) return -1;
	return GetMinimalDistance((double)ComplexList[NComplex].AverageTime);
}

double DynamicCluster::GetDiameter()
{
	if(DMeasure) ReCulculate();
	return MaxTimeDistance;
}

double DynamicCluster::GetAverageDiameter()
{
	if(DMeasure) ReCulculate();
	return AverageTime;
}

double DynamicCluster::GetDispersion()
{
	if(DMeasure) ReCulculate();
	return Dispersion;
}

void DynamicCluster::Compress(double MaxDiameter)
{
	if(DMeasure) ReCulculate();
	vector <int>::iterator pAc = AttachComplexList.begin();
	while(pAc <= AttachComplexList.end()){
		pAc =(ComplexList[*pAc].AverageTime>MaxDiameter) ?
			pAc = AttachComplexList.erase(pAc):	pAc+1;
	}
	ReCulculate();
}

void DynamicCluster::Compress(int Divisor)
{
	if(DMeasure) ReCulculate();
	Compress((double)(AverageTime / (double)Divisor));
}

//////////////////////////////////////////////////////////////////////

SpationCluster::SpationCluster(int NC, vector <ComplexHolder> &CL):ComplexList(CL)
{
	NCell = NC;
	CenterGravity = new float[NC];
	VectorCumulator = new int [NC];
	for(int n=0; n<NC; n++){
		CenterGravity[n] = 0.0f;
		VectorCumulator[n] = 0;
	}
	if(!AttachComplex.empty())AttachComplex.clear();
	Diameter = 0.0;
	DMeasure = true;
	LeechComplexList = NULL;
}

SpationCluster::~SpationCluster()
{
	delete[] CenterGravity;
	delete[] VectorCumulator;
	if(!AttachComplex.empty())AttachComplex.clear();
	if(LeechComplexList != NULL) delete[] LeechComplexList;
}

void SpationCluster::Save(HANDLE hFile)
{
	if(!DMeasure) ReCulculate();
	DWORD dR;
	::WriteFile(hFile,&NCell,4,&dR,NULL);
	int NCom = AttachComplex.size();
	::WriteFile(hFile,&NCom,4,&dR,NULL);
	for(int com=0; com<NCom; com++)
		::WriteFile(hFile,&AttachComplex[com],12,&dR,NULL);

}

void SpationCluster::Load(HANDLE hFile)
{
}

void SpationCluster::SaveDynamicInformation(HANDLE hFile, bool Detail)
{
	if(!DMeasure) ReCulculate();
	DWORD dR;
	int NCom = AttachComplex.size();
	::WriteFile(hFile,&NCom,4,&dR,NULL);
	float fTemp;
	fTemp = (float) Diameter;
	::WriteFile(hFile,&fTemp,4,&dR,NULL);
	fTemp = (float) AverageDiameter;
	::WriteFile(hFile,&fTemp,4,&dR,NULL);
	fTemp = (float) Dispersion;
	::WriteFile(hFile,&fTemp,4,&dR,NULL);
	if(!Detail)return;
	for(int com=0; com<NCom; com++){
		fTemp = (float) AttachComplex[com].Distance;
		::WriteFile(hFile,&fTemp,4,&dR,NULL);
	}
}

void SpationCluster::AddComplex(int NCom)
{
	if(NCom>=ComplexList.size()) return;
	AttachComplex.push_back(SAttachComplex (NCom,0.0));
	bool *Vec = ComplexList[NCom].GetBitList(NCell);
	if(Vec == NULL) return;

	for(int n=0; n<NCell; n++){
		if(Vec[n]) VectorCumulator[n]++;
	}
	ComplexList[NCom].FreeBitList(Vec);
	ReFresh();
	Extract(NCom);
}

void SpationCluster::DeleteComplex(int NComplex)
{
	if(NComplex>=ComplexList.size()) return;
	bool Ready = false;
	vector <SAttachComplex>::iterator pAc = AttachComplex.begin();
	while((pAc <= AttachComplex.end())|(!Ready)){
		if(Ready = ((*pAc).NComplex == NComplex)){
			bool *Vec = ComplexList[(*pAc).NComplex].GetBitList(NCell);
			for(int n=0; n<NCell; n++){
				if(Vec[n]) VectorCumulator[n] --;
			}
			ComplexList[(*pAc).NComplex].FreeBitList(Vec);
			pAc = AttachComplex.erase(pAc);
		}
		else
			pAc++;
	}
	ReFresh();
	ReExtractFlg = false;
}

double SpationCluster::GetDistance(bool *Bl)
{
	if(Bl == NULL) return -1;
	double Distance = 0.0;
	for(int n=0; n<NCell; n++){
		double Dis = (Bl[n])?((double)CenterGravity[n] - 1.0):((double)CenterGravity[n]);
		Distance += Dis*Dis;
	}
	return sqrt(Distance);
}

double	SpationCluster::GetDiameter()
{
	if(!DMeasure) ReCulculate();
	return Diameter;
}

double	SpationCluster::GetAverageDiameter()
{
	if(!DMeasure) ReCulculate();
	return AverageDiameter;
}

double SpationCluster::GetDispersion()
{
	if(!DMeasure) ReCulculate();
	return Dispersion;
}

int *SpationCluster::GetLeechComplexList(int &NComplex)
{
	NComplex = AttachComplex.size();
	if(LeechComplexList == NULL){
		LeechComplexList =new int[NComplex];
		for(int com=0; com<NComplex; com++)
			LeechComplexList[com] = AttachComplex[com].NComplex;
	}
	return LeechComplexList;
}

bool SpationCluster::FreeLeechComplexList(int* &LCL)
{
	if(LeechComplexList == NULL) return false;
	if(LCL != LeechComplexList) return false;
	delete [] LeechComplexList;
	LeechComplexList = LCL = NULL;
	return true;
}


void SpationCluster::Compress(double MaxDiameter)
{
	if(!DMeasure) ReCulculate();
	vector <SAttachComplex>::iterator pAc = AttachComplex.begin();
	while(pAc <= AttachComplex.end()){
		pAc =((*pAc).Distance>MaxDiameter)?
			AttachComplex.erase(pAc):pAc+1;
	}
	ReAttach();
	ReFresh();
	ReExtractFlg = false;
}												

void SpationCluster::Compress(int Divisor)
{
	if(!DMeasure) ReCulculate();
	Compress((double)(AverageDiameter/(double)Divisor));
}

void SpationCluster::ReCulculate()
{
	Diameter = AverageDiameter = 0.0;
	double Dis;
	for(int com=0; com<AttachComplex.size(); com++){
		bool *Vec = ComplexList[AttachComplex[com].NComplex].GetBitList(NCell);
		if(Diameter < (Dis=GetDistance(Vec))) Diameter = Dis;
		ComplexList[AttachComplex[com].NComplex].FreeBitList(Vec);
		AverageDiameter +=Dis;
	}
	AverageDiameter /= (double)AttachComplex.size();

	Dispersion = 0.0;
	for(com=0; com<AttachComplex.size(); com++){
		bool *Vec = ComplexList[AttachComplex[com].NComplex].GetBitList(NCell);
		AttachComplex[com].Distance = Dis = GetDistance(Vec);
		ComplexList[AttachComplex[com].NComplex].FreeBitList(Vec);
		Dispersion +=(Dis - AverageDiameter)*(Dis - AverageDiameter);
	}
	Dispersion /= (double)(AttachComplex.size()-1);
	DMeasure = true;
}

void SpationCluster::ReFresh()
{
	delete[] CenterGravity; CenterGravity = new float[NCell];
	double Div = (double) AttachComplex.size();
	for(int n=0; n<NCell; n++)
		CenterGravity[n] =(float)((double) VectorCumulator[n]/Div);
	DMeasure = false;
}

void SpationCluster::ReAttach()
{
	if(VectorCumulator != NULL) delete[] VectorCumulator;
	VectorCumulator = new int[NCell];
	for(int com=0; com<AttachComplex.size(); com++){
		bool *Vec = ComplexList[AttachComplex[com].NComplex].GetBitList(NCell);
		for(int n=0; n<NCell; n++){
			if(Vec[n]) VectorCumulator[n]++;
		}
		ComplexList[AttachComplex[com].NComplex].FreeBitList(Vec);
	}
}

void SpationCluster::Extract(int NComplex)
{
	if(!ReExtractFlg)ReExtract();
	if(DynamicClusterList.empty()){
		DynamicClusterList.push_back(DynamicCluster (NCell, ComplexList));
		DynamicClusterList[0].AddComplex(NComplex);
		return;
	}
	vector<DynamicCluster>::iterator dDp;
	bool Ready = false;
	for(dDp = DynamicClusterList.begin(); dDp < DynamicClusterList.end(); dDp++){
		Ready = ((*dDp).GetMinimalDistance(NComplex) <= MaxTimeDelay);
		if(Ready){
			(*dDp).AddComplex(NComplex);
			break;
		}
	}
	if(!Ready){
		DynamicClusterList.push_back(DynamicCluster (NCell, ComplexList));
		dDp = DynamicClusterList.end();
		(*dDp).AddComplex(NComplex);
	}
}

int SpationCluster::ReExtract()
{
	DynamicClusterList.clear();
	vector <SAttachComplex>::iterator com;
	for(com=AttachComplex.begin(); com<=AttachComplex.end(); com++)
		Extract((*com).NComplex);
	ReExtractFlg = true;
	return DynamicClusterList.size();
}
//////////////////////////////////////////////////////////////////////

Analyser::Analyser()
{
	AnalysisEpoch = 0.016;
	AveragingStep = 0.005;
	AveragingWindow = 0.01;
	MaxDeltaVrs = 0.1;
	NPits =  0;
	VrsMap = NULL;
	NCells = NLevel = 0;
	pSim = pCom = NULL;
	CurrentRec = 0;
	CellAverage = NULL;
}

Analyser::~Analyser()
{
	if(VrsMap != NULL){
		for(int pit=0; pit<NCells; pit++)
			delete[] VrsMap[pit];
		delete VrsMap;
	}
	if(CellAverage != NULL) delete[] CellAverage;
}

void Analyser::LeechToModFile()
{
	if(hSim == INVALID_HANDLE_VALUE)  return;
	if(pSim != NULL)::UnmapViewOfFile(pSim);

	DWORD HFileSize;
	sizeSim = ::GetFileSize(hSim,&HFileSize);
	HANDLE hMapRecord = ::CreateFileMapping(hSim,NULL,PAGE_READONLY,0,sizeSim,NULL);
	if(hMapRecord == INVALID_HANDLE_VALUE) return;
	pSim = ::MapViewOfFile(hMapRecord,FILE_MAP_READ,0,0,0);
	::CloseHandle(hMapRecord);
	if(pSim == NULL) return;

	NPits = (int)floor(AnalysisEpoch/AveragingStep);
	NRecPerWindow = (int)floor(AveragingWindow/TimeStep);
	NRecPerWindow &= 0xFFFFFFFF-1;
	NRecPerStep = (int)floor(AveragingStep/TimeStep);
	NRec = NRecPerStep*NPits+NRecPerWindow;

	bool NewFile = (CurrentRec == 0);
	if(NewFile) CurrentRec = NRecPerWindow/2;
	if((NRec+CurrentRec) > RecCount) return;
	void *Deep =(void*)((char*)pSim+PointBegin+((CurrentRec-NRecPerWindow/2)*RecLength));
	
	if(!SampleList.empty())SampleList.clear();
	for(int p=0; p<NRec; p++)
		SampleList.push_back(SampleHolder (NCells, NLevel,Deep));

	if(NewFile){
		CurrentVrs = 0;
		if(VrsMap != NULL){
			for(int pit=0; pit<NCells; pit++)delete[] VrsMap[pit];
			delete VrsMap;
		}
		VrsMap = new SVrs*[NCells];
		for(p=0; p<NCells; p++)VrsMap[p] = new SVrs[NPits];

		for(p=0; p<NCells; p++){
			for(int s=0; s<NPits; s++){
				VrsMap[p][s].NComplex = 0;
				VrsMap[p][s].Average = VrsMap[p][s].Vrs = 0.0;
			}
		}
	}

}

void Analyser::Averaging()
{
	if(CellAverage != NULL) delete[] CellAverage;
	CellAverage = new float[NCells];
	for(int p=0; p<NCells; p++){
		CellAverage[p] = 0.0;
		for(int s=0; s<NPits; s++)
				VrsMap[p][s].Average = 0.0;
	}
	int *PitLeft = new int[NPits];
	int *PitRight = new int[NPits];
	for(int s=0; s<NPits; s++){
		PitLeft[s] = (CurrentRec + s * NRecPerStep)-NRecPerWindow/2;
		PitRight[s] = (CurrentRec + s * NRecPerStep)+NRecPerWindow/2;
	}
	
	for(int RecCnt=(CurrentRec-NRecPerWindow/2), pit=CurrentVrs; 
				RecCnt<(NRec+CurrentRec-NRecPerWindow/2); 
				RecCnt++){
		for(int c=0; c<NCells; c++){
			CellAverage[c] += SampleList[RecCnt].Pot[c].Pot;
			for(s=0; s<NPits; s++)
				VrsMap[c][s].Average += ((PitLeft[s]<RecCnt)&(RecCnt<PitRight[s]))? 
					SampleList[RecCnt].Pot[c].Pot : 0;

		}
	}
	for(p=0; p<NCells; p++){
		CellAverage [p] /= (float)NRec;
		for(s=0; s<NPits; s++){
			VrsMap[p][s].Average = (VrsMap[p][s].Average - CellAverage[p])/(float)NRecPerStep;
		}
	}

	delete[] PitLeft;
	delete[] PitRight;
}

void Analyser::Complexing()
{
}