// Analyser.h: interface for the Analyser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ANALYSER_H__B65DA038_01FD_4AA6_90B3_2B564C5FA6AA__INCLUDED_)
#define AFX_ANALYSER_H__B65DA038_01FD_4AA6_90B3_2B564C5FA6AA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <vector>
//#include <list>
using namespace std;

struct PotTh{
	float Pot, Th;
};

class SampleHolder{
public:
	float &CurrentTime;
	short *Pat;
	PotTh *Pot;
	SampleHolder(int NCell, int NLevel, void* &Point);
	void operator = (SampleHolder sm) {
		Pat = sm.Pat;
		Pot = sm.Pot;
	}
};

struct SPit{			
	int N;				//Number of cell
	float Vrs;			//Reduction Scaled Speed
	float Time;			//Time
};

struct SVrs{
	int NComplex;
	float Vrs, Average;
};

struct SAttachComplex{
	int NComplex;
	double Distance;
	SAttachComplex(){this->NComplex=0; this->Distance=0.0;};
	SAttachComplex(int Nc, double D){this->NComplex=Nc; this->Distance=D;};
};

class CumulatedComplex{
public:
	float AverageTime;
	float AverageVrs;
	float DiameterVrs;
	vector <SPit> PitSet;
	vector <int>  Binding;
	CumulatedComplex();
	~CumulatedComplex();
	void Save(HANDLE hFile);
	void Load(HANDLE hFile);
};

class ComplexHolder{
public:
	int &Length;
	float &AverageTime;
	float &AverageVrs;
	float &DiameterVrs;
	int &NPit;
	SPit *PitSet;
	int &NBinding;
	int *Binding;
	void *PointerHolder;
protected:
	bool *BitList;
	int  LengthBitList;
public:
	ComplexHolder(void* &MasPoint);
	~ComplexHolder();
	bool* GetBitList(int NCell);
	bool  FreeBitList(bool* &Bl);
};

class DynamicCluster{
public:
	DynamicCluster(int NC, vector <ComplexHolder> &CL);
	~DynamicCluster();

	void	Load(HANDLE hFile);
	void	Save(HANDLE hFile);
	
	int		AddComplex(int NComplex);
	bool	DeleteComplex(int NComplex);
	void	ReCulculate();

	virtual double	GetDistance(double ComplexTime);
	virtual double	GetDistance(int NComplex);
	virtual double	GetMinimalDistance(double ComplexTime);
	virtual double	GetMinimalDistance(int NComplex);
	double	GetDiameter();
	double	GetAverageDiameter();
	double	GetDispersion();

	virtual void	Compress(double MaxDiameter);
	virtual void	Compress(int Divisor);

	void operator = (DynamicCluster pos) {
		DMeasure = pos.DMeasure;
		AverageTime = pos.AverageTime;
		MaxTimeDistance = pos.MaxTimeDistance;
		Dispersion = pos.Dispersion;
		AttachComplexList = pos.AttachComplexList;
	};

//===================
	vector <int> AttachComplexList;
	bool	DMeasure;
	int		NCell;
	double	AverageTime;
	double	MaxTimeDistance;
	double	Dispersion;
	vector <ComplexHolder> &ComplexList;
};

class SpationCluster{							
public:
	SpationCluster(int NC, vector <ComplexHolder> &CL);
	~SpationCluster();

	void	Load(HANDLE hFile);
	void	Save(HANDLE hFile);
	void	SaveDynamicInformation(HANDLE hFile, bool Detail);
	void	SaveDynamicCluster(HANDLE hFile, int NDCluster);
	void	SaveAllDynamicClusters(HANDLE hFile);

	void	AddComplex(int NComplex);
	void	DeleteComplex(int NComplex);

	double	GetDistance(bool *Bl);
	double	GetDiameter();
	double	GetAverageDiameter();
	double	GetDispersion();

	int		*GetLeechComplexList(int &NComplex);
	bool	FreeLeechComplexList(int* &LCL);

	virtual void	Compress(double MaxDiameter);
	virtual void	Compress(int Divisor);

protected:
	vector <SAttachComplex> AttachComplex;
	bool	DMeasure;
	double	Diameter;
	double	AverageDiameter;
	double	Dispersion;
	int		NCell;
	float	*CenterGravity;
	int		*VectorCumulator;
	vector <ComplexHolder> &ComplexList;
	int		*LeechComplexList;
//=======================================================
public:
	vector <DynamicCluster> DynamicClusterList;
	bool	ReExtractFlg;
	double	MaxTimeDelay;
	int		ReExtract();
//=======================================================
protected:
	void	ReCulculate();
	void	ReFresh();
	void	ReAttach();
	void	Extract(int NComplex);
};										


class Analyser  
{
public:
//	Common
	unsigned int NCells;
	unsigned int NLevel;

//	Step 0 ===> Data Window
	HANDLE	hSim;
	void	*pSim;
	void	*WindowLeft;
	int		sizeSim;
	vector <SampleHolder> SampleList;
	double	TimeStep;
	unsigned int RecLength;
	unsigned int RecCount;
	unsigned int PointBegin;

//	Step 1 ===> Average/Vrs Map	
	double	AnalysisEpoch;
	double	AveragingStep;
	double	AveragingWindow;
	int		NPits;
	unsigned int CurrentRec;
	unsigned int NRec;
	unsigned int NRecPerStep;
	unsigned int NRecPerWindow;

	SVrs	**VrsMap;
	int		CurrentVrs;
	float	*CellAverage;

//	Step 2 ===> Complex List
	HANDLE	hCom;
	void	*pCom;
	int		sizeCom;
	double	MaxDeltaVrs;
	vector <ComplexHolder> ComplexList;
	vector <CumulatedComplex> CurrentComplex;

//	Step 3 ===> Clasters
	double MaxDistance;

	
//================================================================
	Analyser();
	virtual ~Analyser();

	void LeechToModFile();
	void Averaging();
	void Complexing();
};

#endif // !defined(AFX_ANALYSER_H__B65DA038_01FD_4AA6_90B3_2B564C5FA6AA__INCLUDED_)
