/*==========================================================================================*/
/*																							*/
/*							   Integrate-and-Fire Neuron with Synapse						*/
/*                                  by Ruben Tikidji - Hamburyan                            */
/*																							*/
/*==========================================================================================*/

#include "stdafx.h"
#include "NeuroCAD.h"
#include "Cell.h"
#include "math.h"


Cell *Cell::First;	
Cell *Cell::Last;
double Cell::TimeStep;	
double Cell::CurrentTime;

Cell::Cell()
{
	APPotential = 0;
	Axon =  false;
	CurrentTime = TimeStep = 2e-4;
	Fireing = 0;
	Potentioal = 0;
	CountSyn = 0;MaxSyn = 100;
	FireThresholdTime = 2;
	APSingPotential = -5e-3;
	APR = 1e8; APC = 1.875e-6;
	RC = 20e-3;		C = 2e-11;
	ThresholdLevel = Threshold = 10e-3;	ThresholdMTime = ThresholdPTime = -1;
	PotentialStat = 0; I=0;
	Syn = new Synapse[100];
	Syn[0].Delay = 0;//1e-3;
	Syn[0].RisingTime = 2e-3;	Syn[0].FallingTime = 2e-3;
	Syn[0].LearnP = Syn[0].LearnM = 0;//1.7e-1; 
	Syn[0].MaxWeight = 150e-12;	Syn[0].MinWeight = -10e-12; Syn[0].SingleWeight =2e-10;
	Syn[0].dTime = 0;
	Syn[0].I = 0;

	Fine = false;
}

Cell::~Cell()
{
	delete[] Syn;
}

void Cell::Calculate()
{
	for(Cell *P=First; P<=Last; P++){
		P->I=0;
//		double SynK =  -1 + exp(-P->APPotential * P->APPotential /
//										(P->APSingPotential * P->APSingPotential));
//		if(P->Fireing < RefTime)	SynK = 1+SynK;

//		double SynK =  (P->Potentioal - P->APPotential) / fabs(P->APSingPotential);
//		double SynK =  -1 + (fabs(P->Potentioal) - fabs(P->APPotential)) / fabs(P->APSingPotential);
		P->SynK =  (fabs(P->Potentioal) - fabs(P->APPotential)) / fabs(P->APSingPotential);
//		double SynK =  (P->Potentioal - P->APPotential) / fabs(P->APSingPotential);


		for(int i=0; i<P->CountSyn; i++){
			P->Syn[i].dTime += P->TimeStep;

			P->Syn[i].I-=P->Syn[i].I*TimeStep/P->Syn[i].FallingTime;
			if(P->Syn[i].dTime<2*P->Syn[i].RisingTime)
				P->Syn[i].I +=P->Syn[i].SingleWeight*TimeStep/P->Syn[i].RisingTime;
			
			
			if(P->Syn[i].dDelay.size() !=0){
				double LastTime = P->Syn[i].dDelay.front();
				if((LastTime+P->Syn[i].Delay - CurrentTime)<=0){
					P->Syn[i].dTime=0;
					P->Syn[i].dDelay.pop();
				}
			}
			P->I+=P->Syn[i].I;
			

			if(((P->Syn[i].LearnP < -1e-15)|(P->Syn[i].LearnP >1e-15))&
			   ((P->Syn[i].LearnM < -1e-15)|(P->Syn[i].LearnM >1e-15))){
				if(P->SynK<0)
					P->Syn[i].SingleWeight+=P->Syn[i].I*P->Syn[i].LearnM*P->SynK*
						fabs((P->Syn[i].SingleWeight-P->Syn[i].MinWeight)/
																	P->Syn[i].MinWeight);
				else
					P->Syn[i].SingleWeight+=P->Syn[i].I*P->Syn[i].LearnP*P->SynK*
						fabs((P->Syn[i].MaxWeight - P->Syn[i].SingleWeight)/
																	P->Syn[i].MaxWeight);

			}
		}

		double dUap = (P->Potentioal - P->APPotential);
		P->Potentioal  += (P->I + P->APPotential/P->APR - 
						P->Potentioal * (1/P->APR + P->C/P->RC)) * TimeStep/P->C;
		
		double IntTime = 1.0;

		if(P->Fireing <= FireTime/2){
			P->Axon = 1; 
			P->Potentioal += 0.2 * TimeStep / FireTime;
			P->APPotential += P->APSingPotential  * TimeStep / FireTime;
			IntTime = P->FireThresholdTime;
			Fine = false;
		}
		else if((P->Fireing > FireTime/2)&(P->Fireing <= FireTime)){
			P->Axon = 1; 
			P->Potentioal -= 0.2 * TimeStep / FireTime;
			P->APPotential += P->APSingPotential  * TimeStep / FireTime;
			IntTime = P->FireThresholdTime;
			Fine = false;
		}
		else if(P->Fireing <= RefTime){
			P->Axon = 0;
			P->APPotential += dUap*TimeStep/(P->APR*P->APC);
			Fine = false;
		}
		else if((P->ThresholdLevel>(P->Threshold+MaxU))|(P->ThresholdLevel<(P->Threshold-MinU))){
//			P->Fireing = CurrentTime;
			P->Axon = 0;
			P->APPotential += dUap*TimeStep/(P->APR*P->APC);
			Fine = true;
		}
		else if(P->Potentioal>P->ThresholdLevel){
			P->Axon = 1;
			P->Fireing = 0;
			Fine = false;
		}
		else{
			P->Axon = 0;
			P->APPotential += dUap*TimeStep/(P->APR*P->APC);
			Fine = false;
		}

		if((P->ThresholdMTime > 0) & (P->ThresholdPTime > 0)){
			if((P->Potentioal - P->PotentialStat) < 0)
				P->PotentialStat += (P->Potentioal - P->PotentialStat) * IntTime
														* TimeStep / P->ThresholdMTime;
			else
				P->PotentialStat += (P->Potentioal-P->PotentialStat) * IntTime
														* TimeStep / P->ThresholdPTime;
			P->ThresholdLevel = P->Threshold+P->PotentialStat;
		}
		else P->ThresholdLevel = P->Threshold;

		P->Fireing += TimeStep;
	}
	for(P=First; P<=Last; P++){
		for(int i=0; i<P->CountSyn; i++){
			if(((Cell*)P->Syn[i].An)->Fireing < 1.1*TimeStep) 
				P->Syn[i].dDelay.push(CurrentTime);
		}
	}
	CurrentTime += TimeStep; 
}

void Cell::Assemble()
{
}

int Cell::CreateSynapse()
{
	if(CountSyn+1 == MaxSyn){
		Synapse *New = new Synapse[MaxSyn+100];
		for(int i=0; i<CountSyn; i++) New[i] = Syn[i];
		delete[] Syn;
		Syn = New; MaxSyn+=100;
	}
	Syn[CountSyn].I = 0;
	return (CountSyn++);
}
void Cell::DeleteSynapse(int N)
{
	if(N >= CountSyn)return;
	CountSyn--;
	for(int i=N; i<CountSyn; i++) Syn[i] =Syn[i+1];
}

void Cell::operator = (Cell &N)
{
	Axon = N.Axon; 
	Threshold =N.Threshold; 
	ThresholdMTime = N.ThresholdMTime;ThresholdPTime = N.ThresholdPTime;
	ThresholdLevel= N.ThresholdLevel;
	FireThresholdTime = N.FireThresholdTime;
	Potentioal = N.Potentioal; Fireing = N.Fireing;I=N.I;
	RC = N.RC; C = N.C;
	APR = N.APR; APC = N.APC;APSingPotential = N.APSingPotential;
	APPotential = N.APPotential;
	delete[] Syn; Syn = new Synapse[N.MaxSyn];
	CountSyn = N.CountSyn;
	MaxSyn = N.MaxSyn;
	for(int s=0; s<MaxSyn; s++) Syn[s] = N.Syn[s];
}

