/*==========================================================================================*/
/*																							*/
/*							  Input Machine for Intagrate - and -Fire Model					*/
/*                                  by Ruben Tikidji - Hamburyan                            */
/*																							*/
/*==========================================================================================*/

#include "stdafx.h"
#include "stdlib.h"
#include "NeuroCAD.h"
#include "InputM.h"
#include "Model.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

InputM::InputM()
{
	InputCount = 0; InputList = NULL;
	LevelCount = 0; Levels = NULL;
}

InputM::~InputM()
{
	if(InputCount !=0) delete[] InputList;
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			if(Levels[L].PaternCount != 0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					if(Levels[L].Paterns[P].EventCount !=0)
						delete[] Levels[L].Paterns[P].Events;
				}
				delete[] Levels[L].Paterns;
			}
		}
		delete[] Levels;
	}

}

int InputM::GetNumberInput(int NCell)
{
	for(int I=0; I<InputCount; I++){
		if(InputList[I] == NCell)
			return I;
	}
	return -1;
}

int InputM::AddInput(int NCell)
{
	int *New = new int[InputCount+1];
	for(int I=0; I<InputCount; I++) New[I] = InputList[I];
	delete[] InputList; InputList = New;
	InputList[InputCount] = NCell;
	return(InputCount++);
}
void InputM::DeleteInput(int NInput)
{
	if((InputCount == 0)|(NInput>=InputCount)) return;
	if(InputCount == 1){
		delete[] InputList;
		InputCount =0;
		if(LevelCount !=0){
			for(int L=0; L<LevelCount; L++){
				if(Levels[L].PaternCount != 0){
					for(int P=0; P<Levels[L].PaternCount; P++){
						if(Levels[L].Paterns[P].EventCount !=0)
							delete[] Levels[L].Paterns[P].Events;
					}
					delete[] Levels[L].Paterns;
				}
			}
			delete[] Levels;
		}
		return;
	}

	InputCount -- ;
	for(int I=NInput; I<InputCount; I++)	
		InputList[I] = InputList[I+1];
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			if(Levels[L].PaternCount != 0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					if(Levels[L].Paterns[P].EventCount !=0){
						for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
							if(Levels[L].Paterns[P].Events[E].Input == NInput)
								DeleteEvent(L,P,E);
						}
					}
				}
			}
		}
	}

}

void InputM::SaveInputMachine(CFile &File)
{
	File.Write(&InputCount,4);
	if(InputCount !=0){
		for(int I=0; I<InputCount; I++)
			File.Write(&InputList[I],4);
	}
	File.Write(&LevelCount,4);
	for(int L=0; L<LevelCount; L++){
		File.Write(&Levels[L].PaternCount,4);
		for(int P=0; P<Levels[L].PaternCount; P++){
			File.Write(&Levels[L].Paterns[P].Probability,8);
			File.Write(&Levels[L].Paterns[P].EventCount,4);
			for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
				File.Write(&Levels[L].Paterns[P].Events[E].Input,4);
				File.Write(&Levels[L].Paterns[P].Events[E].Time,8);
			}
		}
	}
}

void InputM::LoadInputMachine(CFile &File)
{
	if(InputCount !=0) delete[] InputList;
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			if(Levels[L].PaternCount != 0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					if(Levels[L].Paterns[P].EventCount !=0)
						delete[] Levels[L].Paterns[P].Events;
				}
				delete[] Levels[L].Paterns;
			}
		}
		delete[] Levels;
	}

	File.Read(&InputCount,4);
	if(InputCount !=0){
		InputList = new int[InputCount];
		for(int I=0; I<InputCount; I++)
			File.Read(&InputList[I],4);
	}
	File.Read(&LevelCount,4);
	if(LevelCount !=0){
		Levels = new Level[LevelCount];
		for(int L=0; L<LevelCount; L++){
			File.Read(&Levels[L].PaternCount,4);
			if(Levels[L].PaternCount !=0){
				Levels[L].Paterns = new Patern[Levels[L].PaternCount];
				for(int P=0; P<Levels[L].PaternCount; P++){
					File.Read(&Levels[L].Paterns[P].Probability,8);
					File.Read(&Levels[L].Paterns[P].EventCount,4);
					if(Levels[L].Paterns[P].EventCount !=0){
						Levels[L].Paterns[P].Events=new Event[Levels[L].Paterns[P].EventCount];
						for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
							File.Read(&Levels[L].Paterns[P].Events[E].Input,4);
							File.Read(&Levels[L].Paterns[P].Events[E].Time,8);
						}
					}
				}
			}
		}
	}
}

void InputM::ImportMachine(CFile &File)
{
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			if(Levels[L].PaternCount != 0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					if(Levels[L].Paterns[P].EventCount !=0)
						delete[] Levels[L].Paterns[P].Events;
				}
				delete[] Levels[L].Paterns;
			}
		}
		delete[] Levels;
	}
	File.Read(&LevelCount,4);
	if(LevelCount !=0){
		Levels = new Level[LevelCount];
		for(int L=0; L<LevelCount; L++){
			File.Read(&Levels[L].PaternCount,4);
			if(Levels[L].PaternCount !=0){
				Levels[L].Paterns = new Patern[Levels[L].PaternCount];
				for(int P=0; P<Levels[L].PaternCount; P++){
					File.Read(&Levels[L].Paterns[P].Probability,8);
					File.Read(&Levels[L].Paterns[P].EventCount,4);
					if(Levels[L].Paterns[P].EventCount !=0){
						Levels[L].Paterns[P].Events=new Event[Levels[L].Paterns[P].EventCount];
						for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
							File.Read(&Levels[L].Paterns[P].Events[E].Input,4);
							File.Read(&Levels[L].Paterns[P].Events[E].Time,8);
							if(Levels[L].Paterns[P].Events[E].Input>=InputCount)
								DeleteEvent(L,P,E--);
						}
					}
				}
			}
		}
	}

}

void InputM::ExportMachine(CFile &File)
{
	File.Write(&LevelCount,4);
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			File.Write(&Levels[L].PaternCount,4);
			if(Levels[L].PaternCount !=0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					File.Write(&Levels[L].Paterns[P].Probability,8);
					File.Write(&Levels[L].Paterns[P].EventCount,4);
					if(Levels[L].Paterns[P].EventCount !=0){
						for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
							File.Write(&Levels[L].Paterns[P].Events[E].Input,4);
							File.Write(&Levels[L].Paterns[P].Events[E].Time,8);
						}
					}
				}
			}
		}
	}

}

void InputM::Interpret(char * String, int NChar)
{
	if(LevelCount !=0){
		for(int L=0; L<LevelCount; L++){
			if(Levels[L].PaternCount != 0){
				for(int P=0; P<Levels[L].PaternCount; P++){
					if(Levels[L].Paterns[P].EventCount !=0)
						delete[] Levels[L].Paterns[P].Events;
				}
				delete[] Levels[L].Paterns;
			}
		}
		delete[] Levels;
		LevelCount = 0;
	}
	Levels = NULL;
	int p=0;
	int NLev=0, NPat=0, NEve=0, Nim=0; float FloatPar = 0;
	while (p < NChar){
//		while((String[p] == ' ')&(p<NChar)&(p == '\t')) p++;
//		if(p == NChar) return;
		switch (String[p]){
		case 'L':
		case 'l':	NLev = AddLevel(); p++; break;
		case 'P':
		case 'p':	NPat = AddPatern(NLev); p++;
					while((String[p] == ' ')&(p<NChar)) p++;
					if(p == NChar) return;
					if(sscanf(&String[p],"%f",&FloatPar) == 0)	break;
					while((isdigit((int)String[p]) !=0)|(String[p] == '-')|
						  (String[p] == '.')|(String[p] == 'e')|(String[p] == 'E'))p++;
					Levels[NLev].Paterns[NPat].Probability = (double)FloatPar;
					break;
		case 'E':
		case 'e':	NEve = AddEvent(NLev, NPat); p++;
					while((String[p] == ' ')&(p<NChar)) p++;
					if(p == NChar) return;
					if(sscanf(&String[p],"%d",&Nim) == 0)	break;
					while((isdigit((int)String[p]) !=0))p++;
					Levels[NLev].Paterns[NPat].Events[NEve].Input = Nim;

					while((String[p] == ' ')&(p<NChar)) p++;
					if(p == NChar) return;
					if(sscanf(&String[p],"%f",&FloatPar) == 0)	break;
					while((isdigit((int)String[p]) !=0)|(String[p] == '-')|
						  (String[p] == '.')|(String[p] == 'e')|(String[p] == 'E'))p++;
					Levels[NLev].Paterns[NPat].Events[NEve].Time = (double)FloatPar;
					break;
		case ' ':
		case '\t':
		case (char) 0x0a:
		case (char) 0x0d:	p++; 
							if(p == NChar) return;
							break;
		default:
					p++;
					while((String[p-2] != (char)0x0d)&(String[p-1] != (char)0x0a)
						&(p<NChar)) p++;
					break;
		}
	}
}
int InputM::AddLevel()
{
	Level *NewLevel = new Level[LevelCount+1];
	for(int L=0; L<LevelCount; L++)
		NewLevel[L] = Levels[L];
	if(LevelCount !=0) delete[] Levels;
	Levels = NewLevel;
	Levels[LevelCount].PaternCount = 0;
	return(LevelCount++);
}

void InputM::DeleteLevel(int NLevel)
{
	if((LevelCount == 0)|(NLevel>=LevelCount))return;
	if(Levels[NLevel].PaternCount != 0){
		for(int P=0; P<Levels[NLevel].PaternCount; P++){
			if(Levels[NLevel].Paterns[P].EventCount !=0)
				delete[] Levels[NLevel].Paterns[P].Events;
		}
		delete[] Levels[NLevel].Paterns;
	}
	LevelCount--;
	for(int L=NLevel; L<LevelCount; L++)
		Levels[L]=Levels[L+1];
}

int InputM::AddPatern(int NLevel)
{
	if((NLevel<0)|(NLevel>=LevelCount)) return -1;
	Patern *NewPatern = new Patern [Levels[NLevel].PaternCount+1];
	for(int P=0; P<Levels[NLevel].PaternCount; P++)
		NewPatern[P] = Levels[NLevel].Paterns[P];
	if(Levels[NLevel].PaternCount!=0) delete[] Levels[NLevel].Paterns;
	Levels[NLevel].Paterns = NewPatern;
	Levels[NLevel].Paterns[Levels[NLevel].PaternCount].EventCount=0;
	Levels[NLevel].Paterns[Levels[NLevel].PaternCount].Probability=0.001;
	return (Levels[NLevel].PaternCount++);
}

void InputM::DeletePatern(int NLevel, int NPatern)
{
	if((NLevel<0)|(NLevel>=LevelCount)) return;
	if((NPatern<0)|(NPatern>=Levels[NLevel].PaternCount)) return;
	if(Levels[NLevel].Paterns[NPatern].EventCount !=0)	
		delete[] Levels[NLevel].Paterns[NPatern].Events;
	Levels[NLevel].PaternCount--;
	for(int P=NPatern; P<Levels[NLevel].PaternCount; P++) 
		Levels[NLevel].Paterns[P]=Levels[NLevel].Paterns[P+1];
}

int InputM::AddEvent(int NLevel, int NPatern)
{
	if((NLevel<0)|(NLevel>=LevelCount)) return -1;
	if((NPatern<0)|(NPatern>=Levels[NLevel].PaternCount)) return-1;
	Event* NewEvent = new Event[Levels[NLevel].Paterns[NPatern].EventCount+1];
	for(int E=0; E<Levels[NLevel].Paterns[NPatern].EventCount; E++)
		NewEvent[E] = Levels[NLevel].Paterns[NPatern].Events[E];
	if(Levels[NLevel].Paterns[NPatern].EventCount !=0) delete[] Levels[NLevel].Paterns[NPatern].Events;
	Levels[NLevel].Paterns[NPatern].Events = NewEvent;
	Levels[NLevel].Paterns[NPatern].Events[Levels[NLevel].Paterns[NPatern].EventCount].Input=0;
	Levels[NLevel].Paterns[NPatern].Events[Levels[NLevel].Paterns[NPatern].EventCount].Time=0.009;
	return(Levels[NLevel].Paterns[NPatern].EventCount++);
}

void InputM::DeleteEvent(int NLevel, int NPatern, int NEvent)
{
	if((NLevel<0)|(NLevel>=LevelCount)) return;
	if((NPatern<0)|(NPatern>=Levels[NLevel].PaternCount)) return;
	if((NEvent<0)|(NEvent>=Levels[NLevel].Paterns[NPatern].EventCount))return;
	Levels[NLevel].Paterns[NPatern].EventCount--;
	for(int E=NEvent; E<Levels[NLevel].Paterns[NPatern].EventCount; E++)
		Levels[NLevel].Paterns[NPatern].Events[E] = Levels[NLevel].Paterns[NPatern].Events[E+1];
}

void InputM::Assemble(void* MDL)
{
	Model *Mod = (Model*) MDL;
	for(int c=0; c<InputCount; c++){
		Mod->CellList[InputList[c]].Threshold = 10.0;
		Mod->CellList[InputList[c]].ThresholdPTime = 
		Mod->CellList[InputList[c]].ThresholdMTime =
		Mod->CellList[InputList[c]].FireThresholdTime = -1;
		Mod->CellList[InputList[c]].Potentioal = 0.0; 
		Mod->CellList[InputList[c]].Fireing = 1e36;
		Mod->CellList[InputList[c]].RC = 100;
		Mod->CellList[InputList[c]].C=1;
		Mod->CellList[InputList[c]].APR=1e16;
		Mod->CellList[InputList[c]].APC = 1;
		Mod->CellList[InputList[c]].APSingPotential = 0;
	}
	for(int L=0; L<LevelCount; L++){
		Levels[L].WorkPattern=-1;
		unsigned last=0;//(unsigned)(0.001*(double)0xffffffff);
		for(int P=0; P<Levels[L].PaternCount; P++){
			Levels[L].Paterns[P].From=last + 1;
			Levels[L].Paterns[P].To = last + 1 +
				(unsigned)((double)0xffffffff*Levels[L].Paterns[P].Probability);
			if(Levels[L].Paterns[P].To<last)Levels[L].Paterns[P].To = 0xffffffff;
			else last = Levels[L].Paterns[P].To;
			Levels[L].Paterns[P].StartTime=Levels[L].Paterns[P].TimeEnd=0;
			for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
				if(Levels[L].Paterns[P].Events[E].Time>Levels[L].Paterns[P].TimeEnd)
					Levels[L].Paterns[P].TimeEnd = Levels[L].Paterns[P].Events[E].Time;
			}
			Levels[L].Paterns[P].TimeEnd += FireTime + RefTime;
		}
	}
	Pause = false;
/*	CTime Time;
	int RandInit  = Time.GetSecond()*Time.GetMinute();
	srand( (unsigned) RandInit );
*/	ActCount = 0;
/*	if(Log.Open("Log.txt", CFile::modeWrite,NULL))
		Log.SeekToEnd();
	else
		Log.Open("Log.txt", CFile::modeCreate|CFile::modeWrite,NULL);
*/
}

void InputM::Calculate(void* MDL)
{
	Model *Mod = (Model*) MDL;
	for(int I=0; I<InputCount; I++)
		Mod->CellList[InputList[I]].Potentioal = 0;

	if(Pause) return;
	for(int L=0; L<LevelCount; L++){
		unsigned RND = (((rand()<<15)|rand())<<2)^rand();
		if(Levels[L].WorkPattern>=0){
			int P=Levels[L].WorkPattern;
			for(int E=0; E<Levels[L].Paterns[P].EventCount; E++){
				if(( Mod->CellList[0].CurrentTime >= 
				   (Levels[L].Paterns[P].Events[E].Time+Levels[L].Paterns[P].StartTime))&
				   (Mod->CellList[0].CurrentTime < 
				   (Levels[L].Paterns[P].Events[E].Time+Levels[L].Paterns[P].StartTime+Mod->CellList[0].TimeStep)))
						if(ActCount<10) ActCount++;
						else Mod->CellList[InputList[Levels[L].Paterns[P].Events[E].Input]].Fireing=0;
			}
			if(Mod->CellList[0].CurrentTime>(Levels[L].Paterns[P].StartTime+Levels[L].Paterns[P].TimeEnd)){
				Levels[L].WorkPattern=-1;
				Mod->SWatcher.DeActivate(L,P);
			}
		}
		else{
			for(int P=0; P<Levels[L].PaternCount; P++){
				if((RND > Levels[L].Paterns[P].From)&(RND <= Levels[L].Paterns[P].To)){
					Levels[L].WorkPattern=P;
					Levels[L].Paterns[P].StartTime = Mod->CellList[0].CurrentTime;
					Mod->SWatcher.Activate(L,P);
					L--;
					break;
				}
			}
		}
	}
}
