/*==========================================================================================*/
/*																							*/
/*								Model of Integrate-and-Fire Neurons							*/
/*                                  by Ruben Tikidji - Hamburyan                            */
/*																							*/
/*==========================================================================================*/

#include "stdafx.h"
#include "process.h"
#include "NeuroCAD.h"
#include "Model.h"
#include "SimulatorView.h"

typedef unsigned (__stdcall *PTHREAD_START)(void*);
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


HANDLE Model::hThread; HANDLE Model::hEvent;

Model::Model()
{
	CellList = new Cell[100]; MaxCell = 100;
	VCellList = new VCell[100];
	CurrentCell = 0; 
	for(int i=0; i<MaxCell; i++) 
		VCellList[i].Pos.x = VCellList[i].Pos.y = VCellList[i].Select = 0;
//Map
	ConMap = new VCon*[MaxCell];
	for(i=0; i<MaxCell; i++){
		ConMap[i]= new VCon[MaxCell];
		for(int j=0; j<MaxCell; j++){
			ConMap[i][j].List = NULL;ConMap[i][j].Current = 0;
		}
	}
	DD.DC_APM = DD.DC_APC = DD.DC_APR = DD.DC_APS = DD.DC_C = DD.DC_L = DD.DC_RC = DD.DC_SD =
	DD.DC_SF = DD.DC_SMax = DD.DC_SMin = DD.DC_SR = DD.DC_SS = DD.DC_TH = DD.DC_THTime = true;
	DD.Cos_APM = DD.Cos_APC = DD.Cos_APR = DD.Cos_APS = DD.Cos_C = DD.Cos_L = DD.Cos_RC = 
	DD.Cos_SD =	DD.Cos_SF = DD.Cos_SMax = DD.Cos_SMin = DD.Cos_SR = DD.Cos_SS = DD.Cos_TH = 
	DD.Cos_THTime = true;

	DD.Norm_RC = 20e-3;	DD.Norm_C = 250e-12;	
		DD.Norm_TH = 10e-3;	DD.Norm_THMTime = DD.Norm_THPTime = -1;
	DD.Min_RC = 5e-3;	DD.Min_C = 50e-12;		
		DD.Min_TH = 5e-3;	DD.Min_THMTime = DD.Min_THPTime = 0.05;
	DD.Max_RC = 50e-3;	DD.Max_C = 400e-12;		
		DD.Max_TH = 40e-3;	DD.Max_THMTime = DD.Max_THPTime = 0.7;

	DD.Norm_APS = -5e-3;	DD.Norm_APM = 2;	
		DD.Norm_APR = DD.Norm_RC*10/DD.Norm_C; DD.Norm_APC = DD.Norm_C*5;
	DD.Min_APS = -20e-3;	DD.Min_APM = 0;	
		DD.Min_APR = DD.Min_RC*10/DD.Min_C; DD.Min_APC = DD.Min_C*5;
	DD.Max_APS = 0;			DD.Max_APM = 5;	
		DD.Max_APR = DD.Max_RC/DD.Max_C; DD.Max_APC = DD.Max_C*5;

	DD.Norm_SD = 5e-3;	DD.Norm_SS = 40e-12;	DD.Norm_SMax = 150e-12;	DD.Norm_SMin = -1e-12;
	DD.Min_SD = 0;		DD.Min_SS = 1e-12;		DD.Min_SMax = 60e-12;	DD.Min_SMin = -7e-12;
	DD.Max_SD = 100e-3;	DD.Max_SS = 100e-12;	DD.Max_SMax = 400e-12;	DD.Max_SMin = -40e-12;

	DD.Norm_SR = 2e-3;	DD.Norm_SF = 2e-3;		DD.Norm_LP = DD.Norm_LM = 0;
	DD.Min_SR = 1e-3;	DD.Min_SF = 1e-3;		DD.Min_LP = DD.Min_LM = 0.005;
	DD.Max_SR = 5e-3;	DD.Max_SF = 20e-3;		DD.Max_LP = DD.Max_LM = 0.17;
	FillModel();
	SetDefault();
	SimulationFLG = false;
}

Model::~Model()
{
	for(int i=0; i<MaxCell; i++){
		for(int j=0; j<MaxCell; j++){
			if(ConMap[i][j].Current !=0) delete[] ConMap[i][j].List;
		}
		delete[] ConMap[i];
	}
	delete[] ConMap;
	delete[] VCellList;
	delete[] CellList;
}

void Model::NewModel()
{
	for(int i=0; i<MaxCell; i++){
		for(int j=0; j<MaxCell; j++){
			if(ConMap[i][j].Current !=0) delete[] ConMap[i][j].List;
		}
		delete[] ConMap[i];
	}
	delete[] ConMap;
	delete[] VCellList;
	delete[] CellList;

	CellList = new Cell[100]; MaxCell = 100;
	VCellList = new VCell[100];
	CurrentCell = 0; 
	for( i=0; i<100; i++)
		VCellList[i].Pos.x = VCellList[i].Pos.y = VCellList[i].Select = 0;

//Map
	ConMap = new VCon*[MaxCell];
	for(i=0; i<MaxCell; i++){
		ConMap[i]= new VCon[MaxCell];
		for(int j=0; j<MaxCell; j++){
			ConMap[i][j].List = NULL;ConMap[i][j].Current = 0;
		}
	}
	FillModel();
	SetDefault();
}

void Model::SaveModel(CFile &File)
{
	File.Write(&MaxCell, sizeof(MaxCell));
	File.Write(&CurrentCell, sizeof(MaxCell));
	for(int C = 0; C< CurrentCell+1; C++){
		File.Write(&CellList[C].RC,8);File.Write(&CellList[C].C,8);
		File.Write(&CellList[C].Threshold,8);
		File.Write(&CellList[C].ThresholdPTime,8);
		File.Write(&CellList[C].ThresholdMTime,8);
		File.Write(&CellList[C].FireThresholdTime,8);
		File.Write(&CellList[C].APR,8);
		File.Write(&CellList[C].APC,8);
		File.Write(&CellList[C].APSingPotential,8);
		File.Write(&CellList[C].CountSyn,4);File.Write(&CellList[C].MaxSyn,4);
		for(int S=0; S<CellList[C].CountSyn; S++){
			File.Write(&CellList[C].Syn[S].Delay,8);
			File.Write(&CellList[C].Syn[S].RisingTime,8);
			File.Write(&CellList[C].Syn[S].FallingTime,8);
			File.Write(&CellList[C].Syn[S].LearnP,8);
			File.Write(&CellList[C].Syn[S].LearnM,8);
			File.Write(&CellList[C].Syn[S].MinWeight,8);
			File.Write(&CellList[C].Syn[S].SingleWeight,8);
			File.Write(&CellList[C].Syn[S].MaxWeight,8);
		}
		File.Write(&VCellList[C].Pos,sizeof(VCellList[C].Pos));
		File.Write(&VCellList[C].Select,2);
	}
	int LinkCount = 0;
	for(int OCell=0; OCell<CurrentCell; OCell++){
		for(int ICell=0; ICell<CurrentCell; ICell++){
			if(ConMap[OCell][ICell].Current !=0) LinkCount++;
		}
	}
	
	File.Write(&LinkCount,4);
	for(OCell=0; OCell<CurrentCell; OCell++){
		for(int ICell=0; ICell<CurrentCell; ICell++){
			if(ConMap[OCell][ICell].Current !=0){
				File.Write(&OCell,4);
				File.Write(&ICell,4);
				File.Write(&ConMap[OCell][ICell].Current,4);
				for(int Lnk=0; Lnk<ConMap[OCell][ICell].Current; Lnk++)
					File.Write(&ConMap[OCell][ICell].List[Lnk],4);
			}
		}
	}
	Input.SaveInputMachine(File);
	File.Write(&DD,sizeof(DD));
}

void Model::LoadModel(CFile &File)
{	
	for(int i=0; i<MaxCell; i++){
		for(int j=0; j<MaxCell; j++){
			if(ConMap[i][j].Current !=0) delete[] ConMap[i][j].List;
		}
		delete[] ConMap[i];
	}
	delete[] ConMap;
	delete[] VCellList;
	delete[] CellList;

	File.Read(&MaxCell,4);
	File.Read(&CurrentCell,4);
	CellList = new Cell[MaxCell]; VCellList = new VCell[MaxCell];
	for(int C = 0; C < CurrentCell+1; C++){
		File.Read(&CellList[C].RC,8);File.Read(&CellList[C].C,8);
		File.Read(&CellList[C].Threshold,8);
		File.Read(&CellList[C].ThresholdPTime,8);
		File.Read(&CellList[C].ThresholdMTime,8);
		File.Read(&CellList[C].FireThresholdTime,8);
		File.Read(&CellList[C].APR,8);
		File.Read(&CellList[C].APC,8);
		File.Read(&CellList[C].APSingPotential,8);
		File.Read(&CellList[C].CountSyn,4);File.Read(&CellList[C].MaxSyn,4);
		delete[] CellList[C].Syn; CellList[C].Syn = new Synapse[CellList[C].MaxSyn];
		for(int S=0; S<CellList[C].CountSyn; S++){
			File.Read(&CellList[C].Syn[S].Delay,8);
			File.Read(&CellList[C].Syn[S].RisingTime,8);
			File.Read(&CellList[C].Syn[S].FallingTime,8);
			File.Read(&CellList[C].Syn[S].LearnP,8);
			File.Read(&CellList[C].Syn[S].LearnM,8);
			File.Read(&CellList[C].Syn[S].MinWeight,8);
			File.Read(&CellList[C].Syn[S].SingleWeight,8);
			File.Read(&CellList[C].Syn[S].MaxWeight,8);
		}
		File.Read(&VCellList[C].Pos,sizeof(VCellList[C].Pos));
		File.Read(&VCellList[C].Select,2);
	}
	ConMap = new VCon*[MaxCell];
	for(int OCell=0; OCell<MaxCell; OCell++){
		ConMap[OCell] = new VCon[MaxCell];
		for(int ICell=0; ICell<MaxCell; ICell++){
			ConMap[OCell][ICell].List = NULL;ConMap[OCell][ICell].Current=0;
		}
	}

	int LinkCount = 0, ICell = 0;
	File.Read(&LinkCount,4);
	for(int LinkCnt=0; LinkCnt<LinkCount; LinkCnt++){
		File.Read(&OCell,4);
		File.Read(&ICell,4);
		File.Read(&ConMap[OCell][ICell].Current,4);
		ConMap[OCell][ICell].List = new int[ConMap[OCell][ICell].Current];
		for(int Lnk=0; Lnk<ConMap[OCell][ICell].Current; Lnk++)
			File.Read(&ConMap[OCell][ICell].List[Lnk],4);
	}


	for(C=CurrentCell; C<MaxCell; C++)
		VCellList[C].Select=VCellList[C].Pos.x=VCellList[C].Pos.y=0;

	Input.LoadInputMachine(File);
	File.Read(&DD,sizeof(DD));
	FillModel();
	SetDefault();
}

void Model::LoadModel_Old(CFile &File)
{
		for(int i=0; i<MaxCell; i++){
		for(int j=0; j<MaxCell; j++){
			if(ConMap[i][j].Current !=0) delete[] ConMap[i][j].List;
		}
		delete[] ConMap[i];
	}
	delete[] ConMap;
	delete[] VCellList;
	delete[] CellList;

	File.Read(&MaxCell,4);
	File.Read(&CurrentCell,4);
	CellList = new Cell[MaxCell]; VCellList = new VCell[MaxCell];
	for(int C = 0; C < CurrentCell+1; C++){
		File.Read(&CellList[C].RC,8);File.Read(&CellList[C].C,8);
		File.Read(&CellList[C].Threshold,8);
		File.Read(&CellList[C].ThresholdPTime,8);
		File.Read(&CellList[C].ThresholdMTime,8);
		File.Read(&CellList[C].FireThresholdTime,8);
		File.Read(&CellList[C].APR,8);
		File.Read(&CellList[C].APC,8);
		File.Read(&CellList[C].APSingPotential,8);
		File.Read(&CellList[C].CountSyn,4);File.Read(&CellList[C].MaxSyn,4);
		delete[] CellList[C].Syn; CellList[C].Syn = new Synapse[CellList[C].MaxSyn];
		for(int S=0; S<CellList[C].CountSyn; S++){
			File.Read(&CellList[C].Syn[S].Delay,8);
			File.Read(&CellList[C].Syn[S].RisingTime,8);
			File.Read(&CellList[C].Syn[S].FallingTime,8);
			File.Read(&CellList[C].Syn[S].LearnP,8);
			File.Read(&CellList[C].Syn[S].LearnM,8);
			File.Read(&CellList[C].Syn[S].MinWeight,8);
			File.Read(&CellList[C].Syn[S].SingleWeight,8);
			File.Read(&CellList[C].Syn[S].MaxWeight,8);
		}
		File.Read(&VCellList[C].Pos,sizeof(VCellList[C].Pos));
		File.Read(&VCellList[C].Select,2);
	}
	ConMap = new VCon*[MaxCell];
	for(int OCell=0; OCell<CurrentCell; OCell++){
		ConMap[OCell] = new VCon[MaxCell];
		for(int ICell=0; ICell<CurrentCell; ICell++){
			File.Read(&ConMap[OCell][ICell].Current,4);
			if(ConMap[OCell][ICell].Current !=0){
				ConMap[OCell][ICell].List = new int[ConMap[OCell][ICell].Current];
				for(int Lnk=0; Lnk<ConMap[OCell][ICell].Current; Lnk++)
					File.Read(&ConMap[OCell][ICell].List[Lnk],4);
			}
			else ConMap[OCell][ICell].List = NULL;
		}
		for(ICell=CurrentCell; ICell<MaxCell; ICell++){
			ConMap[OCell][ICell].List = NULL;ConMap[OCell][ICell].Current=0;
		}

	}
	for(OCell=CurrentCell; OCell<MaxCell; OCell++){
		ConMap[OCell] = new VCon[MaxCell];
		for(int ICell=0; ICell<MaxCell; ICell++){
			ConMap[OCell][ICell].List = NULL;ConMap[OCell][ICell].Current=0;
		}
	}


	for(C=CurrentCell; C<MaxCell; C++)
		VCellList[C].Select=VCellList[C].Pos.x=VCellList[C].Pos.y=0;

	Input.LoadInputMachine(File);
	File.Read(&DD,sizeof(DD));
	FillModel();
	SetDefault();
}


int Model::AddCell(Position Point)
{
	AddObject();
	VCellList[CurrentCell].Pos = Point;
	VCellList[CurrentCell].Select = 0;
	return(CurrentCell++);

}
void Model::DeleteCell(int Number)
{
	if(Number >= CurrentCell) return;
	CurrentCell --;
	for(int i=Number; i<CurrentCell; i++){
		CellList[i] = CellList[i+1]; VCellList[i] = VCellList[i+1];
	}
	for(int I=0; I<Input.InputCount; I++){
		if(Input.InputList[I]>Number)Input.InputList[I]--;
	}
	for( i=0; i<CurrentCell; i++){
		if(ConMap[Number][i].Current>0) {
			delete[]ConMap[Number][i].List;ConMap[Number][i].Current=0;}
		if(ConMap[i][Number].Current>0) {
			delete[]ConMap[i][Number].List;ConMap[i][Number].Current=0;}
		for(int j=0; j<CurrentCell; j++){
			if(j>=Number)ConMap[i][j]=ConMap[i][j+1];
			if(i>=Number)ConMap[i][j]=ConMap[i+1][j];
		}
	}
}

int Model::AddLink(int FromCell, int ToCell)
{
	if(((FromCell>CurrentCell)|(FromCell<0))&((ToCell>CurrentCell)|(ToCell<0))) return -1;
	if(ConMap[FromCell][ToCell].Current == 0){
		ConMap[FromCell][ToCell].List = new int[ConMap[FromCell][ToCell].Current=1];
		return(ConMap[FromCell][ToCell].List[0] = CellList[ToCell].CreateSynapse());
	}
	else{
		int *New = new int[ConMap[FromCell][ToCell].Current+1];
		for(int i=0; i<ConMap[FromCell][ToCell].Current; i++) 
			New[i] = ConMap[FromCell][ToCell].List[i];
		delete[] ConMap[FromCell][ToCell].List;
		ConMap[FromCell][ToCell].List = New;
		ConMap[FromCell][ToCell].Current++;
		if(CellList[ToCell].CountSyn+1 < CellList[ToCell].MaxSyn)
			return(ConMap[FromCell][ToCell].List[ConMap[FromCell][ToCell].Current-1]
														= CellList[ToCell].CreateSynapse());
		else{
			ConMap[FromCell][ToCell].List[ConMap[FromCell][ToCell].Current-1]
														= CellList[ToCell].CreateSynapse();
			FillModel();
			SetDefault();
			return ConMap[FromCell][ToCell].List[ConMap[FromCell][ToCell].Current-1];
		}
	}
}

void Model::DeleteLink(int NCell, int NSyn)
{
	if((NCell>=CurrentCell)|(NCell<0)) return;
	if(NSyn >= CellList[NCell].CountSyn) return;
	int i=0, From=-1, S=-1;
	while((i<CurrentCell)&(From<0)){
		for(int j=0; j<ConMap[i][NCell].Current; j++){
			if(ConMap[i][NCell].List[j] == NSyn){
				From = i; S=j; break;
			}
		}
		i++;
	}
	if((From<0)|(S<0))return;
	if(ConMap[From][NSyn].Current>1){
		ConMap[From][NCell].Current--;
		int *New = new int[ConMap[From][NCell].Current];
		for(i=0; i<ConMap[From][NCell].Current; i++){
			if(i<S)	New[i] = ConMap[From][NCell].List[i];
			else	New[i] = ConMap[From][NCell].List[i+1];
		}
		delete[] ConMap[From][NCell].List;
		ConMap[From][NCell].List = New;
	}
	else{
		delete[] ConMap[From][NCell].List;
		ConMap[From][NCell].Current = 0;
	}
	for(i=0; i<CurrentCell; i++){
		for(int j=0;j<ConMap[i][NCell].Current;j++){
			if(ConMap[i][NCell].List[j]>NSyn)ConMap[i][NCell].List[j]--;
		}
	}
	CellList[NCell].DeleteSynapse(NSyn);
}

int Model::AddInputM(Position Point)
{
	AddObject();
	VCellList[CurrentCell].Pos = Point; VCellList[CurrentCell].Select = 0x10;
	Input.AddInput(CurrentCell);
	return(CurrentCell++);
}

void Model::DeleteInputM(int Number)
{
	int NInput = Input.GetNumberInput(Number);
	if(NInput<0) return;
	Input.DeleteInput(NInput);
	DeleteCell(Number);
}

int Model::AddIfM(Position Point)
{
	AddObject();
	VCellList[CurrentCell].Pos = Point; VCellList[CurrentCell].Select = 0x20;
	IfModul.AddInput(CurrentCell);
	return(CurrentCell++);
}

int Model::AddObject()
{
///!!!!!
	if(CurrentCell+1==MaxCell){
		Cell *NewCell = new Cell[MaxCell+100];
		VCell *NewViewCell = new VCell[MaxCell+100];
		VCon **NewMap = new VCon*[MaxCell+100];

		for(int i=0; i<MaxCell+100; i++){
			NewMap[i] = new VCon[MaxCell+100];
			for(int j=CurrentCell; j<MaxCell+100; j++){
				NewMap[i][j].List = NULL;	NewMap[i][j].Current = 0;
			}
			if(i<CurrentCell){
				for(int j=0; j<CurrentCell; j++) NewMap[i][j]=ConMap[i][j];
				delete[] ConMap[i];
				NewCell[i] = CellList[i]; NewViewCell[i] = VCellList[i];
			}
			else{
				for(int j=0; j<CurrentCell; j++){
					NewMap[i][j].List = NULL;	NewMap[i][j].Current = 0;
				}
				NewViewCell[i].Pos.x = NewViewCell[i].Pos.y = NewViewCell[i].Select = 0;
			}

		}


		delete[] CellList; delete[] VCellList; delete[] ConMap;
		VCellList = NewViewCell; CellList = NewCell; ConMap = NewMap;

		MaxCell +=100;
		FillModel();
		SetDefault();
	}
	return CurrentCell;

}

void Model::SetDefault()
{
	unsigned RND;
	for(int C=0; C<MaxCell; C++){
		for(int S= CellList[C].CountSyn; S<CellList[C].MaxSyn; S++){
			if(!DD.DC_SD){
				if(DD.Cos_SD) CellList[C].Syn[S].Delay= DD.Norm_SD;
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].Syn[S].Delay = 
						DD.Min_SD+RND*(DD.Max_SD - DD.Min_SD)/0xffffffff;
				}
			}
			RND = (((rand()<<15)|rand())<<2)^rand();
			if(!DD.DC_SS){
				if(DD.Cos_SS) CellList[C].Syn[S].SingleWeight= DD.Norm_SS;
				else{
					CellList[C].Syn[S].SingleWeight = 
						DD.Min_SS+RND*(DD.Max_SS - DD.Min_SS)/0xffffffff;
				}
			}
			if(!DD.DC_SMin){
				if(DD.Cos_SMin) CellList[C].Syn[S].MinWeight= DD.Norm_SMin;
				else{
					CellList[C].Syn[S].MinWeight = 
						DD.Min_SMin+RND*(DD.Max_SMin - DD.Min_SMin)/0xffffffff;
				}
			}
			if(!DD.DC_SMax){
				if(DD.Cos_SMax) CellList[C].Syn[S].MaxWeight= DD.Norm_SMax;
				else{
					CellList[C].Syn[S].MaxWeight = 
						DD.Min_SMax+RND*(DD.Max_SMax - DD.Min_SMax)/0xffffffff;
				}
			}
			RND = (((rand()<<15)|rand())<<2)^rand();
			if(!DD.DC_SR){
				if(DD.Cos_SR) CellList[C].Syn[S].RisingTime= DD.Norm_SR;
				else{
					CellList[C].Syn[S].RisingTime = 
						DD.Min_SR+RND*(DD.Max_SR - DD.Min_SR)/0xffffffff;
				}
			}
			if(!DD.DC_SF){
				if(DD.Cos_SF) CellList[C].Syn[S].FallingTime= DD.Norm_SF;
				else{
					CellList[C].Syn[S].FallingTime = 
						DD.Min_SF+RND*(DD.Max_SF - DD.Min_SF)/0xffffffff;
				}
			}
			if(!DD.DC_L){
				if(DD.Cos_L){
					CellList[C].Syn[S].LearnP= DD.Norm_LP;
					CellList[C].Syn[S].LearnM= DD.Norm_LM;
				}
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].Syn[S].LearnP = 
						DD.Min_LP+RND*(DD.Max_LP - DD.Min_LP)/0xffffffff;
					CellList[C].Syn[S].LearnM = 
						DD.Min_LM+RND*(DD.Max_LM - DD.Min_LM)/0xffffffff;
				}
			}
		}
		if(C>=CurrentCell){
			if(!DD.DC_RC){
				if(DD.Cos_RC) CellList[C].RC= DD.Norm_RC;
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].RC = 
						DD.Min_RC+RND*(DD.Max_RC - DD.Min_RC)/0xffffffff;
				}
			}
			if(!DD.DC_C){
				if(DD.Cos_C) CellList[C].C= DD.Norm_C;
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].C = 
						DD.Min_C+RND*(DD.Max_C - DD.Min_C)/0xffffffff;
				}
			}
			if(!DD.DC_TH){
				if(DD.Cos_TH) CellList[C].Threshold= DD.Norm_TH;
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].Threshold = 
						DD.Min_TH+RND*(DD.Max_TH - DD.Min_TH)/0xffffffff;
				}
			}
			RND = (((rand()<<15)|rand())<<2)^rand();
			if(!DD.DC_THTime){
				if(DD.Cos_THTime){ 
					CellList[C].ThresholdPTime= DD.Norm_THPTime;
					CellList[C].ThresholdMTime= DD.Norm_THMTime;
				}
				else{
					RND = (((rand()<<15)|rand())<<2)^rand();
					CellList[C].ThresholdPTime = 
						DD.Min_THPTime+RND*(DD.Max_THPTime - DD.Min_THPTime)/0xffffffff;
					CellList[C].ThresholdMTime = 
						DD.Min_THMTime+RND*(DD.Max_THMTime - DD.Min_THMTime)/0xffffffff;
				}
			}
			RND = (((rand()<<15)|rand())<<2)^rand();
			if(!DD.DC_APS){
				if(DD.Cos_APS) CellList[C].APSingPotential = DD.Norm_APS;
				else{
					CellList[C].APSingPotential = 
						DD.Min_APS+RND*(DD.Max_APS - DD.Min_APS)/0xffffffff;
				}
			}
			if(!DD.DC_APM){
				if(DD.Cos_APM) CellList[C].FireThresholdTime = DD.Norm_APM;
				else{
					CellList[C].FireThresholdTime = 
						DD.Min_APM+RND*(DD.Max_APM - DD.Min_APM)/0xffffffff;
				}
			}
			RND = (((rand()<<15)|rand())<<2)^rand();
			if(!DD.DC_APR){
				if(DD.Cos_APR) CellList[C].APR= DD.Norm_APR;
				else{
					CellList[C].APR = 
						DD.Min_APR+RND*(DD.Max_APR - DD.Min_APR)/0xffffffff;
				}
			}
			if(!DD.DC_APC){
				if(DD.Cos_APC) CellList[C].APC = DD.Norm_APC;
				else{
					CellList[C].APC = 
						DD.Min_APC+RND*(DD.Max_APC - DD.Min_APC)/0xffffffff;
				}
			}
		}
	}
}

void Model::FillModel()
{
	for(int C=0; C<MaxCell; C++){
		for(int S= CellList[C].CountSyn; S<CellList[C].MaxSyn; S++){
			CellList[C].Syn[S].Delay= DD.Norm_SD;
			CellList[C].Syn[S].SingleWeight= DD.Norm_SS;
			CellList[C].Syn[S].MinWeight= DD.Norm_SMin;
			CellList[C].Syn[S].MaxWeight= DD.Norm_SMax;
			CellList[C].Syn[S].RisingTime= DD.Norm_SR;
			CellList[C].Syn[S].FallingTime= DD.Norm_SF;
			CellList[C].Syn[S].LearnP= DD.Norm_LP;
			CellList[C].Syn[S].LearnM= DD.Norm_LM;
		}
		if(C>=CurrentCell){
			CellList[C].RC= DD.Norm_RC;
			CellList[C].C= DD.Norm_C;
			CellList[C].Threshold= DD.Norm_TH;
			CellList[C].ThresholdPTime= DD.Norm_THPTime;
			CellList[C].ThresholdMTime= DD.Norm_THMTime;
			CellList[C].APSingPotential= DD.Norm_APS;
			CellList[C].FireThresholdTime= DD.Norm_APM;
			CellList[C].APR= DD.Norm_APR;
			CellList[C].APC= DD.Norm_APC;
		}
	}

}

void Model::Assemble()
{
		CellList[0].First = &CellList[0];
		CellList[0].Last  = &CellList[CurrentCell-1];
		for(int From=0; From<CurrentCell; From++){
			for(int To=0; To<CurrentCell; To++){
				for(int S = 0; S<ConMap[From][To].Current; S++){
					CellList[To].Syn[ConMap[From][To].List[S]].An = &CellList[From];
				}
			}
			for(int S = 0; S < CellList[From].CountSyn; S++){
				CellList[From].Syn[S].dTime = 100;
				CellList[From].Syn[S].I = 0;
			}
			CellList[From].Fireing = 10;
			CellList[From].APPotential = CellList[From].PotentialStat =
				CellList[From].Potentioal = 0;
		}
		CellList->TimeStep = 5e-4;
		Input.Assemble(this);
		IfModul.Assemble(this);
		SWatcher.Assemble(this);
}

bool Model::StartSimulator(bool TS, double PeriodV, CWnd *View)
{
	if(SimulationFLG) return false;
	Period = PeriodV;
	pView = View;
	if(TS){
		//Assemble();
		unsigned Id;
		hThread =(HANDLE) _beginthreadex(NULL,0,(PTHREAD_START)TimeThread,this,CREATE_SUSPENDED,&Id);
		if(!SetThreadPriority(hThread,THREAD_PRIORITY_HIGHEST)) return false;
		hEvent = CreateEvent(NULL,false,true,"UpDate");
		Stop = true;
		if(ResumeThread(hThread) == 0xffffffff) return false;
		SimulationFLG = true;
	}
	else{
		return false;
	}
	return true;
}

bool Model::StepSimulator()
{
	if(SimulationFLG)
		SetEvent(hEvent);	
	return SimulationFLG;

}

bool Model::StopSimulator()
{
	Stop = false;	return StepSimulator();
}

DWORD WINAPI TimeThread(void *pModel)
{
	Model *Mod = (Model*)pModel;SimulatorView *pView = (SimulatorView*)Mod->pView;
	double TimeStart = 0;
	do{
		TimeStart = Mod->CellList[0].CurrentTime;
		while((TimeStart+Mod->Period)>Mod->CellList->CurrentTime){
			Mod->CellList[0].Calculate();
			Mod->Input.Calculate(Mod);
			Mod->IfModul.Calculate(Mod);
			if(pView->PCM != NULL){
				for(int R=0; R<pView->PCMCount; R++){
					if(pView->PCM[R] != NULL) pView->PCM[R]->m_wndView.NewDate();
				}
			}
			if(pView->MR != NULL) pView->MR->Recode();
			if(pView->SimRec != NULL) pView->SimRec->WriteSampl();
			Mod->SWatcher.Recode();
//			Mod->CA.Calculate(Mod);
		}
		pView->SendMessage(WM_REVIEW,0,0);
		WaitForSingleObject(Mod->hEvent,INFINITE);
	}while(Mod->Stop);
	CloseHandle(Mod->hEvent);
	Mod->SimulationFLG = false;
	return 0;
}

DWORD WINAPI StepThread(void *pModel)
{
/*	Model *Mod = (Model*)pModel;CWnd *pView = Mod->pView;
	do{
		pView->SendMessage(WM_REVIEW);
		WaitForSingleObject(Mod->hEvent,INFINITE);
	}while(Mod->Stop);
	CloseHandle(Mod->hEvent);
*/	return 0;
}
