/*==========================================================================================*/
/*																							*/
/*								Model of Integrate-and-Fire Neurons							*/
/*                                  by Ruben Tikidji - Hamburyan                            */
/*																							*/
/*==========================================================================================*/

#if !defined(AFX_MODEL_H__11564214_37F9_11D5_85C7_5254AB1ABD30__INCLUDED_)
#define AFX_MODEL_H__11564214_37F9_11D5_85C7_5254AB1ABD30__INCLUDED_

#define WM_REVIEW		WM_USER+5
#define WM_SHIFTLOCK	WM_USER+6
#define WM_REVIEW_MOD	WM_USER+50
#define WM_REVIEW_CRF	WM_USER+51
#define WM_REVIEW_SIM	WM_USER+52

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Cell.h"
#include "InputM.h"
#include "StatWatcher.h"
#include "IfM.h"
#include "ConvergentAgent.h"

struct Position {
	int x; int y;
	void operator = (Position &pos) {x = pos.x; y=pos.y;};
	bool operator == (Position &pos) {return((x==pos.x)&(y==pos.y));};
	bool operator != (Position &pos) {return(!((x==pos.x)&(y==pos.y)));};
	Position operator - (Position A){return Position ((x - A.x),(y - A.y));};
	Position operator + (Position A){return Position ((x + A.x),(y + A.y));};
	void operator += (Position A){x += A.x; y += A.y;};
	void operator -= (Position A){x -= A.x; y -= A.y;};
	Position(){this->x=0; this->y=0;};
	Position(int X, int Y){this->x = X; this->y = Y;};
//	int operator sizeof(Position pos) {return(sizeof(x)+sizeof(y));};
};
struct VCell	{Position Pos; short Select;};
struct VCon		{int *List, Current;};

struct DefaultData{
	bool DC_RC, DC_C,  DC_TH, DC_THTime, 
		DC_APS, DC_APM, DC_APR, DC_APC,
		DC_SD, DC_SS, DC_SMax, DC_SMin, DC_SR, DC_SF, DC_L;
	bool Cos_RC, Cos_C,  Cos_TH, Cos_THTime, 
		Cos_APS, Cos_APM, Cos_APR, Cos_APC,
		Cos_SD, Cos_SS, Cos_SMax, Cos_SMin, Cos_SR, Cos_SF, Cos_L;
	double Norm_RC, Norm_C,  Norm_TH, Norm_THPTime, Norm_THMTime, 
		 Norm_APS, Norm_APM, Norm_APR, Norm_APC,
		 Norm_SD, Norm_SS, Norm_SMax, Norm_SMin, Norm_SR, Norm_SF, Norm_LP, Norm_LM;
	double Min_RC, Min_C,  Min_TH, Min_THMTime, Min_THPTime, 
		 Min_APS, Min_APM, Min_APR, Min_APC,
		 Min_SD, Min_SS, Min_SMax, Min_SMin, Min_SR, Min_SF, Min_LP, Min_LM;
	double Max_RC, Max_C,  Max_TH, Max_THMTime, Max_THPTime, 
		 Max_APS, Max_APM, Max_APR,Max_APC,
		 Max_SD, Max_SS, Max_SMax, Max_SMin, Max_SR, Max_SF, Max_LP, Max_LM;
};

class Model  
{
public:
//Cell
	Cell *CellList;
	VCell *VCellList;
	int MaxCell, CurrentCell;
	VCon **ConMap;				//[From][To]
	DefaultData DD;

//Input Machine
	InputM Input;

//IF Modul
	IfM IfModul;
//ConvergentAgent
	ConvergentAgent CA;

//Simulate variable
	bool Stop;
	CWnd *pView;
	double Period;
	static	HANDLE hThread, hEvent;
	bool SimulationFLG;

//	HANDLE hThread, hEvent;
	StatWatcher SWatcher;

//Creator
	Model();
	~Model();
	void NewModel();
	void SaveModel(CFile &File);
	void LoadModel(CFile &File);
	void LoadModel_Old(CFile &File);

	int AddCell(Position Point);
	void DeleteCell(int Number);
	int AddLink(int FromCell, int ToCell);
	void DeleteLink(int NCell, int NSyn);
	int AddInputM(Position Point);
	void DeleteInputM(int Number);
	int AddIfM(Position Point);

	void SetDefault();
	void FillModel();

//Simulator
	void Assemble();
	bool StartSimulator(bool TS, double PeriodV, CWnd *View);
	bool StepSimulator();
	bool StopSimulator();

protected:
	inline int AddObject();
};

DWORD WINAPI TimeThread(void *pModel);
DWORD WINAPI StepThread(void *pModel);


#endif // !defined(AFX_MODEL_H__11564214_37F9_11D5_85C7_5254AB1ABD30__INCLUDED_)
