// PolyChannelView.cpp : implementation file
//

#include "stdafx.h"
#include "NeuroCAD.h"
#include "PolyChannelView.h"
#include "math.h"
#include "WriteBmp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PCMChooser dialog


PCMChooser::PCMChooser(CWnd* pParent /*=NULL*/)
	: CDialog(PCMChooser::IDD, pParent)
{
	//{{AFX_DATA_INIT(PCMChooser)
	m_Time = 0.001;
	m_ReDraw = 0.5f;
	//}}AFX_DATA_INIT
	Full = true;
}


void PCMChooser::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(PCMChooser)
	DDX_Text(pDX, IDC_SV_RECODETIME, m_Time);
	DDX_Text(pDX, IDC_SV_REDRAWTIME, m_ReDraw);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(PCMChooser, CDialog)
	//{{AFX_MSG_MAP(PCMChooser)
	ON_BN_CLICKED(IDC_SV_FULL, OnSvFull)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PCMChooser message handlers

void PCMChooser::OnSvFull() 
{
	Full = !Full;
	CButton* FB = (CButton*)GetDlgItem(IDC_SV_FULL);
	CEdit* REdit =(CEdit*)GetDlgItem(IDC_SV_RECODETIME);
	FB->SetCheck((int)Full);
	REdit->EnableWindow(!Full);
}

BOOL PCMChooser::OnInitDialog() 
{
	CDialog::OnInitDialog();
	CButton* FB = (CButton*)GetDlgItem(IDC_SV_FULL);
	CEdit* REdit =(CEdit*)GetDlgItem(IDC_SV_RECODETIME);
	FB->SetCheck((int)Full);
	REdit->EnableWindow(!Full);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// PolyChannelView

IMPLEMENT_DYNCREATE(PolyChannelView, CWnd)

PolyChannelView::PolyChannelView()
{
	CRV = NULL; Mod = NULL; pCell = NULL;
	CellCount = 0;
	pStartData = NULL;
	pBmp.CreateBitmap(1,1,1,32,NULL);
	SizeFlg	= false;
	OldBmp = NULL;

	ColBack = RGB(0,0,0);
	ColLine = RGB(255,255,50);
	ColSelect = RGB(100,150,20);
	ColGrid = RGB(150,0,0);
	ColAP = RGB(100,240,100);
	ColTH = RGB(255,255,255);

	GridFlg = true;
	AllRecords = 0;
	PreCount = 0;
	RecordSize = 0;
	PrePointCount = 0.0f;
	
//	ReDraw = true;
	LastPointY = NULL;
	hDrawing = CreateEvent(NULL,true,true,"Drawinf");
	RecordingFlg = false;



}

PolyChannelView::~PolyChannelView()
{
	if(pCell != NULL) delete[] pCell;
	if(CRV != NULL){
		for(int c=0; c<CellCount; c++)
			if(CRV[c].SRV != NULL) delete[] CRV[c].SRV;
		delete[] CRV;
	}
	StopRecord();
	MDC.DeleteDC();
	delete[] LastPointY;
	CloseHandle(hDrawing);
}


BEGIN_MESSAGE_MAP(PolyChannelView, CWnd)
	//{{AFX_MSG_MAP(PolyChannelView)
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PolyChannelView diagnostics

#ifdef _DEBUG
void PolyChannelView::AssertValid() const
{
	CWnd::AssertValid();
}

void PolyChannelView::Dump(CDumpContext& dc) const
{
	CWnd::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// PolyChannelView message handlers

int PolyChannelView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	chooser.DoModal();
	if(chooser.m_ReDraw<20.0f)	Resolution = chooser.m_ReDraw;
	RecordSize = 1;
	if((Mod != NULL)&(CellCount != 0)&(pCell != NULL)){
		CRV = new CellRecordView[CellCount];
		for(int c=0; c<CellCount; c++){
			RecordSize +=3;
			CRV[c].VP = true; CRV[c].VAP = CRV[c].VTH = CRV[c].AttAP = CRV[c].AttTH =false;
			CRV[c].KP = CRV[c].KAP = CRV[c].KTH = 5e2;
			CRV[c].SRV = new SynpRecordView[Mod->CellList[pCell[c]].CountSyn];
			CRV[c].Count = Mod->CellList[pCell[c]].CountSyn;
			for(int s=0; s<Mod->CellList[pCell[c]].CountSyn; s++){
				CRV[c].SRV[s].VIn = CRV[c].SRV[s].VSIn = false;
				CRV[c].SRV[s].KIn = CRV[c].SRV[s].KSIn = 1e11;
				RecordSize += 2;
			}
		}
		ChannelCount = CellCount;
	}
	SVC.MaxCell = CellCount;
	SVC.CRV = CRV;
	return 0;
}

void PolyChannelView::NewDate()
{
	if(!chooser.Full){
		if(PreCount<chooser.m_Time) {PreCount+=(float)Mod->CellList[0].TimeStep;return;}
		else PreCount = 0;
	}
	if(RecordingFlg){
		if((BYTE*)(pDataLast+RecordSize)>=MaxFileSize) ReMapFile();
		*pDataLast = (float)Mod->CellList[0].CurrentTime;						pDataLast++;
		for(int c=0; c<CellCount; c++){
			*pDataLast = (float)Mod->CellList[pCell[c]].Potentioal;				pDataLast++;
			*pDataLast = (float)Mod->CellList[pCell[c]].APPotential;			pDataLast++;
//			*pDataLast = (float)Mod->CellList[pCell[c]].SynK;					pDataLast++;
			*pDataLast = (float)Mod->CellList[pCell[c]].ThresholdLevel;			pDataLast++;
			for(int S=0; S<Mod->CellList[pCell[c]].CountSyn; S++){
				*pDataLast = (float)Mod->CellList[pCell[c]].Syn[S].I;			pDataLast++;
				*pDataLast = (float)Mod->CellList[pCell[c]].Syn[S].SingleWeight;pDataLast++;
			}
		}
	AllRecords++;
	*((int*)pStartData) = AllRecords;
	}
//	SetScrollRange(SB_HORZ,(int)((float)size.cx*Resolution),AllRecords,true);	
//	SetScrollPos(SB_HORZ,AllRecords,true);


	PrePointCount += Resolution;
	CPen *OPen, NPen(PS_SOLID,2,ColLine);
	CPen APPen(PS_DASH,1, ColAP), THPen(PS_DOT,1, ColTH);

	OPen = MDC.SelectObject(&NPen);
	CPoint Current;
	Current.x = ViewLeft+ (int)floorf(PrePointCount);
	if(Current.x>size.cx) Current.x=LastPointX=0;

	int ChannelP = 0, Ch=0, CHight = 0; 
	for(int Cp = 0; Cp<CellCount; Cp++){
		if(CRV[Cp].VP){
			MDC.MoveTo(LastPointX,LastPointY[Ch]);
			ChannelP = CHight;
			Current.y = ChannelHight*(CHight++) + ChannelHight/2-
				(int)(Mod->CellList[pCell[Cp]].Potentioal * CRV[Cp].KP);
			MDC.LineTo(Current);
			LastPointY[Ch++] = Current.y;
			if(Ch>=ChannelCount) break;
		}
		if(CRV[Cp].VAP){
			MDC.MoveTo(LastPointX,LastPointY[Ch]);
			if(CRV[Cp].AttAP&CRV[Cp].VP){
				MDC.SelectObject(&APPen);
				Current.y = ChannelHight*ChannelP + ChannelHight/2-
					(int)(Mod->CellList[pCell[Cp]].APPotential*CRV[Cp].KP);
			}
			else
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(Mod->CellList[pCell[Cp]].APPotential*CRV[Cp].KAP);
			MDC.LineTo(Current);
			LastPointY[Ch++] = Current.y;
			MDC.SelectObject(&NPen);
			if(Ch>=ChannelCount) break;
		}
		if(CRV[Cp].VTH){
			MDC.MoveTo(LastPointX,LastPointY[Ch]);
			if(CRV[Cp].AttTH&CRV[Cp].VP){
				MDC.SelectObject(&THPen);
				Current.y = ChannelHight*ChannelP + ChannelHight/2 - 
					(int)(Mod->CellList[pCell[Cp]].ThresholdLevel * CRV[Cp].KP);
			}
			else
				Current.y = ChannelHight*(CHight++) + ChannelHight/2 - 
					(int)(Mod->CellList[pCell[Cp]].ThresholdLevel * CRV[Cp].KTH);
			MDC.LineTo(Current);
			LastPointY[Ch++] = Current.y;
			MDC.SelectObject(&NPen);
			if(Ch>=ChannelCount) break;
		}
		for(int s=0; s<Mod->CellList[pCell[Cp]].CountSyn; s++){
			if(CRV[Cp].SRV[s].VIn){
				MDC.MoveTo(LastPointX,LastPointY[Ch]);
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(Mod->CellList[pCell[Cp]].Syn[s].I * CRV[Cp].SRV[s].KIn);
				MDC.LineTo(Current);
				LastPointY[Ch++] = Current.y;
				if(Ch>=ChannelCount) break;
			}
			if(CRV[Cp].SRV[s].VSIn){
				MDC.MoveTo(LastPointX,LastPointY[Ch]);
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(Mod->CellList[pCell[Cp]].Syn[s].SingleWeight * CRV[Cp].SRV[s].KSIn);
				MDC.LineTo(Current);
				LastPointY[Ch++] = Current.y;
				if(Ch>=ChannelCount) break;
			}
		}
		if(Ch>=ChannelCount) break;
	}
	LastPointX=Current.x;
	MDC.SelectObject(OPen);
	if(floorf(PrePointCount) >= 20.0f){
		PrePointCount -= 20.0f;
		ViewLeft+=20;
		if(ViewLeft==size.cx)LastPointX=ViewLeft=0;
		ResetEvent(hDrawing);
		Invalidate(false);
		WaitForSingleObject(hDrawing,INFINITE);
		CBrush GridBrsh;
		CBitmap Grid;Grid.LoadBitmap(IDB_GRED);
		GridBrsh.CreatePatternBrush(&Grid); 
		MDC.SelectObject(&pBmp);
		MDC.FillRect(CRect(ViewLeft,0,ViewLeft+20,size.cy),&GridBrsh);
		MDC.MoveTo(0,0); MDC.LineTo(0,20);
	}
}

void PolyChannelView::ReMapFile()
{
	int Count = (BYTE*)pDataLast - (BYTE*)pStartData;
	::FlushViewOfFile(pStartData,Count);
	::UnmapViewOfFile(pStartData);
	pStartData = NULL;
	HANDLE hRecord = ::CreateFile(FileName, GENERIC_READ | GENERIC_WRITE, 0, NULL,
						OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	int FileSize = (int)::GetFileSize(hRecord,NULL)+131072;
	HANDLE hMapRecord = ::CreateFileMapping(hRecord,NULL,PAGE_READWRITE,0,FileSize,NULL);
	::CloseHandle(hRecord);

	pStartData = ::MapViewOfFile(hMapRecord,FILE_MAP_ALL_ACCESS,0,0,0);
	::CloseHandle(hMapRecord);
	pDataLast = (float*)((BYTE*)pStartData+Count);
	MaxFileSize = (BYTE*)pStartData+FileSize;

}


void PolyChannelView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if((cx == 0)&(cy == 0))return;
	size.cx = 20*(int)((float)cx/(float)20)+20; size.cy = cy;
	SizeFlg=true;
	Invalidate(false);
}

void PolyChannelView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(nSBCode == SB_LINELEFT){
		float LeftWale = (float)size.cx*Resolution; 
		RightRecord -= (int)(LeftWale/10.0f) ; 
		if(RightRecord<(int)LeftWale)RightRecord=(int)LeftWale;
		SetScrollPos(SB_HORZ,RightRecord,true);
	}
	if(nSBCode == SB_LINERIGHT){
		RightRecord +=(int)((float)size.cx*Resolution/10.0f);
		if(RightRecord>AllRecords)RightRecord=AllRecords;
		SetScrollPos(SB_HORZ,RightRecord,true);
	}
	if(nSBCode == SB_PAGELEFT){
		float LeftWale = (float)size.cx*Resolution; 
		RightRecord -= (int)(LeftWale/2.0f) ; 
		if(RightRecord<(int)LeftWale)RightRecord=(int)LeftWale;
		SetScrollPos(SB_HORZ,RightRecord,true);
	}
	if(nSBCode == SB_PAGERIGHT){
		RightRecord +=(int)((float)size.cx*Resolution/2.0f);
		if(RightRecord>AllRecords)RightRecord=AllRecords;
		SetScrollPos(SB_HORZ,RightRecord,true);
	}
///	Invalidate(false);///????
}

void PolyChannelView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	if(SizeFlg)FlashScreen(dc);
	dc.BitBlt(0,0,(size.cx - ViewLeft),size.cy,&MDC,ViewLeft,0,SRCCOPY);
	dc.BitBlt((size.cx - ViewLeft),0,size.cx,size.cy,&MDC,0,0,SRCCOPY);
	SetEvent(hDrawing);
}

void PolyChannelView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CWnd::OnMouseMove(nFlags, point);
}

void PolyChannelView::FlashScreen(CDC &dc)
{
	SizeFlg = false;
	pBmp.DeleteObject();
	pBmp.CreateCompatibleBitmap( &dc, size.cx, size.cy);
	ViewLeft=0; ViewRight=size.cx; DrawEnd=size.cx;
	MDC.DeleteDC();
	MDC.CreateCompatibleDC(&dc);
	CBrush GridBrsh;
	CBitmap Grid;Grid.LoadBitmap(IDB_GRED);
	GridBrsh.CreatePatternBrush(&Grid); 
	MDC.SelectObject(&pBmp);
	MDC.FillRect( CRect(0,0,size.cx,size.cy), &GridBrsh);
	ChannelCount = 0;int HightCount = 0;
	for(int Cp=0; Cp<CellCount; Cp++){
		if(CRV[Cp].VP)ChannelCount++;
		if(CRV[Cp].VAP){
			ChannelCount++;
			if(CRV[Cp].AttAP)HightCount++;
		}
		if(CRV[Cp].VTH){
			ChannelCount++;
			if(CRV[Cp].AttTH)HightCount++;
		}
		for(int s=0; s<CRV[Cp].Count; s++){
			if(CRV[Cp].SRV[s].VIn)ChannelCount++;
			if(CRV[Cp].SRV[s].VSIn)ChannelCount++;
		}
	}
	ChannelHight = (int)floorf((float)size.cy/(float)(ChannelCount-HightCount));
	if(LastPointY != NULL) delete[]LastPointY;
	LastPointY = new int[ChannelCount];
	for(int Ch=0; Ch<ChannelCount; Ch++)LastPointY[Ch] = ChannelHight*Ch + ChannelHight/2;
	LastPointX=0;
}


void PolyChannelView::StartRecord()
{
	if(pStartData != NULL) return;
	HANDLE hRecord = ::CreateFile(FileName, GENERIC_READ | GENERIC_WRITE, 0, NULL,
						OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hRecord == INVALID_HANDLE_VALUE)  return;

	HANDLE hMapRecord = ::CreateFileMapping(hRecord,NULL,PAGE_READWRITE,0,131072,NULL);
	::CloseHandle(hRecord);
	if(hMapRecord == INVALID_HANDLE_VALUE) return;

	pStartData = ::MapViewOfFile(hMapRecord,FILE_MAP_ALL_ACCESS,0,0,0);
	::CloseHandle(hMapRecord);
	if(pStartData == NULL) return;
	pDataLast = (float*) pStartData; pDataLast++;
	MaxFileSize = (BYTE*)pStartData+131072;

	*((int*)pDataLast) = CellCount; pDataLast++;
	for(int c=0; c<CellCount; c++){
		*((int*)pDataLast) = pCell[c]; pDataLast++;
		*((int*)pDataLast) = Mod->CellList[pCell[c]].CountSyn; pDataLast++;
	}
	*((int*)pDataLast) = RecordSize; pDataLast++;
	if(chooser.Full)*pDataLast = Mod->CellList->TimeStep;
	else *pDataLast = chooser.m_Time;
	pDataLast++;
	pFirstRecord = pDataLast;
	RecordingFlg = true;
	AllRecords = 0;
}

void PolyChannelView::StopRecord()
{
	if(pStartData == NULL) return;
	*((int*)pStartData) = AllRecords;
	int Count = (BYTE*)pDataLast - (BYTE*)pStartData;
	::FlushViewOfFile(pStartData,Count);
	::UnmapViewOfFile(pStartData);
	pStartData = NULL;
	RecordingFlg = false;
}
