// PolyChannelsMonitor.cpp : implementation file
//

#include "stdafx.h"
#include "NeuroCAD.h"
#include "PolyChannelsMonitor.h"
#include "SimulatorView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PolyChannelsMonitor

IMPLEMENT_DYNCREATE(PolyChannelsMonitor, CFrameWnd)

PolyChannelsMonitor::PolyChannelsMonitor()
{
	pSim = NULL;
}

PolyChannelsMonitor::~PolyChannelsMonitor()
{
	if(pSim != NULL){
		for(int r=0; r<((SimulatorView*)pSim)->PCMCount; r++)
			if(((SimulatorView*)pSim)->PCM[r] == this)
				((SimulatorView*)pSim)->PCM[r] = NULL;
	}
}

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_SEPARATOR,
};

BEGIN_MESSAGE_MAP(PolyChannelsMonitor, CFrameWnd)
	//{{AFX_MSG_MAP(PolyChannelsMonitor)
	ON_WM_CREATE()
	ON_COMMAND(ID_SV_COMUTATOR, OnSvComutator)
	ON_COMMAND(ID_SV_POP_TIME, OnSvPopTime)
	ON_UPDATE_COMMAND_UI(ID_SV_POP_TIME, OnUpdateSvPopTime)
	ON_COMMAND(ID_SV_PUSH_TIME, OnSvPushTime)
	ON_UPDATE_COMMAND_UI(ID_SV_PUSH_TIME, OnUpdateSvPushTime)
	ON_COMMAND(ID_SV_RECORD, OnSvRecord)
	ON_UPDATE_COMMAND_UI(ID_SV_RECORD, OnUpdateSvRecord)
	ON_COMMAND(ID_SV_STOP, OnSvStop)
	ON_UPDATE_COMMAND_UI(ID_SV_STOP, OnUpdateSvStop)
	ON_COMMAND(ID_SV_PAUSE, OnSvPause)
	ON_UPDATE_COMMAND_UI(ID_SV_PAUSE, OnUpdateSvPause)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PolyChannelsMonitor message handlers

int PolyChannelsMonitor::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
//	if(m_wndView.Mod = NULL) return -1;
	
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW|WS_HSCROLL,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_SIGNALVIEW))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;	
}

void PolyChannelsMonitor::AddCellRecord(int NCell)
{
	if(m_wndView.Mod == 0) return;
	if(m_wndView.CellCount == 0) {
		m_wndView.pCell = new int[1];
		m_wndView.pCell[m_wndView.CellCount++] = NCell;
	}
	else{
		int *Tmp = new int[m_wndView.CellCount];
		for(int p=0; p<m_wndView.CellCount; p++) Tmp[p] = m_wndView.pCell[p];
		delete[] m_wndView.pCell;
		m_wndView.pCell = new int[m_wndView.CellCount+1];
		for(p=0; p<m_wndView.CellCount; p++) m_wndView.pCell[p] = Tmp[p];
		delete[] Tmp;
		m_wndView.pCell[m_wndView.CellCount++]=NCell;
	}
}


void PolyChannelsMonitor::OnSvComutator() 
{
	m_wndView.SVC.DoModal();
	m_wndView.SizeFlg = true;
	m_wndView.Invalidate(false);
}


void PolyChannelsMonitor::OnSvPopTime() 
{
	m_wndView.Resolution *= 2;
	if(m_wndView.Resolution>4.0f)m_wndView.Resolution=4.0f;
}

void PolyChannelsMonitor::OnUpdateSvPopTime(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndView.Resolution<4.0f);
}

void PolyChannelsMonitor::OnSvPushTime() 
{
	m_wndView.Resolution /= 2;
	if(m_wndView.RecordingFlg){
		if((int)(m_wndView.Resolution*(float)m_wndView.size.cx)>=m_wndView.AllRecords)
			m_wndView.Resolution=(float)m_wndView.AllRecords/(float)m_wndView.size.cx;
	}
}

void PolyChannelsMonitor::OnUpdateSvPushTime(CCmdUI* pCmdUI) 
{
	if(m_wndView.RecordingFlg)
		pCmdUI->Enable((m_wndView.Resolution*m_wndView.size.cx) < m_wndView.AllRecords);
	else
		pCmdUI->Enable(true);
}


void PolyChannelsMonitor::OnSvRecord() 
{
	CFileDialog FDlg(false,"crf","*.crf",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Cell Record File (*.crf)|*.crf|(*.*)|*.*||");
	if(FDlg.DoModal() != IDOK) return;
	m_wndView.FileName = FDlg.GetPathName();
	m_wndView.StartRecord();
}

void PolyChannelsMonitor::OnUpdateSvRecord(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)(m_wndView.pStartData != NULL));	
}

void PolyChannelsMonitor::OnSvStop() 
{
	m_wndView.StopRecord();	
}

void PolyChannelsMonitor::OnUpdateSvStop(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)(m_wndView.pStartData == NULL));	
}

void PolyChannelsMonitor::OnSvPause() 
{
	m_wndView.RecordingFlg = !m_wndView.RecordingFlg;
}

void PolyChannelsMonitor::OnUpdateSvPause(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)!m_wndView.RecordingFlg);	
}
