// SimulatorView.cpp : implementation file
//

#include "stdafx.h"
#include "math.h"
#include "NeuroCAD.h"
#include "SimulatorView.h"
#include "InputMachin.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SimulatorView

SimulatorView::SimulatorView(void *MF)
{
	Condition  = 0;
	Resolution = 1;
	CurrentPoint.x = CurrentPoint.y=0;
	MaxSize.cx = 0;
	ReDraw = false;
	Bmp.CreateBitmap(1,1,1,1,NULL);
	ViewCell = SelectCell =-1; SelectLink =-1;
	InView = OutView = NumberView = true;
	CV = NULL;
	PCM = NULL;
	PCMCount = 0;
	MR = NULL;
	m_StatusBar = NULL;
	From_1.x =-1;
	Mod = NULL;
	MiniView = true;
	SimRec=NULL;
	MFrm = MF;
}

SimulatorView::~SimulatorView()
{
	if(CV !=NULL) CV->DestroyWindow();
	for(int R=0; R<PCMCount; R++)
		if(PCM[R] !=NULL) PCM[R]->DestroyWindow();
	if(PCM != NULL) delete[] PCM;
	if(Condition >0)OnSimulatorStop();
	if(Mod->SWatcher.CellAccount != NULL){
		for(int c=0; c<Mod->SWatcher.CellCount; c++)
			delete[] Mod->SWatcher.CellAccount[c].SynapsesAccount;
		delete[] Mod->SWatcher.CellAccount;
	}
	if(SimRec != NULL){
		SimRec->EndRecord();
		delete SimRec;
	}
}


BEGIN_MESSAGE_MAP(SimulatorView, CWnd)
	//{{AFX_MSG_MAP(SimulatorView)
	ON_MESSAGE (WM_REVIEW, OnReView)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_SIMULATOR_ITERATION, OnSimulatorIteration)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_ITERATION, OnUpdateSimulatorIteration)
	ON_COMMAND(ID_SIMULATOR_PAUSE, OnSimulatorPause)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_PAUSE, OnUpdateSimulatorPause)
	ON_COMMAND(ID_SIMULATOR_PLAY, OnSimulatorPlay)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_PLAY, OnUpdateSimulatorPlay)
	ON_COMMAND(ID_SIMULATOR_PROPERTY, OnSimulatorProperty)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_PROPERTY, OnUpdateSimulatorProperty)
	ON_COMMAND(ID_SIMULATOR_STEP, OnSimulatorStep)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_STEP, OnUpdateSimulatorStep)
	ON_COMMAND(ID_SIMULATOR_STOP, OnSimulatorStop)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_STOP, OnUpdateSimulatorStop)
	ON_COMMAND(ID_SIMULATOR_SET_SIGNAL, OnSimulatorSetSignal)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_SET_SIGNAL, OnUpdateSimulatorSetSignal)
	ON_COMMAND(ID_RECODER_START, OnRecoderStart)
	ON_UPDATE_COMMAND_UI(ID_RECODER_START, OnUpdateRecoderStart)
	ON_COMMAND(ID_RECODER_PROPERTY, OnRecoderProperty)
	ON_UPDATE_COMMAND_UI(ID_RECODER_PROPERTY, OnUpdateRecoderProperty)
	ON_COMMAND(ID_RECODER_PAUSE, OnRecoderPause)
	ON_UPDATE_COMMAND_UI(ID_RECODER_PAUSE, OnUpdateRecoderPause)
	ON_COMMAND(ID_RECORDER_STOP, OnRecorderStop)
	ON_UPDATE_COMMAND_UI(ID_RECORDER_STOP, OnUpdateRecorderStop)
	ON_COMMAND(ID_SIMULATOR_IM_PAUSE, OnSimulatorImPause)
	ON_COMMAND(ID_SIMULATOR_IM_VIEW, OnSimulatorImView)
	ON_COMMAND(ID_SIMULATOR_IM_INPORT, OnSimulatorImInport)
	ON_COMMAND(ID_SIMULATOR_CODOGRAMM, OnSimulatorCodogramm)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_CODOGRAMM, OnUpdateSimulatorCodogramm)
	ON_COMMAND(ID_SIMULATOR_POLY, OnSimulatorPoly)
	ON_UPDATE_COMMAND_UI(ID_SIMULATOR_POLY, OnUpdateSimulatorPoly)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// SimulatorView message handlers
BOOL SimulatorView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;
	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style |= WS_HSCROLL|WS_VSCROLL;
	cs.lpszClass = AfxRegisterWndClass(CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW),(HBRUSH)(COLOR_WINDOW +2));
	return TRUE;
}

void SimulatorView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	CurSize.cy =cy; CurSize.cx = cx;
	MiniView = ( nType != SIZE_MINIMIZED);
	if(MaxSize.cx == 0){
		MaxSize.cx = CurSize.cx * 10;	MaxSize.cy = CurSize.cy * 10; 
		CurrentPoint.x = MaxSize.cx/2;	CurrentPoint.y = MaxSize.cy/2;
	}
	ReDraw = true;
	DrawView();
}
void SimulatorView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(nSBCode == SB_LINELEFT){
		if((CurrentPoint.x - CurSize.cx/(10*Resolution)) > CurSize.cx/(2*Resolution))
			CurrentPoint.x -= CurSize.cx/(10*Resolution);
		else 
			CurrentPoint.x = CurSize.cx/(2*Resolution);
	}
	if(nSBCode == SB_LINERIGHT){
		CurrentPoint.x += CurSize.cx/(10*Resolution);
		if(CurrentPoint.x + CurSize.cx/(2*Resolution) > MaxSize.cx)
			MaxSize.cx = CurrentPoint.x + CurSize.cx/(Resolution);
	}

	if(nSBCode == SB_PAGELEFT){
		if((CurrentPoint.x - CurSize.cx/(2*Resolution)) > CurSize.cx/(2*Resolution))
			CurrentPoint.x -= CurSize.cx/(2*Resolution);
		else 
			CurrentPoint.x = CurSize.cx/(2*Resolution);
	}
	if(nSBCode == SB_PAGERIGHT){
		CurrentPoint.x += CurSize.cx/(2*Resolution);
		if(CurrentPoint.x + CurSize.cx/(2*Resolution) > MaxSize.cx)
			MaxSize.cx = CurrentPoint.x + CurSize.cx/(Resolution);
	}
	DrawView();
}

void SimulatorView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(nSBCode == SB_LINELEFT){
		if((CurrentPoint.y - CurSize.cy/(10*Resolution)) > CurSize.cy/(2*Resolution))
			CurrentPoint.y -= CurSize.cy/(10*Resolution);
		else 
			CurrentPoint.y = CurSize.cy/(2*Resolution);
	}
	if(nSBCode == SB_LINERIGHT){
		CurrentPoint.y += CurSize.cy/(10*Resolution);
		if(CurrentPoint.y + CurSize.cy/(2*Resolution) > MaxSize.cy)
			MaxSize.cy = CurrentPoint.y + CurSize.cy/(Resolution);
	}

	if(nSBCode == SB_PAGELEFT){
		if((CurrentPoint.y - CurSize.cy/(2*Resolution)) > CurSize.cy/(2*Resolution))
			CurrentPoint.y -= CurSize.cy/(2*Resolution);
		else 
			CurrentPoint.y = CurSize.cy/(2*Resolution);
	}
	if(nSBCode == SB_PAGERIGHT){
		CurrentPoint.y += CurSize.cy/(2*Resolution);
		if(CurrentPoint.y + CurSize.cy/(2*Resolution) > MaxSize.cy)
			MaxSize.cy = CurrentPoint.y + CurSize.cy/(Resolution);
	}	
	DrawView();
}

void SimulatorView::DrawView()
{
//Scroll
	SetScrollRange(SB_HORZ,CurSize.cx/(2*Resolution),MaxSize.cx*Resolution,true);	
	SetScrollPos(SB_HORZ,CurrentPoint.x*Resolution,true);
	SetScrollRange(SB_VERT,CurSize.cy/(2*Resolution),MaxSize.cy*Resolution,true);	
	SetScrollPos(SB_VERT,CurrentPoint.y*Resolution,true);
//	if(MiniView)ReBuild();
	if(MiniView)Invalidate(false);
}
void SimulatorView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	ReBuild(dc);
//	dc.BitBlt(0,0,CurSize.cx, CurSize.cy,&mDC,0,0,SRCCOPY);
}

void SimulatorView::ReView()
{
//	if(MiniView)ReBuild();
	if(!Mod->Stop){
		Mod->StepSimulator();
		Condition = 0;
		Invalidate(false);
		return;
	}

//	if(MiniView)Invalidate(false);
//	OnPaint();
//	Invalidate(false);
	switch(Condition){
	case 2:
		Mod->StepSimulator();
		break;
	case 3:
		Mod->StopSimulator();
		Condition = 0;
		break;
	}
}

void SimulatorView::OnReView(UINT wParam, LONG lParam)
{
//	if(MiniView)ReBuild();
	if(!Mod->Stop){
		Mod->StepSimulator();
		Condition = 0;
		Invalidate(false);
		return;
	}

	if(MiniView)Invalidate(false);
//	if(MiniView)OnPaint();
	switch(Condition){
	case 2:
		Mod->StepSimulator();
		break;
	case 3:
		Mod->StopSimulator();
		Condition = 0;
		break;
	}
}

void SimulatorView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if((Condition == 5)&(From_1.x >=0)){
		CursorPoint = point;
		Invalidate(false);
		return;
	}
	Position RPoint = LPtoMOD(point);
	ViewCell = -1;
	for(int i=0; i<Mod->CurrentCell; i++){
		if((RPoint.x>Mod->VCellList[i].Pos.x-10/Resolution)&
		   (RPoint.x<Mod->VCellList[i].Pos.x+10/Resolution)&
		   (RPoint.y>Mod->VCellList[i].Pos.y-10/Resolution)&
		   (RPoint.y<Mod->VCellList[i].Pos.y+10/Resolution)){
			ViewCell=i;
			break;
		}
	}
	if(ViewCell >=0) Invalidate(false);
}


void SimulatorView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	Position RPoint = LPtoMOD(point);
	SelectCell = -1;
	for(int i=0; i<Mod->CurrentCell; i++){
		if((RPoint.x>Mod->VCellList[i].Pos.x-10/Resolution)&
		   (RPoint.x<Mod->VCellList[i].Pos.x+10/Resolution)&
		   (RPoint.y>Mod->VCellList[i].Pos.y-10/Resolution)&
		   (RPoint.y<Mod->VCellList[i].Pos.y+10/Resolution)){
			SelectCell=i;
			break;
		}
	}
	if((SelectCell>=0)&(Condition == 5)) {
		PCM[PCMCount-1] = new PolyChannelsMonitor;
		PCM[PCMCount-1]->m_wndView.Mod = Mod;
		PCM[PCMCount-1]->pSim = this;
		PCM[PCMCount-1]->AddCellRecord(SelectCell);
		CString str; 
		PCM[PCMCount-1]->Create(NULL,str,
				WS_OVERLAPPEDWINDOW,
				CRect(0,0,240,674),NULL,NULL,0,NULL);
		PCM[PCMCount-1]->ShowWindow(SW_SHOW);
		Condition = 0;
	}
	else if(Condition == 5)
		From_1 = RPoint;


}

void SimulatorView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	Position RPoint = LPtoMOD(point);
	if(Condition == 5){
		PCM[PCMCount-1] = new PolyChannelsMonitor;
		PCM[PCMCount-1]->m_wndView.Mod = Mod;
		PCM[PCMCount-1]->pSim = this;
		Position From_2 = LPtoMOD(point);
		if(From_1.x < From_2.x){
			if(From_1.y < From_2.y){
				for(int C=0; C<Mod->CurrentCell; C++){
					if((Mod->VCellList[C].Pos.x>From_1.x)&(Mod->VCellList[C].Pos.x<From_2.x)&
					   (Mod->VCellList[C].Pos.y>From_1.y)&(Mod->VCellList[C].Pos.y<From_2.y))
					   PCM[PCMCount-1]->AddCellRecord(C);
				}
			}
			else{
				for(int C=0; C<Mod->CurrentCell; C++){
					if((Mod->VCellList[C].Pos.x>From_1.x)&(Mod->VCellList[C].Pos.x<From_2.x)&
					   (Mod->VCellList[C].Pos.y>From_2.y)&(Mod->VCellList[C].Pos.y<From_1.y))
					   PCM[PCMCount-1]->AddCellRecord(C);
				}
			}
		}
		else{
			if(From_1.y < From_2.y){
				for(int C=0; C<Mod->CurrentCell; C++){
					if((Mod->VCellList[C].Pos.x>From_2.x)&(Mod->VCellList[C].Pos.x<From_1.x)&
					   (Mod->VCellList[C].Pos.y>From_1.y)&(Mod->VCellList[C].Pos.y<From_2.y))
					   PCM[PCMCount-1]->AddCellRecord(C);
				}
			}
			else{
				for(int C=0; C<Mod->CurrentCell; C++){
					if((Mod->VCellList[C].Pos.x>From_2.x)&(Mod->VCellList[C].Pos.x<From_1.x)&
					   (Mod->VCellList[C].Pos.y>From_2.y)&(Mod->VCellList[C].Pos.y<From_1.y))
					   PCM[PCMCount-1]->AddCellRecord(C);
				}
			}
		}
		CString str;
		PCM[PCMCount-1]->Create(NULL,str,
				WS_OVERLAPPEDWINDOW,
				CRect(0,0,240,674),NULL,NULL,0,NULL);
		PCM[PCMCount-1]->ShowWindow(SW_SHOW);
		From_1.x=-1;
		Invalidate(false);
		Condition = 0;
	}
}
void SimulatorView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	Position RPoint = LPtoMOD(point);
	if(Condition == 5) return;
	int C = -1;
	for(int i=0; i<Mod->CurrentCell; i++){
		if((RPoint.x>Mod->VCellList[i].Pos.x-10/Resolution)&
		   (RPoint.x<Mod->VCellList[i].Pos.x+10/Resolution)&
		   (RPoint.y>Mod->VCellList[i].Pos.y-10/Resolution)&
		   (RPoint.y<Mod->VCellList[i].Pos.y+10/Resolution)){
			C=i;
			break;
		}
	}
	if(C<0){
		if(CV != NULL){
			if(CV->DestroyWindow())CV=NULL;
			return;
		}
	}
	else{
		if(Mod->VCellList[C].Select>=0x20){
			Mod->IfModul.EditCondition(Mod);
		}
		else if(Mod->VCellList[C].Select>=0x10){
			InputMachin IM;
			IM.IM = &Mod->Input;
			if((IM.DoModal() == IDOK)&(Condition == 0))
				Mod->Input.Assemble(Mod);
			//Mod->Input.Pause = !Mod->Input.Pause;
		}
		else{
			if(CV == NULL){
				CPoint Pos = point; ::ClientToScreen(this->m_hWnd,&Pos);
				CV = new CellView;
				if(Pos.x > 400) Pos.x -= 250;
				if(Pos.y >400) Pos.y -=200; else if(Pos.y>200) Pos.y -=100;
				CV->Create(NULL,NULL,WS_POPUP,CRect(Pos.x,Pos.y,Pos.x+350,Pos.y+700),NULL,0);
				CV->ShowWindow(SW_SHOW);
			}
			CV->pCell = &Mod->CellList[C];
			CV->Invalidate(false);
		}
	}
}

void SimulatorView::OnContextMenu(CWnd* pWnd, CPoint point)
{
	POINT P = (POINT)point; ::ScreenToClient(this->m_hWnd,&P);
	
	Position RPoint = LPtoMOD((CPoint)P);
	if(SelectCell>=0) Mod->VCellList[SelectCell].Select&=0xf0;
	int C = -1;
	for(int i=0; i<Mod->CurrentCell; i++){
		if((RPoint.x>Mod->VCellList[i].Pos.x-10/Resolution)&
		   (RPoint.x<Mod->VCellList[i].Pos.x+10/Resolution)&
		   (RPoint.y>Mod->VCellList[i].Pos.y-10/Resolution)&
		   (RPoint.y<Mod->VCellList[i].Pos.y+10/Resolution)){
			C=i;
			break;
		}
	}
	if(C<0) return;
	if(Mod->VCellList[SelectCell].Select>=0x10) {
		CMenu menu;
		menu.LoadMenu(IDR_SIMULATOR_M);
		menu.GetSubMenu(1)->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON, point.x, point.y, this);   
	}
}
/*===========================================================================================*/
Position SimulatorView::LPtoMOD(CPoint point)
{
	return(	Position(CurrentPoint.x+(2*point.x-CurSize.cx)/(2*Resolution),
					 CurrentPoint.y+(2*point.y-CurSize.cy)/(2*Resolution)));
}
CPoint SimulatorView::MODtoLP(Position point)
{
	return(
		CPoint((point.x - CurrentPoint.x) * Resolution + CurSize.cx/2,
			   (point.y - CurrentPoint.y) * Resolution + CurSize.cy/2));
}


void SimulatorView::DrawSynapse(CDC &dc, int &From, int &To, int Select)
{
	if(Mod->ConMap[From][To].Current == 0) return;//???
	Position Res;
	COLORREF Synapse, SynapseA;
	if(Mod->CellList[From].Axon)	{Synapse = RGB(180,255,0);SynapseA =RGB(255,255,255);}
	else							{Synapse = RGB(255,50,50);SynapseA =RGB(255,255,0);}
	CPen	SynPen(PS_SOLID,(Resolution<1)?1:Resolution, Synapse); 
	CBrush	SynBrs(Synapse);
	CPen	SynPenA(PS_SOLID,(Resolution<1)?1:Resolution, SynapseA); 
	CBrush	SynBrsA(SynapseA);

	double Kx=((double)Mod->VCellList[From].Pos.y - (double)Mod->VCellList[To].Pos.y)/
		      ((double)Mod->VCellList[From].Pos.x - (double)Mod->VCellList[To].Pos.x);
	double Ky=((double)Mod->VCellList[From].Pos.x - (double)Mod->VCellList[To].Pos.x)/
		      ((double)Mod->VCellList[From].Pos.y - (double)Mod->VCellList[To].Pos.y);
	const double R=10;
	double dx, dy;
	if((Mod->ConMap[From][To].Current == 1)&(Mod->ConMap[To][From].Current == 0)){
		dx = (Mod->VCellList[From].Pos.x > Mod->VCellList[To].Pos.x) ? 
							R/sqrt(1+Kx*Kx): -R/sqrt(1+Kx*Kx);
		dy = (Mod->VCellList[From].Pos.y > Mod->VCellList[To].Pos.y) ? 
							R/sqrt(1+Ky*Ky): -R/sqrt(1+Ky*Ky);
		Res.x=Mod->VCellList[To].Pos.x + (int)dx;		  
		Res.y=Mod->VCellList[To].Pos.y + (int)dy;		  
		Position Blo(Res.x-2,Res.y-2), Tlo(Res.x+2,Res.y+2);

		if((To == SelectCell)&(Mod->ConMap[From][To].List[0] == SelectLink)){
			dc.SelectObject(&SynPenA); dc.SelectObject(&SynBrsA);
		}
		else{
			dc.SelectObject(&SynPen); dc.SelectObject(&SynBrs);
		}
		dc.MoveTo(MODtoLP(Mod->VCellList[From].Pos));
		dc.LineTo(MODtoLP(Res));
		dc.Ellipse(MODtoLP(Tlo).x,MODtoLP(Tlo).y,MODtoLP(Blo).x,MODtoLP(Blo).y);
	}
	else if((Mod->ConMap[From][To].Current == 1)&(Mod->ConMap[To][From].Current >= 1)){
		dx = (Mod->VCellList[From].Pos.x > Mod->VCellList[To].Pos.x) ? 
							R/sqrt(1+Ky*Ky): -R/sqrt(1+Ky*Ky);
		dy = (Mod->VCellList[From].Pos.y > Mod->VCellList[To].Pos.y) ? 
							R/sqrt(1+Kx*Kx): -R/sqrt(1+Kx*Kx);
		Res.x=Mod->VCellList[To].Pos.x + (int)dx;		  
		Res.y=Mod->VCellList[To].Pos.y + (int)dy;		  
		Position Blo(Res.x-2,Res.y-2), Tlo(Res.x+2,Res.y+2);

		if((To == SelectCell)&(Mod->ConMap[From][To].List[0] == SelectLink)){
			dc.SelectObject(&SynPenA); dc.SelectObject(&SynBrsA);
		}
		else{
			dc.SelectObject(&SynPen); dc.SelectObject(&SynBrs);
		}
		dc.MoveTo(MODtoLP(Mod->VCellList[From].Pos));
		dc.LineTo(MODtoLP(Res));
		dc.Ellipse(MODtoLP(Tlo).x,MODtoLP(Tlo).y,MODtoLP(Blo).x,MODtoLP(Blo).y);
	}
	else if((Mod->ConMap[From][To].Current > 1)&(Mod->ConMap[To][From].Current == 0)){
		for(int S=0; S<Mod->ConMap[From][To].Current; S++){
			dx = (Mod->VCellList[From].Pos.x > Mod->VCellList[To].Pos.x) ? 
								(S*4+R)/sqrt(1+Kx*Kx): -(S*4+R)/sqrt(1+Kx*Kx);
			dy = (Mod->VCellList[From].Pos.y > Mod->VCellList[To].Pos.y) ? 
								(S*4+R)/sqrt(1+Ky*Ky): -(S*4+R)/sqrt(1+Ky*Ky);
			Res.x=Mod->VCellList[To].Pos.x + (int)dx;		  
			Res.y=Mod->VCellList[To].Pos.y + (int)dy;		  
			Position Blo(Res.x-2,Res.y-2), Tlo(Res.x+2,Res.y+2);

			if((To == SelectCell)&(Mod->ConMap[From][To].List[S] == SelectLink)){
				dc.SelectObject(&SynPenA); dc.SelectObject(&SynBrsA);
			}
			else{
				dc.SelectObject(&SynPen); dc.SelectObject(&SynBrs);
			}
			dc.MoveTo(MODtoLP(Mod->VCellList[From].Pos));
			dc.LineTo(MODtoLP(Res));
			dc.Ellipse(MODtoLP(Tlo).x,MODtoLP(Tlo).y,MODtoLP(Blo).x,MODtoLP(Blo).y);
		}
	}
	else if((Mod->ConMap[From][To].Current > 1)&(Mod->ConMap[To][From].Current >=1)){
		for(int S=0; S<Mod->ConMap[From][To].Current; S++){
			dx = (Mod->VCellList[From].Pos.x > Mod->VCellList[To].Pos.x) ? 
								(S*4+R)/sqrt(1+Ky*Ky): -(S*4+R)/sqrt(1+Ky*Ky);
			dy = (Mod->VCellList[From].Pos.y > Mod->VCellList[To].Pos.y) ? 
								(S*4+R)/sqrt(1+Kx*Kx): -(S*4+R)/sqrt(1+Kx*Kx);
			Res.x=Mod->VCellList[To].Pos.x + (int)dx;		  
			Res.y=Mod->VCellList[To].Pos.y + (int)dy;		  
			Position Blo(Res.x-2,Res.y-2), Tlo(Res.x+2,Res.y+2);

			if((To == SelectCell)&(Mod->ConMap[From][To].List[S] == SelectLink)){
				dc.SelectObject(&SynPenA); dc.SelectObject(&SynBrsA);
			}
			else{
				dc.SelectObject(&SynPen); dc.SelectObject(&SynBrs);
			}
			dc.MoveTo(MODtoLP(Mod->VCellList[From].Pos));
			dc.LineTo(MODtoLP(Res));
			dc.Ellipse(MODtoLP(Tlo).x,MODtoLP(Tlo).y,MODtoLP(Blo).x,MODtoLP(Blo).y);
		}
	}
}

void SimulatorView::ReBuild(CDC &dc)
{
	//Draw Cells
	CPoint Ziro(CurrentPoint.x - CurSize.cx/(2*Resolution),
		        CurrentPoint.y - CurSize.cy/(2*Resolution));
	CRect RWnd(CurrentPoint.x - CurSize.cx/(2*Resolution)-7,
		       CurrentPoint.y - CurSize.cy/(2*Resolution)-7,
		       CurrentPoint.x + CurSize.cx/(2*Resolution)+7,
			   CurrentPoint.y + CurSize.cy/(2*Resolution)+7);
	
	CPen	CellPen(PS_SOLID,(Resolution<1)?1:Resolution, RGB(0,0,0));
	CBrush	TBR(RGB(255,255,255));
	COLORREF CellREF, InREF;
		
	dc.SelectObject(&Bmp);
	dc.FillSolidRect(CRect(0,0,CurSize.cx,CurSize.cy),RGB(110,110,110));

	CPen *OldPen = dc.SelectObject(&CellPen); CBrush *OldBrs=dc.SelectObject(&TBR);
	dc.SetTextColor(RGB(255,255,255));
	CPoint Pos;
	CString str; 
	if(Mod == NULL) return;
	for(int i=0; i<Mod->CurrentCell; i++){
		Pos = MODtoLP(Mod->VCellList[i].Pos);
		if(Mod->VCellList[i].Select<3){
			if(Mod->CellList[i].Axon)
				CellREF = RGB(180,255,0);
			else if(Mod->CellList[i].Potentioal>=0)
				CellREF = 
					RGB((int)((double)255*Mod->CellList[i].Potentioal/Mod->CellList[i].Threshold),0,0);
			else{
				int gp=(int)((double)255*Mod->CellList[i].Potentioal/(-5e-3));
				CellREF = RGB(0,0,(gp>255) ? 255:gp);
			}

			CBrush	CellBrs(CellREF);
			dc.SelectObject(&CellPen);		dc.SelectObject(&CellBrs);		
			dc.Ellipse(Pos.x-10*Resolution,Pos.y-10*Resolution,
			            Pos.x+10*Resolution,Pos.y+10*Resolution);
		}
		else if((Mod->VCellList[i].Select>=0x20)&(Mod->VCellList[i].Select<0x23)){
			if(Mod->Input.Pause)			InREF = RGB(50,50,50);
			else if(Mod->CellList[i].Axon)	InREF = RGB(180,255,0);
			else							InREF = RGB(180,50,180);
			CBrush	InBrs(InREF);
			CPoint Pol[4];
			dc.SelectObject(&CellPen);		dc.SelectObject(&InBrs);
			Pol[0]=CPoint(Pos.x,Pos.y-20*Resolution);
			Pol[2]=CPoint(Pos.x,Pos.y+20*Resolution);
			Pol[1]=CPoint(Pos.x-20*Resolution,Pos.y);
			Pol[3]=CPoint(Pos.x+20*Resolution,Pos.y);
			dc.Polygon(Pol, 4 );
			if(NumberView){
				str.Format("%d",Mod->IfModul.GetNumberInput(i));
				dc.TextOut(Pos.x-18*Resolution,Pos.y-8*Resolution,str);
			}
		}
		else if((Mod->VCellList[i].Select>=0x10)&(Mod->VCellList[i].Select<0x13)){
			if(Mod->Input.Pause)			InREF = RGB(50,50,50);
			else if(Mod->CellList[i].Axon)	InREF = RGB(180,255,0);
			else							InREF = RGB(180,50,180);
			CBrush	InBrs(InREF);
			dc.SelectObject(&CellPen);		dc.SelectObject(&InBrs);		
			dc.Rectangle(Pos.x-20*Resolution,Pos.y-10*Resolution,
			              Pos.x+20*Resolution,Pos.y+10*Resolution);
			if(NumberView){
				str.Format("%d",Mod->Input.GetNumberInput(i));
				dc.TextOut(Pos.x-18*Resolution,Pos.y-8*Resolution,str);
			}
		}
		for(int To=0; To<Mod->CurrentCell; To++){
			if((Mod->ConMap[i][To].Current>0)&OutView&Mod->CellList[i].Axon){
				DrawSynapse(dc,i,To,((SelectCell == i)|(SelectCell == To)) ? 1:0);
			}
			if((Mod->ConMap[To][i].Current>0)&InView){
				DrawSynapse(dc,To,i,((SelectCell == i)|(SelectCell == To)) ? 1:0);
			}
		}

	}
	if((Condition == 5)& (From_1.x >=0)){
		dc.MoveTo(MODtoLP(From_1));
		dc.LineTo(MODtoLP(From_1).x, CursorPoint.y);
		dc.LineTo(CursorPoint);
		dc.LineTo(CursorPoint.x, MODtoLP(From_1).y);
		dc.LineTo(MODtoLP(From_1));
	}

	str.Format("Current Time: %f(sec.)",Mod->CellList[0].CurrentTime);
	m_StatusBar->SetPaneText(1,str,true);
	
	dc.SelectObject(OldPen);
	dc.SelectObject(OldBrs);
	if(CV != NULL) CV->Invalidate(false);
	if(ViewCell>=0){
		CString	str; str.Format("Celect Cell: %d, V:%f, Vap:%f, Vth:%f",
					ViewCell,
					(float)(Mod->CellList[ViewCell].Potentioal+
								(double)Mod->CellList[ViewCell].Axon*0.1),
					(float)Mod->CellList[ViewCell].APPotential,
					(float)Mod->CellList[ViewCell].ThresholdLevel);//ThesholdLevel
		m_StatusBar->SetPaneText(0,str,true);
	}

}

/*===========================================================================================*/

void SimulatorView::OnSimulatorIteration() 
{
	switch(Condition){
	case 0:
		Mod->CellList[0].Calculate();
		Mod->Input.Calculate(Mod);
		Invalidate(false);
		break;
	}

}

void SimulatorView::OnUpdateSimulatorIteration(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void SimulatorView::OnSimulatorPause() 
{
	// TODO: Add your command handler code here
	
}

void SimulatorView::OnUpdateSimulatorPause(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void SimulatorView::OnSimulatorPlay() 
{
	if(Condition == 2) return;
	if(Condition == 4){
		Mod->StepSimulator();
		Condition = 2;
		return;
	}
	Condition = 2;
	Mod->StartSimulator(true,0.001,this);
}

void SimulatorView::OnUpdateSimulatorPlay(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void SimulatorView::OnSimulatorProperty() 
{
	// TODO: Add your command handler code here
	
}

void SimulatorView::OnUpdateSimulatorProperty(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void SimulatorView::OnSimulatorStep() 
{
	if(Condition == 2) Condition = 4;
	if(Condition == 0){
		Mod->StartSimulator(true, 0.001,this);
		Condition = 4;
	}
	if(Condition == 4)
		Mod->StepSimulator();
	
}

void SimulatorView::OnUpdateSimulatorStep(CCmdUI* pCmdUI) 
{
}

void SimulatorView::OnSimulatorStop() 
{
	switch(Condition){
	case 2:
		Condition =3;
		break;
	case 4:
		Condition =3;
		Mod->StepSimulator();
		break;
	}
}

void SimulatorView::OnUpdateSimulatorStop(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void SimulatorView::OnSimulatorSetSignal() 
{
	if(PCM == NULL){
		PCM = new PolyChannelsMonitor*[PCMCount = 1];
		PCM[0] = NULL;
	}
	else if(PCM[PCMCount-1] != NULL){
			PolyChannelsMonitor **Tmp = new PolyChannelsMonitor*[PCMCount];
			for(int r=0; r<PCMCount; r++)Tmp[r] = PCM[r];
			delete[] PCM;
			PCM = new PolyChannelsMonitor*[PCMCount + 1];
			for(r=0; r<PCMCount; r++)	PCM[r] = Tmp[r];
			PCM[PCMCount++] = NULL;
			delete[] Tmp;
	}

	
	if(Condition == 0)	Condition = 5;
	else Condition = 0;
}

void SimulatorView::OnUpdateSimulatorSetSignal(CCmdUI* pCmdUI) 
{
	if((Condition == 0)|(Condition == 5)) pCmdUI->Enable(true);
	else pCmdUI->Enable(false);
	if(Condition == 5) pCmdUI->SetCheck(1);
	else pCmdUI->SetCheck(0);
}

void SimulatorView::OnRecoderProperty() 
{
	if(MR != NULL) MR->RecodePro();	
}

void SimulatorView::OnUpdateRecoderProperty(CCmdUI* pCmdUI) 
{
	if(MR == NULL)pCmdUI->Enable(false);
}

void SimulatorView::OnRecoderStart() 
{
	if(MR != NULL)  return;
	CFileDialog FDlg(true,"mrf","*.mrf",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Model Recod File(*.mrf)|*.mrf|(*.*)|*.*||");
	if(FDlg.DoModal() == IDOK)
		MR = new ModelRecoder((LPCTSTR) FDlg.GetPathName(), Mod);
}

void SimulatorView::OnUpdateRecoderStart(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)(MR != NULL));
}

void SimulatorView::OnRecoderPause() 
{
	if(MR!=NULL) MR->Pause();
}

void SimulatorView::OnUpdateRecoderPause(CCmdUI* pCmdUI) 
{
	if(MR == NULL)pCmdUI->Enable(false);
	else{
		pCmdUI->Enable(true);
		pCmdUI->SetCheck(MR->PauseFlg);
	}
}

void SimulatorView::OnRecorderStop() 
{
	if(MR == NULL)return;
	delete MR; MR = NULL;
}

void SimulatorView::OnUpdateRecorderStop(CCmdUI* pCmdUI) 
{
	if(MR == NULL) pCmdUI->Enable(false); else pCmdUI->Enable(true);
}

void SimulatorView::OnSimulatorImPause() 
{
	Mod->Input.Pause = !Mod->Input.Pause;
}

void SimulatorView::OnSimulatorImView() 
{
	InputMachin IM;
	IM.IM = &Mod->Input;
	if((IM.DoModal() == IDOK)&(Condition == 0))
		Mod->Input.Assemble(Mod);
}

void SimulatorView::OnSimulatorImInport() 
{
	CFile File;
	CFileDialog FDlg(true,"im","*.im",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"InputMachin(*.im)|*.im|(*.*)|*.*||");
	if(FDlg.DoModal() == IDOK){
		File.Open((LPCTSTR) FDlg.GetPathName(),CFile::modeRead,NULL);
		Mod->Input.ImportMachine(File);
		File.Close();
	}
}

void SimulatorView::OnSimulatorCodogramm() 
{
	if(Mod->SWatcher.WriteFlg)
		Mod->SWatcher.StopCodoGramme();
	else
		Mod->SWatcher.StartCodoGramme();
}

void SimulatorView::OnUpdateSimulatorCodogramm(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)Mod->SWatcher.WriteFlg);
}



void SimulatorView::OnSimulatorPoly() 
{
	if(SimRec ==NULL){
		CFileDialog FDlg(false,"sim","*.sim",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
						"Simulation File(*.sim)|*.sim|(*.*)|*.*||");
		if(FDlg.DoModal() != IDOK) return;
		SimRec = new SimRecoder(Mod);
		CString Title;((CMainFrame*)MFrm)->GetWindowText(Title);
		SimRec->CreateRecord(Title.GetBuffer(Title.GetLength()+1),
			FDlg.GetPathName().GetBuffer(FDlg.GetPathName().GetLength()));
		return;
	}
	SimRec->EndRecord();
	delete SimRec;
	SimRec = NULL;
}

void SimulatorView::OnUpdateSimulatorPoly(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)(SimRec != NULL));
	
}
