// StatWatcher.cpp: implementation of the StatWatcher class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NeuroCAD.h"
#include "StatWatcher.h"
#include "Model.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

StatWatcher::StatWatcher()
{
	Mod = NULL; CellPointer = NULL;
	PatternAccount = NULL;
	FullSpikeCount = 0;
	CellCount = 0;
	ReportFileName.Format("StatisticalReport.txt");
	Setter.FileName = ReportFileName;
	ReportTime = 0.2f;
	CellAccount = NULL;

	WriteFlg = false;
}

StatWatcher::~StatWatcher()
{
	if(CellPointer != NULL) delete[] CellPointer;
	if(WriteFlg) CodFile.Close();
	if(CellAccount != NULL){
		for(int c=0; c<CellCount; c++){
			if(CellAccount[c].SynapsesAccount != NULL)
				delete [] CellAccount[c].SynapsesAccount;
		}
		delete [] CellAccount;
	}
}

void StatWatcher::AddCell(int Number)
{
	if(CellPointer == NULL)	CellPointer = new int[++CellCount];
	else{
		for(int c=0; c<CellCount; c++)
			if(CellPointer[c] == Number) return;
		int *New = new int[++CellCount];
		for(c=0; c<CellCount-1; c++)
			New[c] = CellPointer[c];
		delete[] CellPointer;
		CellPointer = New;
	}
	CellPointer[CellCount-1]=Number;
}

bool StatWatcher::DeleteCell(int Number)
{
	if(CellPointer == NULL) return false;
	int Find = -1;
	for(int i=0; i<CellCount; i++)
		if(CellPointer[i] == Number){
			Find = i;
			break;
		}
	if(Find<0)return false;
	CellCount--;
	for(i=Find; i<CellCount; i++) CellPointer[i]=CellPointer[i+1];
	return true;
}

bool StatWatcher::Assemble(void *md)
{
	Mod = md;
	if((CellPointer == NULL)|(CellCount == 0)) return false;
	Setter.m_STime = ReportTime;
	Setter.DoModal();
	ReportFileName = Setter.FileName;
	ReportTime = Setter.m_STime;
	CellAccount = new CellAccountStr[CellCount];
	CFile File;
	if(!File.Open(ReportFileName, CFile::modeWrite, NULL))
		File.Open(ReportFileName, CFile::modeCreate|CFile::modeWrite, NULL);
	File.SeekToEnd();
	char String[10000];int p=0;
	p+=	_snprintf(String,10000,"\t");
	for(int c=0; c<CellCount; c++){
		CellAccount[c].SynapsesAccount = 
			new SynapsesAccountStr[((Model*)Mod)->CellList[CellPointer[c]].CountSyn];
		p+=	_snprintf(&String[p],10000-p,"%d\t\t",CellPointer[c]);
		for(int s=0; s<((Model*)Mod)->CellList[CellPointer[c]].CountSyn; s++){
			CellAccount[c].SynapsesAccount[s].AverageInjection = 
				CellAccount[c].SynapsesAccount[s].AverageSpeed = 0;
					CellAccount[c].SynapsesAccount[s].LastWeight = 
						((Model*)Mod)->CellList[CellPointer[c]].Syn[s].SingleWeight;
			CellAccount[c].SynapsesAccount[s].CountInjection =
				CellAccount[c].SynapsesAccount[s].CountSpeed = 0;
			p+=	_snprintf(&String[p],10000-p,"\t\t");
		}
		CellAccount[c].AveragePotential = 0;
		CellAccount[c].CountPotential = CellAccount[c].SpikeCount = 0;
	}
	CurrentTime = ReportTime-0.05;
	AverageTotalInjection = AverageTotalSpeed = AverageTotalPotential = 0;
	CountTotalInjection = CountTotalSpeed = CountTotalPotential = 0;
	FullSpikeCount = 0;
	p+=	_snprintf(&String[p],10000-p,"\n");
	File.Write(String, p); File.Close();
	return true;

}

void StatWatcher::Activate(int Level, int Patern)
{
	if(WriteFlg){
		char Out[100];
		CodFile.Write(&Out,sprintf(Out,"I\t%d\t%d\t%f\n",
			Level,Patern,((Model*)Mod)->CellList->CurrentTime));
	}
}

void StatWatcher::DeActivate(int Level, int Pattern)
{
}

void StatWatcher::Recode()
{
	if(WriteFlg){
		for(int c=0; c<(((Model*)Mod)->CurrentCell); c++){
			char Out[100];
			if((((Model*)Mod)->CellList[c].Fireing<=((Model*)Mod)->CellList->TimeStep*1.5)&
				(((Model*)Mod)->VCellList[c].Select<0x10)){
				CodFile.Write(&Out,sprintf(Out,"C\t%d\t%f\t",//\n",
									c,((Model*)Mod)->CellList->CurrentTime));
				CodFile.Write(&Out,sprintf(Out," %d %5.4f %5.4f %5.4f %5.4f\n", c,
					(float)((Model*)Mod)->CellList[c].Potentioal,
					(float)((Model*)Mod)->CellList[c].APPotential,
					(float)((Model*)Mod)->CellList[c].ThresholdLevel,
					(float)((Model*)Mod)->CellList[c].SynK));
			}
		}
	}
	if((CellPointer == NULL)|(CellCount == 0)) return;
	CurrentTime+=((Model*)Mod)->CellList->TimeStep;
	if(CurrentTime >= ReportTime){
		CFile File;
		if(!File.Open(ReportFileName, CFile::modeWrite, NULL))
			File.Open(ReportFileName, CFile::modeCreate|CFile::modeWrite, NULL);
		File.SeekToEnd();
		char String[10000];

		AverageTotalInjection = AverageTotalSpeed = AverageTotalPotential = 0;
		CountTotalInjection = CountTotalSpeed = CountTotalPotential = 0;
		FullSpikeCount = 0;

		int p=_snprintf(String,10000,"%f\t",((Model*)Mod)->CellList->CurrentTime);
		for(int c=0; c<CellCount; c++){
/*			if((Setter.m_SType == 0)|(Setter.m_SType == 2)|(Setter.m_SType == 3)){
				p+=_snprintf(&String[p],10000-p,"%f\t%f\t", 
					(float)CellAccount[c].AveragePotential/(float)CellAccount[c].CountPotential,
					((float)CellAccount[c].SpikeCount)/(float)ReportTime);
			}

			AverageTotalPotential += CellAccount[c].AveragePotential;
			CountTotalPotential += CellAccount[c].CountPotential;
			FullSpikeCount += CellAccount[c].SpikeCount;

			CellAccount[c].AveragePotential = 0;
			CellAccount[c].CountPotential = CellAccount[c].SpikeCount = 0;
*/			for(int s=0; s<((Model*)Mod)->CellList[CellPointer[c]].CountSyn; s++){
/*				if((Setter.m_SType == 0)|(Setter.m_SType == 1)|(Setter.m_SType == 3)){
					p+=_snprintf(&String[p],10000-p,"%E\t%E\t", 
						CellAccount[c].SynapsesAccount[s].AverageInjection/
							(double)CellAccount[c].SynapsesAccount[s].CountInjection,
						CellAccount[c].SynapsesAccount[s].AverageSpeed/
							(double)CellAccount[c].SynapsesAccount[s].CountSpeed);
*/				p+=_snprintf(&String[p],10000-p,"%E\t", 
						CellAccount[c].SynapsesAccount[s].AverageInjection/
							(double)CellAccount[c].SynapsesAccount[s].CountInjection);
/*				}
				AverageTotalInjection += CellAccount[c].SynapsesAccount[s].AverageInjection;
				CountTotalInjection +=CellAccount[c].SynapsesAccount[s].CountInjection;
				AverageTotalSpeed += CellAccount[c].SynapsesAccount[s].AverageSpeed;
				CountTotalSpeed += CellAccount[c].SynapsesAccount[s].CountSpeed;

					CellAccount[c].SynapsesAccount[s].AverageInjection = 
						CellAccount[c].SynapsesAccount[s].AverageSpeed = 0;
					CellAccount[c].SynapsesAccount[s].CountSpeed = 
						CellAccount[c].SynapsesAccount[s].CountInjection = 0;
*/			}
		}
/*		if((Setter.m_SType == 0)|(Setter.m_SType == 4)){
			p+=_snprintf(&String[p],10000-p,"%f\t%f\t%E\t%E", 
				AverageTotalPotential/(double)CountTotalPotential,
				((double)FullSpikeCount)/ReportTime,
				AverageTotalInjection /(double)CountTotalInjection,
				AverageTotalSpeed/(double)CountTotalSpeed);
		}
*/		p+=_snprintf(&String[p],10000-p,"\n");
		File.Write(String,p);
		File.Close();
		CurrentTime = 0;
	}
	else{
		for(int c=0; c<CellCount; c++){
			CellAccount[c].AveragePotential+=
				((Model*)Mod)->CellList[CellPointer[c]].Potentioal+
				(double)((Model*)Mod)->CellList[CellPointer[c]].Axon*
				0.2*((Model*)Mod)->CellList->TimeStep/FireTime;
			CellAccount[c].CountPotential++;
			if(((Model*)Mod)->CellList[CellPointer[c]].Fireing<=
				((Model*)Mod)->CellList->TimeStep*1.5){
				CellAccount[c].SpikeCount++;
				//Add here code for Claster
			}
			for(int s=0; s<((Model*)Mod)->CellList[CellPointer[c]].CountSyn; s++){
				CellAccount[c].SynapsesAccount[s].AverageInjection+=
					((Model*)Mod)->CellList[CellPointer[c]].Syn[s].SingleWeight;
				CellAccount[c].SynapsesAccount[s].CountInjection++;
				CellAccount[c].SynapsesAccount[s].AverageSpeed+=
					(((Model*)Mod)->CellList[CellPointer[c]].Syn[s].SingleWeight - 
					CellAccount[c].SynapsesAccount[s].LastWeight)/
					((Model*)Mod)->CellList->TimeStep;
				CellAccount[c].SynapsesAccount[s].CountSpeed++;
				CellAccount[c].SynapsesAccount[s].LastWeight = 
					((Model*)Mod)->CellList[CellPointer[c]].Syn[s].SingleWeight;
			}
		}
	}
}

void StatWatcher::StartCodoGramme()
{
	if(WriteFlg) return;
	CFileDialog FDlg(true,"cod","*.cod",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Codogramme(*.cod)|*.cod|(*.*)|*.*||");
	if(FDlg.DoModal() == IDOK){
		if(!CodFile.Open((LPCTSTR) FDlg.GetPathName(),CFile::modeWrite,NULL)){
			if(CodFile.Open((LPCTSTR) FDlg.GetPathName(),CFile::modeCreate|CFile::modeWrite,NULL))
				WriteFlg = true;
		}
		else{
			CodFile.SeekToEnd();
			WriteFlg = true;
		}
	}
}
void StatWatcher::StopCodoGramme()
{
	if(!WriteFlg) return;
	CodFile.Close();
	WriteFlg = false;
}
