// SignalView.cpp : implementation of the SignalView class
//

#include "stdafx.h"
#include "Poly Channel Viewer.h"
#include "SignalView.h"
#include "math.h"
#include "MainFrm.h"
#include "WriteBmp.h"
#include "SetTimePoint.h"
#include "WavCreator.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SignalView

SignalView::SignalView()
{
	CellCount = 0;
	pStartData = NULL;
	MaxFileSize = 0;
	pBmp.CreateBitmap(1,1,1,32,NULL);
	ReBuildFlg = false;
	OldBmp = NULL;

	ColBack = RGB(0,0,0);
	ColLine = RGB(255,255,50);
	ColSelect = RGB(100,150,20);
	ColGrid = RGB(150,0,0);
	ColAP = RGB(100,240,100);
	ColTH = RGB(255,255,255);
	ColMark = RGB(255,100,100);

	GridFlg = true;
	AllRecords = 0;
	RecordSize = 0;
	
	Min = Max = NULL;
	LoadFileFlg = false;

	CRV = NULL;

	ReDraw = false;
	GridFlg = true;
	Resolution = 1e3f;
	LeftRecord = 0;
	SelectChannel = 2;
	Marker = NULL;
	BWCreate = false;
	TimeStep = 1e5;
	Magnet = false;
	HVFlag = false;
}

SignalView::~SignalView()
{
	if(CRV != NULL){
		for(int c=0; c<CellCount; c++)
			if(CRV[c].SRV != NULL) delete[] CRV[c].SRV;
		delete[] CRV;
	}
	MDC.DeleteDC();
	if(Min != NULL) delete[] Min;
	if(Max != NULL) delete[]Max;
	if(pStartData != NULL){
		::UnmapViewOfFile(pStartData);
		pStartData = NULL;
	}
}


BEGIN_MESSAGE_MAP(SignalView,CWnd )
	//{{AFX_MSG_MAP(SignalView)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_POP, OnPop)
	ON_UPDATE_COMMAND_UI(ID_POP, OnUpdatePop)
	ON_COMMAND(ID_PUSH, OnPush)
	ON_UPDATE_COMMAND_UI(ID_PUSH, OnUpdatePush)
	ON_COMMAND(ID_SEEK_HOME, OnSeekHome)
	ON_UPDATE_COMMAND_UI(ID_SEEK_HOME, OnUpdateSeekHome)
	ON_COMMAND(ID_SEEK_END, OnSeekEnd)
	ON_UPDATE_COMMAND_UI(ID_SEEK_END, OnUpdateSeekEnd)
	ON_COMMAND(ID_REFRESH, OnRefresh)
	ON_COMMAND(ID_CENTRE, OnCentre)
	ON_UPDATE_COMMAND_UI(ID_CENTRE, OnUpdateCentre)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_COMMAND(ID_GRID_VIEW, OnGridView)
	ON_UPDATE_COMMAND_UI(ID_GRID_VIEW, OnUpdateGridView)
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_UP, OnUp)
	ON_UPDATE_COMMAND_UI(ID_UP, OnUpdateUp)
	ON_COMMAND(ID_DOWN, OnDown)
	ON_UPDATE_COMMAND_UI(ID_DOWN, OnUpdateDown)
	ON_COMMAND(ID_BMP, OnBmp)
	ON_UPDATE_COMMAND_UI(ID_BMP, OnUpdateBmp)
	ON_COMMAND(ID_MARKER, OnMarker)
	ON_UPDATE_COMMAND_UI(ID_MARKER, OnUpdateMarker)
	ON_COMMAND(ID_MARGNET, OnMargnet)
	ON_UPDATE_COMMAND_UI(ID_MARGNET, OnUpdateMargnet)
	ON_COMMAND(ID_SEEK_MARK, OnSeekMark)
	ON_UPDATE_COMMAND_UI(ID_SEEK_MARK, OnUpdateSeekMark)
	ON_COMMAND(ID_END, OnEnd)
	ON_COMMAND(ID_HOME, OnHome)
	ON_COMMAND(ID_LINE_DOWN, OnLineDown)
	ON_COMMAND(ID_LINE_LEFT, OnLineLeft)
	ON_COMMAND(ID_LINE_RIGHT, OnLineRight)
	ON_COMMAND(ID_LINE_UP, OnLineUp)
	ON_COMMAND(ID_PADGE_UP, OnPadgeUp)
	ON_COMMAND(ID_PAGE_DOWN, OnPageDown)
	ON_COMMAND(ID_SEEK_TIME, OnSeekTime)
	ON_UPDATE_COMMAND_UI(ID_SEEK_TIME, OnUpdateSeekTime)
	ON_UPDATE_COMMAND_UI(ID_REFRESH, OnUpdateRefresh)
	ON_COMMAND(ID_WAV, OnWav)
	ON_UPDATE_COMMAND_UI(ID_WAV, OnUpdateWav)
	ON_COMMAND(ID_HIGHVIEW, OnHighview)
	ON_UPDATE_COMMAND_UI(ID_HIGHVIEW, OnUpdateHighview)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// SignalView message handlers

BOOL SignalView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW), NULL);

	return TRUE;
}

void SignalView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	if(ReBuildFlg) ReBuild(dc);
	if(ReDraw){
		CBrush *Old,SelectBrsh(ColSelect), UnSelectBrsh(ColBack);
		CRect rect;
		Old = dc.SelectObject(&UnSelectBrsh);
		dc.PatBlt(0,0,10,size.cy,PATCOPY);
		dc.SelectObject(&SelectBrsh);
		dc.PatBlt(0,ChannelHight*SelectChannel,10,ChannelHight,PATCOPY);
	}


	dc.BitBlt(10,0,(size.cx - LastPointX),size.cy,&MDC,LastPointX,0,SRCCOPY);
	dc.BitBlt((size.cx - LastPointX+10),0,size.cx,size.cy,&MDC,0,0,SRCCOPY);
	CPen *OldPen, MarkerL(PS_SOLID,1,ColMark);
	OldPen = dc.SelectObject(&MarkerL);
	dc.MoveTo(Cursor.x,0); dc.LineTo(Cursor.x,size.cy);
	dc.SelectObject(OldPen);
	if(HVFlag) HV.ReDraw(HVPoint, RecordSize,HVWeight);
}

void SignalView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if((cx == 0)&(cy == 0))return;
	size.cx = 20*(int)ceil((float)(cx-10)/(float)20)+20; size.cy = cy;
	if(LoadFileFlg){
		ReBuildFlg=true;
		Invalidate(false);
	}
}

void SignalView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(nSBCode == SB_LINELEFT){
		if(LeftRecord <= 0) return;
		LeftRecord -= RightDrawing(LeftRecord,20);
		if(LeftRecord <= 0){
			LeftRecord = LastPointX = 0;
			Position = 0.0f;
			LeftDrawing(0,size.cx);
		}
		SetScrollPos(SB_HORZ,LeftRecord,true);
	}
	if(nSBCode == SB_LINERIGHT){
		if(LeftRecord >= (AllRecords-(int)floorf((float)size.cx/Resolution)))return;
		LeftRecord += LeftDrawing(LeftRecord+(int)floorf((float)size.cx/Resolution), 20);
		if(LeftRecord >= (AllRecords-(int)floorf((float)size.cx/Resolution))){
			LeftRecord =  AllRecords-(int)floorf((float)size.cx/Resolution);
			LastPointX = 0; Position = 0.0f;
			RightDrawing(AllRecords,size.cx);
		}
		SetScrollPos(SB_HORZ,LeftRecord,true);
	}
	if(nSBCode == SB_PAGELEFT){
		if(LeftRecord <= 0) return;
		LeftRecord -= RightDrawing(LeftRecord,60);
		if(LeftRecord <= 0){
			LeftRecord = LastPointX = 0;
			Position = 0.0f;
			LeftDrawing(0,size.cx);
		}
		SetScrollPos(SB_HORZ,LeftRecord,true);
	}
	if(nSBCode == SB_PAGERIGHT){
		if(LeftRecord >= (AllRecords-(int)floorf((float)size.cx/Resolution)))return;
		LeftRecord += LeftDrawing(LeftRecord+(int)floorf((float)size.cx/Resolution), 60);
		if(LeftRecord >= (AllRecords-(int)floorf((float)size.cx/Resolution))){
			LeftRecord =  AllRecords-(int)floorf((float)size.cx/Resolution);
			LastPointX = 0; Position = 0.0f;
			RightDrawing(AllRecords,size.cx);
		}
		SetScrollPos(SB_HORZ,LeftRecord,true);
	}
	Invalidate(false);
}

void SignalView::OnMouseMove(UINT nFlags, CPoint point) 
{
	
	if((!LoadFileFlg)|(point.x<=10)) return;
	Cursor = point;
	ParamView(LeftRecord+(int)floorf((float)(point.x-10)/Resolution),
			(int)floorf((float)point.y/(float)ChannelHight));
	Invalidate(false);
}

void SignalView::LoadFile(LPCTSTR lpszFileName)
{
	if(pStartData != NULL){
		::UnmapViewOfFile(pStartData);
		pStartData = NULL;
	}
	if(CRV != NULL){
		for(int c=0; c<CellCount; c++)
			if(CRV[c].SRV != NULL) delete[] CRV[c].SRV;
		delete[] CRV;
	}

	LoadFileFlg = false;

	pStartData = NULL;
	CRV = NULL;
	HANDLE hRecord = ::CreateFile(lpszFileName, GENERIC_READ, 0, NULL,
						OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hRecord == INVALID_HANDLE_VALUE)  return;

	HANDLE hMapRecord = ::CreateFileMapping(hRecord,NULL,PAGE_READONLY,0,0,NULL);
	MaxFileSize = ::GetFileSize(hRecord,NULL);
	::CloseHandle(hRecord);

	if(hMapRecord == INVALID_HANDLE_VALUE) return;
	pStartData = ::MapViewOfFile(hMapRecord,FILE_MAP_READ,0,0,0);
	::CloseHandle(hMapRecord);
	if(pStartData == NULL) return;
	((CMainFrame*)Fraim)->SetWindowText(lpszFileName);
	pDataLast = (float*)pStartData;

	LeftRecord = 0;
	AllRecords = *((int*) pDataLast); pDataLast++;
	CellCount  = *((int*) pDataLast); pDataLast++;
	CRV = new CellRecordView[CellCount];
	for(int c=0; c<CellCount; c++){
		CRV[c].N = *((int*) pDataLast); pDataLast++;
		CRV[c].Count =  *((int*) pDataLast); pDataLast++;
		CRV[c].VP = true; CRV[c].VAP = CRV[c].VTH = CRV[c].AttAP = CRV[c].AttTH = false;
		CRV[c].KP = CRV[c].KAP = CRV[c].KTH = 400;
		CRV[c].SRV = new SynpRecordView[CRV[c].Count];
		for(int s=0; s<CRV[c].Count; s++){
			CRV[c].SRV[s].VIn = CRV[c].SRV[s].VSIn = false;
			CRV[c].SRV[s].KIn = CRV[c].SRV[s].KSIn = 1e12;
		}
	}
	RecordSize = *((int*)pDataLast); pDataLast++;
	TimeStep = *pDataLast;pDataLast++;
	LastTime = 0.0f;
	pFirstRecord = pDataLast;
	ReDraw = true;
	ReBuildFlg=true;
	Resolution = (float)size.cx/(float)AllRecords;
	SetScrollRange(SB_HORZ,0,AllRecords-(int)floorf((float)size.cx/Resolution),true);	
	SetScrollPos(SB_HORZ,LeftRecord,true);
	SelectChannel = 0;
	ReDraw = true;
	Invalidate(false);
	SVC.MaxCell = CellCount;
	SVC.CRV = CRV;
	LoadFileFlg = true;
}
void SignalView::ReBuild(CDC &dc)
{
	ReBuildFlg = false;
	pBmp.DeleteObject();
	pBmp.CreateCompatibleBitmap( &dc, size.cx, size.cy);
	MDC.DeleteDC();
	MDC.CreateCompatibleDC(&dc);
	ChannelCount = 0;int HightCount = 0;
	for(int Cp=0; Cp<CellCount; Cp++){
		if(CRV[Cp].VP)ChannelCount++;
		if(CRV[Cp].VAP){
			ChannelCount++;
			if(CRV[Cp].AttAP)HightCount++;
		}
		if(CRV[Cp].VTH){
			ChannelCount++;
			if(CRV[Cp].AttTH)HightCount++;
		}
		for(int s=0; s<CRV[Cp].Count; s++){
			if(CRV[Cp].SRV[s].VIn)ChannelCount++;
			if(CRV[Cp].SRV[s].VSIn)ChannelCount++;
		}
	}
	if(Magnet&GridFlg)
		ChannelHight = (int)((float)20*floorf((float)size.cy
												/(float)((ChannelCount-HightCount)*20)));
	else
		ChannelHight = (int)floorf((float)size.cy/(float)(ChannelCount-HightCount));
	if(Min != NULL) delete[] Min;
	if(Max != NULL) delete[] Max;
	Min = new int[ChannelCount];
	Max = new int[ChannelCount];
	LastPointX=0;Position = 0.0f;
	for(int Ch=0; Ch<ChannelCount; Ch++){
		Min[Ch] = Max[Ch] = ChannelHight*Ch + ChannelHight/2;
	}
	LeftDrawing(LeftRecord,size.cx);
}

void SignalView::OnFileOpen() 
{
	CFileDialog FDlg(true,"crf","*.crf",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Cell Record File (*.crf)|*.crf|(*.*)|*.*||");
	if(FDlg.DoModal() == IDOK) LoadFile(FDlg.GetPathName());
}

int SignalView::LeftDrawing(int Record, int Points)
{
	CPen ZPen(PS_DASHDOT,1,ColGrid);
	CPen JampPen(PS_SOLID,1,RGB(255,255,255)),BPen(PS_SOLID,1,RGB(0,0,0));
	CBrush GridBrsh;
	CBitmap Grid;Grid.LoadBitmap(IDB_GRID);
	CBitmap BWGrid; BWGrid.LoadBitmap(IDB_BW_GRID);
	if(!BWCreate)
		if(GridFlg)	GridBrsh.CreatePatternBrush(&Grid); 
		else		GridBrsh.CreateSolidBrush(ColBack);
	else
		if(GridFlg)	GridBrsh.CreatePatternBrush(&BWGrid); 
		else		GridBrsh.CreateSolidBrush(RGB(255,255,255));

	MDC.SelectObject(&pBmp);
	MDC.SelectObject(&ZPen);
	MDC.SelectObject(&GridBrsh);
	if((LastPointX+Points)<size.cx){
		MDC.PatBlt(LastPointX,0,Points,size.cy,PATCOPY);
		for(int Ch=0; Ch<ChannelCount; Ch++){
			MDC.MoveTo(LastPointX,ChannelHight*Ch + ChannelHight/2); 
			MDC.LineTo(LastPointX+Points,ChannelHight*Ch + ChannelHight/2);
		}
	}
	else{
		int To;
		MDC.PatBlt(LastPointX,0,size.cx-LastPointX,size.cy,PATCOPY);
		MDC.PatBlt(0,0,To = Points - size.cx + LastPointX,size.cy,PATCOPY);
		for(int Ch=0; Ch<ChannelCount; Ch++){
			MDC.MoveTo(LastPointX,ChannelHight*Ch + ChannelHight/2); 
			MDC.LineTo(size.cx,ChannelHight*Ch + ChannelHight/2);
			MDC.MoveTo(0,ChannelHight*Ch + ChannelHight/2);
			MDC.LineTo(To,ChannelHight*Ch + ChannelHight/2);
		}
	}

	PreSet(Record);
	CPoint Current;
	CString str;
	for(int R=0, PointCount = 0; PointCount<Points; R++){
		float *Rec =(float*)(pFirstRecord+RecordSize*(Record+R));
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) break;
		if(Rec == Marker){
			CPen MarkerPen(PS_SOLID,2,ColMark);
			MDC.SelectObject(&MarkerPen);
			MDC.MoveTo(Current.x,0); MDC.LineTo(Current.x,size.cy);
		}
		if(((*Rec)-LastTime)>(2*TimeStep)){ 
			MDC.SelectObject(&JampPen);
			MDC.MoveTo(LastPointX,0);
			MDC.LineTo(LastPointX,size.cy);
			MDC.SelectObject(&BPen);
			MDC.MoveTo(LastPointX-1,0);
			MDC.LineTo(LastPointX-1,size.cy);
			MDC.MoveTo(LastPointX+1,0);
			MDC.LineTo(LastPointX+1,size.cy);
		}
		LastTime = *(Rec++);
		
		Position += Resolution;
		Current.x= (int)floorf(Position);
		bool MM = (Current.x == LastPointX);
		if(!MM){
			str.Format(" %d%% compleat", (int)((float)R*Resolution*100.0f/(float)Points));
			m_StatusBar->SetPaneText(0,str,true);
			PointCount += Current.x - LastPointX;
			if(PointCount>Points){
				str.Format("Ready");
				m_StatusBar->SetPaneText(0,str,true);
				MDC.SelectObject(&JampPen);
				MDC.MoveTo(0,0); MDC.LineTo(40,0);	MDC.MoveTo(0,2); MDC.LineTo(40,2);
				MDC.MoveTo(0,4); MDC.LineTo(40,4);	MDC.SelectObject(&BPen);
				MDC.MoveTo(0,1); MDC.LineTo(40,1);	MDC.MoveTo(0,3); MDC.LineTo(40,3);
				str.Format("%f",40.0f/Resolution*TimeStep);
				MDC.TextOut(0,10,str);
				return R;
			}
			if(Current.x>=size.cx){
				Position -=(float)size.cx;
				Current.x = LastPointX = (int)floorf(Position);
			}
		}
		OneRecordDwar(Rec,Current,MM);
		LastPointX=Current.x;
	}
	str.Format("Ready");
	m_StatusBar->SetPaneText(0,str,true);
	MDC.SelectObject(&JampPen);
	MDC.MoveTo(0,0); MDC.LineTo(40,0);
	MDC.MoveTo(0,2); MDC.LineTo(40,2);
	MDC.MoveTo(0,4); MDC.LineTo(40,4);
	MDC.SelectObject(&BPen);
	MDC.MoveTo(0,1); MDC.LineTo(40,1);
	MDC.MoveTo(0,3); MDC.LineTo(40,3);
	str.Format("%f",40.0f/Resolution*TimeStep);
	MDC.TextOut(0,10,str);
	return R;
}

int SignalView::RightDrawing(int Record, int Points)
{
	CPen ZPen(PS_DASHDOT,1,ColGrid);
	CPen JampPen(PS_SOLID,1,RGB(255,255,255)),BPen(PS_SOLID,1,RGB(0,0,0));
	CBrush GridBrsh;
	CBitmap Grid;Grid.LoadBitmap(IDB_GRID);
	CBitmap BWGrid; BWGrid.LoadBitmap(IDB_BW_GRID);
	if(!BWCreate)
		if(GridFlg)	GridBrsh.CreatePatternBrush(&Grid); 
		else		GridBrsh.CreateSolidBrush(ColBack);
	else
		if(GridFlg)	GridBrsh.CreatePatternBrush(&BWGrid); 
		else		GridBrsh.CreateSolidBrush(RGB(255,255,255));

	MDC.SelectObject(&pBmp);
	MDC.SelectObject(&ZPen);
	MDC.SelectObject(&GridBrsh);
	if((LastPointX-Points)>0){
		MDC.PatBlt(LastPointX-Points,0,Points,size.cy,PATCOPY);
		for(int Ch=0; Ch<ChannelCount; Ch++){
			MDC.MoveTo(LastPointX-Points,ChannelHight*Ch + ChannelHight/2); 
			MDC.LineTo(LastPointX,ChannelHight*Ch + ChannelHight/2);
		}
	}
	else{
		int To;
		MDC.PatBlt(0,0,LastPointX,size.cy,PATCOPY);
		MDC.PatBlt(To = size.cx + LastPointX - Points,0,Points - LastPointX,size.cy,PATCOPY);
		for(int Ch=0; Ch<ChannelCount; Ch++){
			MDC.MoveTo(LastPointX,ChannelHight*Ch + ChannelHight/2); 
			MDC.LineTo(0,ChannelHight*Ch + ChannelHight/2);
			MDC.MoveTo(size.cx,ChannelHight*Ch + ChannelHight/2);
			MDC.LineTo(To,ChannelHight*Ch + ChannelHight/2);
		}
	}

	PreSet(Record);
	CPoint Current;
	CString str;
	for(int R=0, PointCount = 0; PointCount<Points; R++){
		float *Rec =(float*)(pFirstRecord+RecordSize*(Record - R));
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) break;
		if(Rec < pFirstRecord) break;
		if(Rec == Marker){
			CPen MarkerPen(PS_SOLID,2,ColMark);
			MDC.SelectObject(&MarkerPen);
			MDC.MoveTo(Current.x,0); MDC.LineTo(Current.x,size.cy);
		}
		if((LastTime-(*Rec))>(2*TimeStep)){ 
			MDC.SelectObject(&JampPen);
			MDC.MoveTo(LastPointX,0);
			MDC.LineTo(LastPointX,size.cy);
			MDC.SelectObject(&BPen);
			MDC.MoveTo(LastPointX-1,0);
			MDC.LineTo(LastPointX-1,size.cy);
			MDC.MoveTo(LastPointX+1,0);
			MDC.LineTo(LastPointX+1,size.cy);
		}
		LastTime = *(Rec++);

		Position -= Resolution;
		Current.x= (int)floorf(Position);
		bool MM = (Current.x == LastPointX);
		if(!MM){
			str.Format(" %d%% compleat", (int)floorf((float)R*Resolution*100.0f/(float)Points));
			m_StatusBar->SetPaneText(0,str,true);
			PointCount += LastPointX - Current.x;
			if(PointCount>Points){
				str.Format("Ready");
				m_StatusBar->SetPaneText(0,str,true);
				MDC.SelectObject(&JampPen);
				MDC.MoveTo(0,0); MDC.LineTo(40,0);	MDC.MoveTo(0,2); MDC.LineTo(40,2);
				MDC.MoveTo(0,4); MDC.LineTo(40,4);	MDC.SelectObject(&BPen);
				MDC.MoveTo(0,1); MDC.LineTo(40,1);	MDC.MoveTo(0,3); MDC.LineTo(40,3);
				str.Format("%f",40.0f/Resolution*TimeStep);
				MDC.TextOut(0,10,str);
				return R;
			}
			if(Current.x < 0){
				Position +=(float)size.cx;
				Current.x = LastPointX = (int)floorf(Position);
			}
		}
		OneRecordDwar(Rec,Current,MM);
		LastPointX=Current.x;
	}
	str.Format("Ready");
	m_StatusBar->SetPaneText(0,str,true);
	MDC.SelectObject(&JampPen);
	MDC.MoveTo(0,0); MDC.LineTo(40,0);
	MDC.MoveTo(0,2); MDC.LineTo(40,2);
	MDC.MoveTo(0,4); MDC.LineTo(40,4);
	MDC.SelectObject(&BPen);
	MDC.MoveTo(0,1); MDC.LineTo(40,1);
	MDC.MoveTo(0,3); MDC.LineTo(40,3);
	str.Format("%f",40.0f/Resolution*TimeStep);
	MDC.TextOut(0,10,str);
	return R;
}

void SignalView::OneRecordDwar(float *Rec, CPoint &Current, bool MM)
{
	CPen LPen(PS_SOLID,2, ColLine);
	CPen APPen(PS_DASH,1, ColAP), THPen(PS_DOT,1, ColTH);

	int ChannelP=0, CHight = 0;
	int Ch=0;
	if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
	if(Rec < pFirstRecord) return;
	MDC.SelectObject(&LPen);

	for(int Cp=0; Cp<CellCount; Cp++){
		if(CRV[Cp].VP){
			ChannelP = CHight;
			Current.y = ChannelHight*(CHight++) + ChannelHight/2-
									(int)(*(Rec++) * CRV[Cp].KP);
			if(!MM){
				MDC.MoveTo(LastPointX,Min[Ch]);
				MDC.LineTo(Current);
				MDC.MoveTo(LastPointX,Max[Ch]);
				MDC.LineTo(Current);
				Min[Ch] = Max[Ch] = Current.y;
			}
			else{
				if(Current.y>Max[Ch])Max[Ch] = Current.y;
				if(Current.y<Min[Ch])Min[Ch] = Current.y;
			}
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;

		if(CRV[Cp].VAP){
			if(CRV[Cp].AttAP&CRV[Cp].VP){
				MDC.SelectObject(&APPen);
				Current.y = ChannelHight*ChannelP + ChannelHight/2-
					(int)(*(Rec++)*CRV[Cp].KP);
			}
			else
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++)*CRV[Cp].KAP);
			if(!MM){
				MDC.MoveTo(LastPointX,Min[Ch]);
				MDC.LineTo(Current);
				MDC.MoveTo(LastPointX,Max[Ch]);
				MDC.LineTo(Current);
				Min[Ch] = Max[Ch] = Current.y;
			}
			else{
				if(Current.y>Max[Ch])Max[Ch] = Current.y;
				if(Current.y<Min[Ch])Min[Ch] = Current.y;
			}
			MDC.SelectObject(&LPen);
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;
		if(CRV[Cp].VTH){
			if(CRV[Cp].AttTH&CRV[Cp].VP){
				MDC.SelectObject(&THPen);
				Current.y = ChannelHight*ChannelP + ChannelHight/2 - 
					(int)(*(Rec++) * CRV[Cp].KP);
			}
			else
				Current.y = ChannelHight*(CHight++) + ChannelHight/2 - 
					(int)(*(Rec++) * CRV[Cp].KTH);
			if(!MM){
				MDC.MoveTo(LastPointX,Min[Ch]);
				MDC.LineTo(Current);
				MDC.MoveTo(LastPointX,Max[Ch]);
				MDC.LineTo(Current);
				Min[Ch] = Max[Ch] = Current.y;
			}
			else{
				if(Current.y>Max[Ch])Max[Ch] = Current.y;
				if(Current.y<Min[Ch])Min[Ch] = Current.y;
			}
			MDC.SelectObject(&LPen);
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;

		for(int s=0; s<CRV[Cp].Count; s++){
			if(CRV[Cp].SRV[s].VIn){
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++) * CRV[Cp].SRV[s].KIn);
				if(!MM){
					MDC.MoveTo(LastPointX,Min[Ch]);
					MDC.LineTo(Current);
					MDC.MoveTo(LastPointX,Max[Ch]);
					MDC.LineTo(Current);
					Min[Ch] = Max[Ch] = Current.y;
				}
				else{
					if(Current.y>Max[Ch])Max[Ch] = Current.y;
					if(Current.y<Min[Ch])Min[Ch] = Current.y;
				}
				Ch++; if(Ch>ChannelCount) break;
			}
			else Rec++;
			if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
			if(Rec < pFirstRecord) return;
				if(CRV[Cp].SRV[s].VSIn){
				Current.y = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++) * CRV[Cp].SRV[s].KSIn);
				if(!MM){
					MDC.MoveTo(LastPointX,Min[Ch]);
					MDC.LineTo(Current);
					MDC.MoveTo(LastPointX,Max[Ch]);
					MDC.LineTo(Current);
					Min[Ch] = Max[Ch] = Current.y;
				}
				else{
					if(Current.y>Max[Ch])Max[Ch] = Current.y;
					if(Current.y<Min[Ch])Min[Ch] = Current.y;
				}
				Ch++; if(Ch>ChannelCount) break;
			}
			else Rec++;
			if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
			if(Rec < pFirstRecord) return;
		}
	}
}

void SignalView::PreSet(int Record)
{
	float *Rec = pFirstRecord + RecordSize * Record;

	int ChannelP=0, CHight = 0;
	int Ch=0;
	if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
	if(Rec < pFirstRecord) return;
	LastTime = *(Rec++);
	for(int Cp=0; Cp<CellCount; Cp++){
		if(CRV[Cp].VP){
			ChannelP = CHight;
			Min[Ch] = Max[Ch] = ChannelHight*(CHight++) + ChannelHight/2-
									(int)(*(Rec++) * CRV[Cp].KP);
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;

		if(CRV[Cp].VAP){
			if(CRV[Cp].AttAP&CRV[Cp].VP)
				Min[Ch] = Max[Ch] = ChannelHight*ChannelP + ChannelHight/2-
					(int)(*(Rec++)*CRV[Cp].KP);
			else
				Min[Ch] = Max[Ch] = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++)*CRV[Cp].KAP);
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;

		if(CRV[Cp].VTH){
			if(CRV[Cp].AttTH&CRV[Cp].VP){
				Min[Ch] = Max[Ch] = ChannelHight*ChannelP + ChannelHight/2 - 
					(int)(*(Rec++) * CRV[Cp].KP);
			}
			else
				Min[Ch] = Max[Ch] = ChannelHight*(CHight++) + ChannelHight/2 - 
					(int)(*(Rec++) * CRV[Cp].KTH);
			Ch++; if(Ch>ChannelCount) break;
		}
		else Rec++;
		if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
		if(Rec < pFirstRecord) return;

		for(int s=0; s<CRV[Cp].Count; s++){
			if(CRV[Cp].SRV[s].VIn){
				Min[Ch] = Max[Ch] = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++) * CRV[Cp].SRV[s].KIn);
				Ch++; if(Ch>ChannelCount) break;
			}
			else Rec++;
			if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
			if(Rec < pFirstRecord) return;
				if(CRV[Cp].SRV[s].VSIn){
				Min[Ch] = Max[Ch] = ChannelHight*(CHight++) + ChannelHight/2-
					(int)(*(Rec++) * CRV[Cp].SRV[s].KSIn);
				Ch++; if(Ch>ChannelCount) break;
			}
			else Rec++;
			if(Rec > (float*)((BYTE*)pStartData + MaxFileSize)) return;
			if(Rec < pFirstRecord) return;
		}
	}

}

void SignalView::ParamView(int Record, int ViewP)
{
	float *Rec = pFirstRecord + RecordSize * Record;
	float *Next = pFirstRecord + RecordSize * (Record + 1);
	if(Rec >= (float*)((BYTE*)pStartData + (MaxFileSize-RecordSize))) return;
	if(Rec < pFirstRecord) return;

	int Ch=0;

	float Time = *(Rec);
	CString str;

	int Wighd=0;
	for(int Cp=0; Cp<CellCount; Cp++){
		if(Cp !=0)	Wighd += CRV[Cp-1].Count*2+3;
		if(CRV[Cp].VP){
			if(Ch == ViewP){
				str.Format("Cell %d, Time %f, Potential %e",CRV[Cp].N,Time,*(HVPoint = Rec+1+Wighd));
				m_StatusBar->SetPaneText(0,str,true);
				if(HVPoint> pFirstRecord + RecordSize * AllRecords - 100)
					HVPoint = pFirstRecord + RecordSize * AllRecords - 100;
				HVWeight = (float)CRV[Cp].KP;
				if(Marker != NULL){
					str.Format("Mark: Time %f, Potential %e",*Marker,*(Marker+1+Wighd));
					m_StatusBar->SetPaneText(1,str,true);
					str.Format("D: Time %f, Potential %e",(*Rec)-(*Marker),
						(*(Rec+1+Wighd)) - (*(Marker+1+Wighd)));
					m_StatusBar->SetPaneText(2,str,true);
				}
				return;
			}
			Ch++; if(Ch>ChannelCount) break;

		}

		if(CRV[Cp].VAP & (!(CRV[Cp].AttAP&CRV[Cp].VP))){
			if(Ch == ViewP){
				str.Format("Cell %d, Time, %f, AfterPolarization %e",CRV[Cp].N,Time,*(HVPoint = Rec+2+Wighd));
				m_StatusBar->SetPaneText(0,str,true);
				if(HVPoint> pFirstRecord + RecordSize * AllRecords - 100)
					HVPoint = pFirstRecord + RecordSize * AllRecords - 100;
				HVWeight = (float)CRV[Cp].KAP;
				if(Marker != NULL){
					str.Format("Mark: Time %f, AfterPolar. %e",*Marker,*(Marker+2+Wighd));
					m_StatusBar->SetPaneText(1,str,true);
					str.Format("D: Time %f, AfterPolarization %e",(*Rec)-(*Marker),
						(*(Rec+2+Wighd)) - (*(Marker+2+Wighd)));
					m_StatusBar->SetPaneText(2,str,true);
				}
				return;
			}
			Ch++; if(Ch>ChannelCount) break;
		}

		if(CRV[Cp].VTH & (!(CRV[Cp].AttTH&CRV[Cp].VP))){
			if(Ch == ViewP){
				str.Format("Cell %d, Time, %f, Threshold %e",CRV[Cp].N,Time,*(HVPoint = Rec+3+Wighd));
				m_StatusBar->SetPaneText(0,str,true);
				if(HVPoint> pFirstRecord + RecordSize * AllRecords - 100)
					HVPoint = pFirstRecord + RecordSize * AllRecords - 100;
				HVWeight = (float)CRV[Cp].KTH;
				if(Marker != NULL){
					str.Format("Mark: Time %f, Threshold %e",*Marker,*(Marker+3+Wighd));
					m_StatusBar->SetPaneText(1,str,true);
					str.Format("D: Time %f, Threshold %e",(*Rec)-(*Marker),
						(*(Rec+3+Wighd)) - (*(Marker+3+Wighd)));
					m_StatusBar->SetPaneText(2,str,true);
				}
				return;
			}
			Ch++; if(Ch>ChannelCount) break;
		}

		for(int s=0; s<CRV[Cp].Count; s++){
			if(CRV[Cp].SRV[s].VIn){
				if(Ch == ViewP){
					str.Format("Cell %d, Time, %f, Synapse %d, Injection %e",
										CRV[Cp].N,Time,s,*(HVPoint = Rec+(s*2+4)+Wighd));
					m_StatusBar->SetPaneText(0,str,true);
					if(HVPoint> pFirstRecord + RecordSize * AllRecords - 100)
						HVPoint = pFirstRecord + RecordSize * AllRecords - 100;
					HVWeight = (float)CRV[Cp].SRV[s].KIn;
					if(Marker != NULL){
						str.Format("Mark: Time %f, Injection %e",*Marker,*(Marker+(s*2+4)+Wighd));
						m_StatusBar->SetPaneText(1,str,true);
						str.Format("D: Time %f, Injection %e",(*Rec)-(*Marker),
							(*(Rec+(s*2+4)+Wighd)) - (*(Marker+(s*2+4)+Wighd)));
						m_StatusBar->SetPaneText(2,str,true);
					}
					return;
				}
				Ch++; if(Ch>ChannelCount) break;
			}
			
			if(CRV[Cp].SRV[s].VSIn){
				if(Ch == ViewP){
					str.Format("Cell %d, Time, %f, Synapse %d, Single Injection %e",
										CRV[Cp].N,Time,s,*(HVPoint = Rec+(s*2+5)+Wighd));
					m_StatusBar->SetPaneText(0,str,true);
					if(HVPoint> pFirstRecord + RecordSize * AllRecords - 100)
						HVPoint = pFirstRecord + RecordSize * AllRecords - 100;
					HVWeight = (float)CRV[Cp].SRV[s].KSIn;
					if(Marker != NULL){
						str.Format("Mark: Time %f, Sigle Injection %e",*Marker,
							*(Marker+(s*2+5)+Wighd));
						m_StatusBar->SetPaneText(1,str,true);
						str.Format("D: Time %f, Sigle Injection %e",(*Rec)-(*Marker),
							(*(Rec+(s*2+5)+Wighd)) - (*(Marker+(s*2+5)+Wighd)));
						m_StatusBar->SetPaneText(2,str,true);
					}
					return;
				}
				Ch++; if(Ch>ChannelCount) break;
			}
		}
	}
}

void SignalView::OnPush() 
{
	Resolution /= 2;
	if((int)floorf((float)size.cx/Resolution) > AllRecords) 
		Resolution = (float)size.cx/(float)AllRecords;
	ReBuildFlg=true;	
	SetScrollRange(SB_HORZ,0,AllRecords-(int)floorf((float)size.cx/Resolution),true);	
	Invalidate(false);
}

void SignalView::OnUpdatePush(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((int)floorf((float)size.cx/Resolution) < AllRecords);	
}

void SignalView::OnPop() 
{
	Resolution *= 2;
	if(Resolution > 10.0f) Resolution = 10.0f;
	ReBuildFlg=true;	
	SetScrollRange(SB_HORZ,0,AllRecords-(int)floorf((float)size.cx/Resolution),true);	
	Invalidate(false);
}

void SignalView::OnUpdatePop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(Resolution<10.0f);	
}


void SignalView::OnSeekHome() 
{
	LeftRecord = 0;
	ReBuildFlg = true;
	SetScrollPos(SB_HORZ,LeftRecord,true);	
	Invalidate(false);
}

void SignalView::OnUpdateSeekHome(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LeftRecord>0);
}

void SignalView::OnSeekEnd() 
{
	LeftRecord = AllRecords - (int)floorf((float)size.cx/Resolution);
	ReBuildFlg = true;
	SetScrollPos(SB_HORZ,LeftRecord,true);	
	Invalidate(false);
}

void SignalView::OnUpdateSeekEnd(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LeftRecord<(AllRecords - (int)floorf((float)size.cx/Resolution)));
}

void SignalView::OnRefresh() 
{
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnCentre() 
{
	SVC.Resolution = 20.0f/Resolution * TimeStep;
	SVC.DoModal();
	Resolution = 20.0f*TimeStep / SVC.Resolution;
	SetScrollRange(SB_HORZ,0,AllRecords-(int)floorf((float)size.cx/Resolution),true);	
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateCentre(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
}


void SignalView::OnFilePrint() 
{
	CFileDialog FDlg(false,"bmp","*.bmp",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Bitmap File(*.bmp)|*.bmp|(*.*)|*.*||");
	if(FDlg.DoModal() != IDOK)return;
	BWCreate = true;
	ReBuildFlg = true;
	SelectChannel = -1;
	COLORREF tColBack = ColBack; ColBack = RGB(255,255,255);
	COLORREF tColLine = ColLine; ColLine = RGB(0,0,0);
	COLORREF tColGrid = ColGrid; ColGrid = RGB(0,0,0);
	COLORREF tColAP = ColAP; ColAP = RGB(100,100,100);
	COLORREF tColTH = ColTH; ColTH = RGB(50,50,50);
	OnPaint();
	CPen P(PS_SOLID,1,RGB(0,0,0));
	MDC.SelectObject(&P);
	MDC.MoveTo(0,0);MDC.LineTo(0,size.cy-1);
	MDC.LineTo(size.cx-1,size.cy-1);MDC.LineTo(size.cx-1,0);MDC.LineTo(0,0);
	WriteBmp(FDlg.GetPathName().GetBuffer(100), pBmp);
	ColGrid = tColGrid;
	ColLine = tColLine;
	ColBack = tColBack;
	ColAP = tColAP;
	ColTH = tColTH;
	BWCreate = false;	
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
}

void SignalView::OnBmp() 
{
	CFileDialog FDlg(false,"bmp","*.bmp",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Bitmap File(*.bmp)|*.bmp|(*.*)|*.*||");
	if(FDlg.DoModal() != IDOK)return;
	ReBuildFlg = true;
	SelectChannel = -1;
	OnPaint();
	WriteBmp(FDlg.GetPathName().GetBuffer(100), pBmp);
	CPen P(PS_SOLID,1,RGB(255,255,255));
	MDC.SelectObject(&P);
	MDC.MoveTo(0,0);MDC.LineTo(0,size.cy-1);
	MDC.LineTo(size.cx-1,size.cy-1);MDC.LineTo(size.cx-1,0);MDC.LineTo(0,0);
	WriteBmp(FDlg.GetPathName().GetBuffer(100), pBmp);
	ReBuildFlg = true;
	Invalidate(false);	
}

void SignalView::OnUpdateBmp(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
}

void SignalView::OnGridView() 
{
	GridFlg = !GridFlg;
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateGridView(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
	pCmdUI->SetCheck((int)GridFlg);
}

void SignalView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(!LoadFileFlg) return;
	switch(nFlags){
	case MK_SHIFT:
		Marker = pFirstRecord+(LeftRecord+(int)floorf((float)(point.x-10)/Resolution))*RecordSize;
		ReBuildFlg = true;
		break;
	default:
		SelectChannel = (int)floorf((float)point.y/(float)ChannelHight);
		ReDraw = true;
		break;
	}
	Invalidate(false);
}

void SignalView::OnUp() 
{
	for(int c=0, Ch=0; c<CellCount; c++){
		if(CRV[c].VP){
			if(SelectChannel == Ch){
				CRV[c].KP *=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		if(CRV[c].VAP&(!(CRV[c].VP&CRV[c].AttAP))){
			if(SelectChannel == Ch){
				CRV[c].KAP *=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		if(CRV[c].VTH&(!(CRV[c].VP&CRV[c].AttTH))){
			if(SelectChannel == Ch){
				CRV[c].KTH *=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		for(int s=0; s<CRV[c].Count; s++){
			if(CRV[c].SRV[s].VIn){
				if(SelectChannel == Ch){
					CRV[c].SRV[s].KIn *=2;
					ReBuildFlg = true;
					Invalidate(false);
					return;
				}
				Ch++; if(Ch>ChannelCount) return;
			}
			if(CRV[c].SRV[s].VSIn){
				if(SelectChannel == Ch){
					CRV[c].SRV[s].KSIn *=2;
					ReBuildFlg = true;
					Invalidate(false);
					return;
				}
				Ch++; if(Ch>ChannelCount) return;
			}
		}
	}
}

void SignalView::OnUpdateUp(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&(SelectChannel>=0));	
}

void SignalView::OnDown() 
{
	for(int c=0, Ch=0; c<CellCount; c++){
		if(CRV[c].VP){
			if(SelectChannel == Ch){
				CRV[c].KP /=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		if(CRV[c].VAP&(!(CRV[c].VP&CRV[c].AttAP))){
			if(SelectChannel == Ch){
				CRV[c].KAP /=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		if(CRV[c].VTH&(!(CRV[c].VP&CRV[c].AttTH))){
			if(SelectChannel == Ch){
				CRV[c].KTH /=2;
				ReBuildFlg = true;
				Invalidate(false);
				return;
			}
			Ch++; if(Ch>ChannelCount) return;
		}
		for(int s=0; s<CRV[c].Count; s++){
			if(CRV[c].SRV[s].VIn){
				if(SelectChannel == Ch){
					CRV[c].SRV[s].KIn /=2;
					ReBuildFlg = true;
					Invalidate(false);
					return;
				}
				Ch++; if(Ch>ChannelCount) return;
			}
			if(CRV[c].SRV[s].VSIn){
				if(SelectChannel == Ch){
					CRV[c].SRV[s].KSIn /=2;
					ReBuildFlg = true;
					Invalidate(false);
					return;
				}
				Ch++; if(Ch>ChannelCount) return;
			}
		}
	}
}

void SignalView::OnUpdateDown(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&(SelectChannel>=0));		
}

void SignalView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
/*	Marker = pFirstRecord+(LeftRecord+(int)floorf((float)point.x/Resolution))*RecordSize;
	ReBuildFlg = true;
	Invalidate(false);
*/}

void SignalView::OnMarker() 
{
	if(Marker != NULL) Marker = NULL;	
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateMarker(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(Marker !=NULL);
}

void SignalView::OnMargnet() 
{
	Magnet = !Magnet;
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateMargnet(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&GridFlg);
	pCmdUI->SetCheck((int)Magnet);
}

void SignalView::OnSeekMark() 
{
	LeftRecord = (int)(Marker - pFirstRecord);
	LeftRecord /= RecordSize;
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateSeekMark(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&(Marker != NULL));
}

void SignalView::OnHome() 
{	OnSeekHome();	}
void SignalView::OnEnd() 
{	OnSeekEnd();	}


void SignalView::OnLineDown() 
{
	SelectChannel++;
	ParamView(LeftRecord+(int)floorf((float)(Cursor.x-10)/Resolution),
			SelectChannel);
	Invalidate(false);
}

void SignalView::OnLineUp() 
{
	SelectChannel--;
	if(SelectChannel<0)SelectChannel=0;
	ParamView(LeftRecord+(int)floorf((float)(Cursor.x-10)/Resolution),
			SelectChannel);
	Invalidate(false);
}

void SignalView::OnLineLeft() 
{
	Cursor.x --;
	if(Cursor.x <= 10){
		Cursor.x = 10;
		OnHScroll(SB_LINELEFT,0,NULL);
	}
	ParamView(LeftRecord+(int)floorf((float)(Cursor.x-10)/Resolution),
			SelectChannel);
	Invalidate(false);
	ReDraw = true;
	Invalidate(false);
}

void SignalView::OnLineRight() 
{
	Cursor.x ++;
	if(Cursor.x >= size.cx){
		Cursor.x = size.cx;
		OnHScroll(SB_LINERIGHT,0,NULL);
	}
	ParamView(LeftRecord+(int)floorf((float)(Cursor.x-10)/Resolution),
			SelectChannel);
	Invalidate(false);
}

void SignalView::OnPadgeUp() 
{
	OnHScroll(SB_PAGELEFT,0,NULL);
	Invalidate(false);
}

void SignalView::OnPageDown() 
{
	OnHScroll(SB_PAGERIGHT,0,NULL);
	Invalidate(false);
}

void SignalView::OnSeekTime() 
{
	SetTimePoint STP;
	if(STP.DoModal() != IDOK) return;
	if(STP.m_Time<0)return;
	int Record = 0;
	while((Record < AllRecords)&
							(*(pFirstRecord+Record*RecordSize)<STP.m_Time))
		Record++;
	LeftRecord = Record;
	ReBuildFlg = true;
	Invalidate(false);
}

void SignalView::OnUpdateSeekTime(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&GridFlg);
}

void SignalView::OnUpdateRefresh(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg&GridFlg);
	
}

void SignalView::OnWav() 
{
	WavCreator WC;
	WC.AllRecords = AllRecords;
	WC.CRV = CRV;
	WC.MaxCell = SVC.MaxCell;
	WC.pFirstRecord = pFirstRecord;
	WC.RecordSize = RecordSize;
	WC.Frequency = (int)(1.0f/TimeStep);
	WC.DoModal();
}

void SignalView::OnUpdateWav(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
}

void SignalView::OnHighview() 
{
	if(HVFlag){
		HVFlag = ! HV.DestroyWindow( );
	}
	else{
		HVFlag = HV.Create(NULL, "tyu", WS_OVERLAPPED|WS_BORDER|WS_CLIPCHILDREN,
				CRect(10,0,110,100),this,0,NULL);
		HV.ShowWindow(SW_SHOW);
	}
}

void SignalView::OnUpdateHighview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(LoadFileFlg);
	pCmdUI->SetCheck((int)HVFlag);
}
