// WavCreator.cpp : implementation file
//

#include "stdafx.h"
#include "Poly Channel Viewer.h"
#include "WavCreator.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// WavCreator dialog


WavCreator::WavCreator(CWnd* pParent /*=NULL*/)
	: CDialog(WavCreator::IDD, pParent)
{
	//{{AFX_DATA_INIT(WavCreator)
	m_FileName = _T("");
	m_Mono = FALSE;
	//}}AFX_DATA_INIT
	CRV = NULL;
	MaxCell = RecordSize = AllRecords = Frequency = 0;
	pFirstRecord = NULL;
	LeftIndex = RightIndex = NULL;
}


void WavCreator::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(WavCreator)
	DDX_Control(pDX, IDC_CREATOR, m_CreatorBar);
	DDX_Control(pDX, IDC_RIGHT_CHANNEL, m_RightList);
	DDX_Control(pDX, IDC_LEFT_CHANNEL, m_LeftList);
	DDX_Text(pDX, IDC_FILENAME_EDIT, m_FileName);
	DDX_Check(pDX, IDC_MONO, m_Mono);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(WavCreator, CDialog)
	//{{AFX_MSG_MAP(WavCreator)
	ON_BN_CLICKED(IDC_MONO, OnMono)
	ON_BN_CLICKED(IDC_FILENAME_BUT, OnFilenameBut)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// WavCreator message handlers

BOOL WavCreator::OnInitDialog() 
{
	CDialog::OnInitDialog();
	if(CRV == NULL) return FALSE;
	if((MaxCell == 0)|(RecordSize == 0)|(AllRecords == 0)|(Frequency == 0))return FALSE;
	if(pFirstRecord == NULL) return FALSE;
	
	LeftIndex = new int[MaxCell]; RightIndex = new int[MaxCell];
	for(int cl=0; cl<MaxCell; cl++){
		CString str; str.Format("Cell %d", CRV[cl].N);
		LeftIndex[cl] = m_LeftList.AddString(str);
		RightIndex[cl] = m_RightList.AddString(str);
	}
	m_CreatorBar.SetRange(0, AllRecords*2);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void WavCreator::OnMono() 
{
	UpdateData(true);
	if(m_Mono)	m_RightList.ShowWindow(SW_HIDE);
	else		m_RightList.ShowWindow(SW_SHOW);
}

void WavCreator::OnOK() 
{
	UpdateData(true);
	CFile File;
	if(File.Open(m_FileName, CFile::modeCreate|CFile::modeWrite, NULL) == 0){
		if(LeftIndex != NULL) delete[] LeftIndex;
		if(RightIndex!= NULL) delete[] RightIndex;
		CDialog::OnOK();
		return;
	}

	if(m_Mono)	MonoC(File);
	else		StereoC(File);

	File.Close();

	if(LeftIndex != NULL) delete[] LeftIndex;
	if(RightIndex!= NULL) delete[] RightIndex;
	CDialog::OnOK();
}

void WavCreator::OnCancel() 
{
	if(LeftIndex != NULL) delete[] LeftIndex;
	if(RightIndex!= NULL) delete[] RightIndex;
	CDialog::OnCancel();
}

void WavCreator::OnFilenameBut() 
{
	CFileDialog FDlg(false,"wav","*.wav",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"SoundFile(*.wav)|*.wav|(*.*)|*.*||");
	if(FDlg.DoModal() == IDOK){
		m_FileName = FDlg.GetPathName();
		UpdateData(false);
	}
}

void WavCreator::MonoC(CFile &File)
{
	int CellCnt = 0;
	CellCnt = m_LeftList.GetSelCount();
	if(CellCnt == LB_ERR) return;
	int *CellOffset = new int[CellCnt];
	m_LeftList.GetSelItems(CellCnt, CellOffset);
	int Offset = 1; CellCnt = 0;
	for(int cl=0; cl<MaxCell; cl++){
		if(CellOffset[CellCnt] == cl)
			CellOffset[CellCnt++] = Offset;
		Offset+= 3+2*CRV[cl].Count;
	}
//Wave hader
	char RIFF[] = "RIFF", WAVE[] = "WAVE", FMT_[] = "fmt ", DATA[] = "data";
	short Channels = 1;
	short SamplSize = Channels*2;
	int Long =(int) Channels*2*AllRecords+0x2c-8;//???
	int ChunkSize = 16;
	int BytePerSecond = Channels*Frequency*2;
	short FormatTag = 1;
	short BitSamplSize = 16;
	int DataLong = Channels*2*AllRecords;//???

	File.Write(&RIFF, 4); File.Write(&Long, 4);
	File.Write(&WAVE, 4); File.Write(&FMT_, 4);
	File.Write(&ChunkSize, 4);
	File.Write(&FormatTag, 2);
	File.Write(&Channels,2);
	File.Write(&Frequency,4);
	File.Write(&BytePerSecond,4);
	File.Write(&SamplSize,2);
	File.Write(&BitSamplSize,2);
	File.Write(&DATA, 4);
	File.Write(&DataLong, 4);
	float Min = 0.0f, Max = 0.0f, Norm = 0.0f;
	for(int RecCnt=0; RecCnt<AllRecords; RecCnt++){
		float Sum = 0.0f;
		for(int cn=0; cn<CellCnt; cn++)
			Sum += *(pFirstRecord+RecordSize*RecCnt+CellOffset[cn]);
//		Su;
		if(Sum<Min) Min = Sum;
		if(Sum>Max) Max = Sum;
		m_CreatorBar.SetPos(RecCnt);
	}	
	if(fabs(Min)>fabs(Max)) Norm = (float)((double)0x7fff/(double)fabs(Min));
	else					Norm = (float)((double)0x7fff/(double)fabs(Max));

	for(RecCnt=0; RecCnt<AllRecords; RecCnt++){
		float Sum = 0.0f;
		for(int cn=0; cn<CellCnt; cn++)
			Sum += *(pFirstRecord+RecordSize*RecCnt+CellOffset[cn]);
		short Rec = (short)(Sum*Norm);
//		short Rec = (short)(Sum*5e5);
		File.Write(&Rec, 2);
		m_CreatorBar.SetPos(RecCnt+AllRecords);
	}	
	delete [] CellOffset;
}

void WavCreator::StereoC(CFile &File)
{
	int LCellCnt = 0, RCellCnt = 0;
	LCellCnt = m_LeftList.GetSelCount();
	RCellCnt = m_RightList.GetSelCount();
	if(LCellCnt == LB_ERR) return;
	if(RCellCnt == LB_ERR) return;
	int *LCellOffset = new int[LCellCnt];
	int *RCellOffset = new int[RCellCnt];
	m_LeftList.GetSelItems(LCellCnt, LCellOffset);
	m_RightList.GetSelItems(RCellCnt, RCellOffset);
	int LOffset = 1; LCellCnt = 0;
	int ROffset = 1; RCellCnt = 0;
	for(int cl=0; cl<MaxCell; cl++){
		if(LCellOffset[RCellCnt] == cl)
			LCellOffset[LCellCnt++] = LOffset;
		LOffset+= 3+2*CRV[cl].Count;
		if(RCellOffset[RCellCnt] == cl)
			RCellOffset[RCellCnt++] = ROffset;
		ROffset+= 3+2*CRV[cl].Count;
	}

//Wave hader
	char RIFF[] = "RIFF", WAVE[] = "WAVE", FMT_[] = "fmt ", DATA[] = "data";
	short Channels = 2;
	short SamplSize = Channels*2;
	int Long =(int) Channels*2*AllRecords+0x2c-8;//???
	int ChunkSize = 16;
	int BytePerSecond = Channels*Frequency*2;
	short FormatTag = 1;
	short BitSamplSize = 16;
	int DataLong = Channels*2*AllRecords;//???

	File.Write(&RIFF, 4); File.Write(&Long, 4);
	File.Write(&WAVE, 4); File.Write(&FMT_, 4);
	File.Write(&ChunkSize, 4);
	File.Write(&FormatTag, 2);
	File.Write(&Channels,2);
	File.Write(&Frequency,4);
	File.Write(&BytePerSecond,4);
	File.Write(&SamplSize,2);
	File.Write(&BitSamplSize,2);
	File.Write(&DATA, 4);
	File.Write(&DataLong, 4);

	float LMin = 0.0f, LMax = 0.0f, LNorm = 0.0f;
	float RMin = 0.0f, RMax = 0.0f, RNorm = 0.0f;
	for(int RecCnt=0; RecCnt<AllRecords; RecCnt++){
		float LSum = 0.0f, RSum = 0.0f;
		for(int lcn=0; lcn<LCellCnt; lcn++)
			LSum += *(pFirstRecord+RecordSize*RecCnt+LCellOffset[lcn]);
		if(LSum<LMin) LMin = LSum;
		if(LSum>LMax) LMax = LSum;

		for(int rcn=0; rcn<RCellCnt; rcn++)
			RSum += *(pFirstRecord+RecordSize*RecCnt+RCellOffset[rcn]);
		if(RSum<RMin) RMin = RSum;
		if(RSum>RMax) RMax = RSum;

		m_CreatorBar.SetPos(RecCnt);
	}	
	if(fabs(LMin)>fabs(LMax))	LNorm = (float)((double)0x7fff/(double)fabs(LMin));
	else						LNorm = (float)((double)0x7fff/(double)fabs(LMax));
	if(fabs(RMin)>fabs(RMax))	RNorm = (float)((double)0x7fff/(double)fabs(RMin));
	else						RNorm = (float)((double)0x7fff/(double)fabs(RMax));

	for(RecCnt=0; RecCnt<AllRecords; RecCnt++){
		float LSum = 0.0f, RSum = 0.0f;
		for(int lcn=0; lcn<LCellCnt; lcn++)
			LSum += *(pFirstRecord+RecordSize*RecCnt+LCellOffset[lcn]);
		short Rec = (short)(LSum*LNorm);
		File.Write(&Rec, 2);

		for(int rcn=0; rcn<RCellCnt; rcn++)
			RSum += *(pFirstRecord+RecordSize*RecCnt+RCellOffset[rcn]);
		Rec = (short)(RSum*RNorm);
		File.Write(&Rec, 2);
		m_CreatorBar.SetPos(RecCnt+AllRecords);
	}	
	delete [] LCellOffset;
	delete [] RCellOffset;
}
